searchData={"items":[{"type":"module","title":"Flop","doc":"Flop is a helper library for filtering, ordering and pagination with Ecto.","ref":"Flop.html"},{"type":"module","title":"Usage - Flop","doc":"The simplest way of using this library is just to use\n`Flop.validate_and_run/3` and `Flop.validate_and_run!/3`. Both functions\ntake a queryable and a parameter map, validate the parameters, run the query\nand return the query results and the meta information.\n\n    iex> Flop.Repo.insert_all(MyApp.Pet, [\n    ...>   %{name: \"Harry\", age: 4, species: \"C. lupus\"},\n    ...>   %{name: \"Maggie\", age: 1, species: \"O. cuniculus\"},\n    ...>   %{name: \"Patty\", age: 2, species: \"C. aegagrus\"}\n    ...> ])\n    iex> params = %{order_by: [\"name\", \"age\"], page: 1, page_size: 2}\n    iex> {:ok, {results, meta}} =\n    ...>   Flop.validate_and_run(\n    ...>     MyApp.Pet,\n    ...>     params,\n    ...>     repo: Flop.Repo\n    ...>   )\n    iex> Enum.map(results, & &1.name)\n    [\"Harry\", \"Maggie\"]\n    iex> meta.total_count\n    3\n    iex> meta.total_pages\n    2\n    iex> meta.has_next_page?\n    true\n\nUnder the hood, these functions just call `Flop.validate/2` and `Flop.run/3`,\nwhich in turn calls `Flop.all/3` and `Flop.meta/3`. If you need finer control\nabout if and when to execute each step, you can call those functions directly.\n\nSee `Flop.Meta` for descriptions of the meta fields.","ref":"Flop.html#module-usage"},{"type":"module","title":"Global configuration - Flop","doc":"You can set some global options like the default Ecto repo via the application\nenvironment. All global options can be overridden by passing them directly to\nthe functions or configuring the options for a schema module via\n`Flop.Schema`.\n\n    import Config\n\n    config :flop, repo: MyApp.Repo\n\nSee `t:Flop.option/0` for a description of all available options.","ref":"Flop.html#module-global-configuration"},{"type":"module","title":"Config modules - Flop","doc":"Instead of setting global options in the application environment, you can also\ncreate a Flop config module. This is especially useful in an umbrella\napplication, or if you have multiple Repos.\n\n```elixir\ndefmodule MyApp.Flop do\n  use Flop, repo: MyApp.Repo, default_limit: 25\nend\n```\n\n> #### `use Flop` {: .info}\n>\n> When you `use Flop`, the Flop module will define wrapper functions around\n> all of the `Flop` functions that take a query, the Flop parameters, and\n> options as arguments. The options passed to `use Flop` will be used as\n> default options in all the wrapper functions, but you can still override\n> them.\n\nThe wrapped functions are:\n\n- `Flop.all/3`\n- `Flop.count/3`\n- `Flop.filter/3`\n- `Flop.meta/3`\n- `Flop.order_by/3`\n- `Flop.paginate/3`\n- `Flop.query/3`\n- `Flop.run/3`\n- `Flop.validate_and_run/3`\n- `Flop.validate_and_run!/3`\n\nSo instead of using `Flop.validate_and_run/3`, you would call\n`MyApp.Flop.validate_and_run/3`.\n\nIf you have both a config module and a global application config, Flop will\nfall back to the application config if an option is not set.\n\nSee `t:Flop.option/0` for a description of all available options.","ref":"Flop.html#module-config-modules"},{"type":"module","title":"Schema options - Flop","doc":"You can set some options for a schema by deriving `Flop.Schema`. The options\nare evaluated at the validation step.\n\n    defmodule Pet do\n      use Ecto.Schema\n\n      @derive {Flop.Schema,\n               filterable: [:name, :species],\n               sortable: [:name, :age],\n               default_limit: 20,\n               max_limit: 100}\n\n      schema \"pets\" do\n        field :name, :string\n        field :age, :integer\n        field :species, :string\n        field :social_security_number, :string\n      end\n    end\n\nYou need to pass the schema to `Flop.validate/2` or any function that\nincludes the validation step with the `:for` option.\n\n    iex> params = %{\"order_by\" => [\"name\", \"age\"], \"limit\" => 5}\n    iex> {:ok, flop} = Flop.validate(params, for: MyApp.Pet)\n    iex> flop.limit\n    5\n\n    iex> params = %{\"order_by\" => [\"name\", \"age\"], \"limit\" => 10_000}\n    iex> {:error, meta} = Flop.validate(params, for: MyApp.Pet)\n    iex> [limit: [{msg, _}]] = meta.errors\n    iex> msg\n    \"must be less than or equal to %{number}\"\n\n    iex> params = %{\"order_by\" => [\"name\", \"age\"], \"limit\" => 10_000}\n    iex> {:error, %Flop.Meta{} = meta} =\n    ...>   Flop.validate_and_run(\n    ...>     MyApp.Pet,\n    ...>     params,\n    ...>     for: MyApp.Pet\n    ...>   )\n    iex> [limit: [{msg, _}]] = meta.errors\n    iex> msg\n    \"must be less than or equal to %{number}\"","ref":"Flop.html#module-schema-options"},{"type":"module","title":"Ordering - Flop","doc":"To add an ordering clause to a query, you need to set the `:order_by` and\noptionally the `:order_directions` parameter. `:order_by` should be the list\nof fields, while `:order_directions` is a list of `t:Flop.order_direction/0`.\n`:order_by` and `:order_directions` are zipped when generating the `ORDER BY`\nclause. If no order directions are given, `:asc` is used as default.\n\n    iex> params = %{\n    ...>   \"order_by\" => [\"name\", \"age\"],\n    ...>   \"order_directions\" => [\"asc\", \"desc\"]\n    ...> }\n    iex> {:ok, flop} = Flop.validate(params)\n    iex> flop.order_by\n    [:name, :age]\n    iex> flop.order_directions\n    [:asc, :desc]\n\nFlop uses these two fields instead of a keyword list, so that the order\ninstructions can be easily passed in a query string.","ref":"Flop.html#module-ordering"},{"type":"module","title":"Pagination - Flop","doc":"For queries using `OFFSET` and `LIMIT`, you have the choice between\npage-based pagination parameters:\n\n    %{page: 5, page_size: 20}\n\nand offset-based pagination parameters:\n\n    %{offset: 100, limit: 20}\n\nFor cursor-based pagination, you can either use `:first`/`:after` or\n`:last`/`:before`. You also need to pass the `:order_by` parameter or set a\ndefault order for the schema via `Flop.Schema`.\n\n    iex> Flop.Repo.insert_all(MyApp.Pet, [\n    ...>   %{name: \"Harry\", age: 4, species: \"C. lupus\"},\n    ...>   %{name: \"Maggie\", age: 1, species: \"O. cuniculus\"},\n    ...>   %{name: \"Patty\", age: 2, species: \"C. aegagrus\"}\n    ...> ])\n    iex>\n    iex> # forward (first/after)\n    iex>\n    iex> params = %{first: 2, order_by: [:species, :name]}\n    iex> {:ok, {results, meta}} = Flop.validate_and_run(MyApp.Pet, params)\n    iex> Enum.map(results, & &1.name)\n    [\"Patty\", \"Harry\"]\n    iex> meta.has_next_page?\n    true\n    iex> end_cursor = meta.end_cursor\n    \"g3QAAAACdwRuYW1lbQAAAAVIYXJyeXcHc3BlY2llc20AAAAIQy4gbHVwdXM=\"\n    iex> params = %{first: 2, after: end_cursor, order_by: [:species, :name]}\n    iex> {:ok, {results, meta}} = Flop.validate_and_run(MyApp.Pet, params)\n    iex> Enum.map(results, & &1.name)\n    [\"Maggie\"]\n    iex> meta.has_next_page?\n    false\n    iex>\n    iex> # backward (last/before)\n    iex>\n    iex> params = %{last: 2, order_by: [:species, :name]}\n    iex> {:ok, {results, meta}} = Flop.validate_and_run(MyApp.Pet, params)\n    iex> Enum.map(results, & &1.name)\n    [\"Harry\", \"Maggie\"]\n    iex> meta.has_previous_page?\n    true\n    iex> start_cursor = meta.start_cursor\n    \"g3QAAAACdwRuYW1lbQAAAAVIYXJyeXcHc3BlY2llc20AAAAIQy4gbHVwdXM=\"\n    iex> params = %{last: 2, before: start_cursor, order_by: [:species, :name]}\n    iex> {:ok, {results, meta}} = Flop.validate_and_run(MyApp.Pet, params)\n    iex> Enum.map(results, & &1.name)\n    [\"Patty\"]\n    iex> meta.has_previous_page?\n    false\n\nBy default, it is assumed that the query result is a list of maps or structs.\nIf your query returns a different data structure, you can pass the\n`:cursor_value_func` option to retrieve the cursor values. See\n`t:Flop.option/0` and `Flop.Cursor` for more information.\n\nYou can restrict which pagination types are available. See `t:Flop.option/0`\nfor details.","ref":"Flop.html#module-pagination"},{"type":"module","title":"Filters - Flop","doc":"Filters can be passed as a list of maps. It is recommended to define the\nfilterable fields for a schema using `Flop.Schema`.\n\n    iex> Flop.Repo.insert_all(MyApp.Pet, [\n    ...>   %{name: \"Harry\", age: 4, species: \"C. lupus\"},\n    ...>   %{name: \"Maggie\", age: 1, species: \"O. cuniculus\"},\n    ...>   %{name: \"Patty\", age: 2, species: \"C. aegagrus\"}\n    ...> ])\n    iex>\n    iex> params = %{filters: [%{field: :name, op: :=~, value: \"Mag\"}]}\n    iex> {:ok, {results, meta}} = Flop.validate_and_run(MyApp.Pet, params)\n    iex> meta.total_count\n    1\n    iex> [pet] = results\n    iex> pet.name\n    \"Maggie\"\n\nSee `t:Flop.Filter.op/0` for a list of all available filter operators.","ref":"Flop.html#module-filters"},{"type":"module","title":"GraphQL and Relay - Flop","doc":"The parameters used for cursor-based pagination follow the Relay\nspecification, so you can just pass the arguments you get from the client on\nto Flop.\n\n`Flop.Relay` can convert the query results returned by\n`Flop.validate_and_run/3` into `Edges` and `PageInfo` formats required for\nRelay connections.\n\nFor example, if you have a context module like this:\n\n    defmodule MyApp.Flora\n      import Ecto.query, warn: false\n\n      alias MyApp.Flora.Plant\n\n      def list_plants_by_continent(%Continent{} = continent, %{} = args) do\n        Plant\n        |> where(continent_id: ^continent.id)\n        |> Flop.validate_and_run(args, for: Plant)\n      end\n    end\n\nThen your Absinthe resolver for the `plants` connection may look something\nlike this:\n\n    def list_plants(args, %{source: %Continent{} = continent}) do\n      with {:ok, result} <-\n             Flora.list_plants_by_continent(continent, args) do\n        {:ok, Flop.Relay.connection_from_result(result)}\n      end\n    end","ref":"Flop.html#module-graphql-and-relay"},{"type":"function","title":"Flop.__struct__/0","doc":"Options specific to the Ecto adapter.\n\n- `:repo` - The Ecto Repo module used for database queries.\n- `:query_opts` - Options passed to the `Ecto.Repo` query functions. Refer to\n  the Ecto documentation for `c:Ecto.Repo.all/2`, `c:Ecto.Repo.aggregate/3`,\n  and the [\"Shared Options\"](https://hexdocs.pm/ecto/Ecto.Repo.html#module-shared-options)\n  section of `Ecto.Repo`.","ref":"Flop.html#__struct__/0"},{"type":"function","title":"Flop.aliases/2","doc":"Returns the names of the alias fields that are required for the order clause\nof the given Flop.\n\nThe second argument is the schema module that derives `Flop.Schema`.\n\nFor example, your schema module might define an alias field called\n`:pet_count`.\n\n    @derive {\n      Flop.Schema,\n      filterable: [],\n      sortable: [:name, :pet_count],\n      adapter_opts: [\n        alias_fields: [:pet_count]\n      ]\n    }\n\nIf you pass a Flop that orders by the `:pet_count` field, the returned list\nwill include the `:pet_count` alias.\n\n    iex> aliases(%Flop{order_by: [:name, :pet_count]}, MyApp.Owner)\n    [:pet_count]\n\nIf on the other hand only normal fields are used in the `order_by` parameter,\nan empty list will be returned.\n\n    iex> aliases(%Flop{order_by: [:name]}, MyApp.Owner)\n    []\n\nYou can use this to dynamically build the select clause needed for the query.\n\nFor more information about alias fields, refer to the module documentation of\n`Flop.Schema`.","ref":"Flop.html#aliases/2"},{"type":"function","title":"Flop.all/3","doc":"Applies the given Flop to the given queryable and returns all matchings\nentries.\n\n    iex> Flop.all(MyApp.Pet, %Flop{}, repo: Flop.Repo)\n    []\n\nYou can also configure a default repo in your config files:\n\n    config :flop, repo: MyApp.Repo\n\nThis allows you to omit the third argument:\n\n    iex> Flop.all(MyApp.Pet, %Flop{}, for: MyApp.Pet)\n    []\n\nNote that when using cursor-based pagination, the applied limit will be\n`first + 1` or `last + 1`. The extra record is removed by `Flop.run/3`, but\nnot by this function.\n\nAlso note that you will need to pass the `for` option in order for Flop to be\nable to find your join, compound, alias and custom field configuration.\n\nThis function does _not_ validate or apply default parameters to the given\nFlop struct. Be sure to validate any user-generated parameters with\n`validate/2` or `validate!/2` before passing them to this function.","ref":"Flop.html#all/3"},{"type":"function","title":"Flop.count/3","doc":"Returns the total count of entries matching the filter conditions of the\nFlop.\n\nThe pagination and ordering option are disregarded.\n\n    iex> Flop.count(MyApp.Pet, %Flop{}, repo: Flop.Repo)\n    0\n\nYou can also configure a default repo in your config files:\n\n    config :flop, repo: MyApp.Repo\n\nThis allows you to omit the third argument:\n\n    iex> Flop.count(MyApp.Pet, %Flop{})\n    0\n\nYou can override the default query by passing the `:count_query` option. This\ndoesn't make a lot of sense when you use `count/3` directly, but allows you to\noptimize the count query when you use one of the `run/3`,\n`validate_and_run/3` and `validate_and_run!/3` functions.\n\n    query = join(Pet, :left, [p], o in assoc(p, :owner))\n    count_query = Pet\n    count(query, %Flop{}, count_query: count_query, for: Pet)\n\nThe filter parameters of the given Flop are applied to the custom count query.\n\nIf for some reason you already have the count, you can pass it as the `:count`\noption.\n\n    count(query, %Flop{}, count: 42, for: Pet)\n\nIf you pass both the `:count` and the `:count_query` options, the `:count`\noption will take precedence.\n\nThis function does _not_ validate or apply default parameters to the given\nFlop struct. Be sure to validate any user-generated parameters with\n`validate/2` or `validate!/2` before passing them to this function.\n\nNote that you will need to pass the `for` option in order for Flop to be\nable to find your join, compound, alias and custom field configuration.","ref":"Flop.html#count/3"},{"type":"function","title":"Flop.current_order/2","doc":"Returns the current order direction for the given field.","ref":"Flop.html#current_order/2"},{"type":"function","title":"Examples - Flop.current_order/2","doc":"iex> flop = %Flop{order_by: [:name, :age], order_directions: [:desc]}\n    iex> current_order(flop, :name)\n    :desc\n    iex> current_order(flop, :age)\n    :asc\n    iex> current_order(flop, :species)\n    nil","ref":"Flop.html#current_order/2-examples"},{"type":"function","title":"Flop.filter/3","doc":"Applies the `filter` parameter of a `t:Flop.t/0` to an `t:Ecto.Queryable.t/0`.\n\nUsed by `Flop.query/2`.\n\nThis function does _not_ validate or apply default parameters to the given\nFlop struct. Be sure to validate any user-generated parameters with\n`validate/2` or `validate!/2` before passing them to this function.\n\nNote that you will need to pass the `for` option in order for Flop to be\nable to find your join, compound, alias and custom field configuration.","ref":"Flop.html#filter/3"},{"type":"function","title":"Flop.get_option/3","doc":"Returns the option with the given key.\n\nThe look-up order is:\n\n1. the keyword list passed as the second argument\n2. the schema module that derives `Flop.Schema`, if the passed list includes\n   the `:for` option\n3. the backend module with `use Flop`\n4. the application environment\n5. the default passed as the last argument","ref":"Flop.html#get_option/3"},{"type":"function","title":"Flop.map_to_filter_params/2","doc":"Converts a map of filter conditions into a list of Flop filter params.\n\nThe default operator is `:==`. `nil` values are excluded from the result.\n\n    iex> params = map_to_filter_params(\n    ...>   %{name: \"George\", age: 8, species: nil}\n    ...> )\n    iex> Enum.sort(params)\n    [\n      %{field: :age, op: :==, value: 8},\n      %{field: :name, op: :==, value: \"George\"}\n    ]\n\n    iex> params = map_to_filter_params(\n    ...>   %{\"name\" => \"George\", \"age\" => 8, \"cat\" => true}\n    ...>  )\n    iex> Enum.sort(params)\n    [\n      %{\"field\" => \"age\", \"op\" => :==, \"value\" => 8},\n      %{\"field\" => \"cat\", \"op\" => :==, \"value\" => true},\n      %{\"field\" => \"name\", \"op\" => :==, \"value\" => \"George\"}\n    ]\n\nYou can optionally pass a mapping from field names to operators as a map\nwith atom keys.\n\n    iex> params = map_to_filter_params(\n    ...>   %{name: \"George\", age: 8, species: nil},\n    ...>   operators: %{name: :ilike_and}\n    ...> )\n    iex> Enum.sort(params)\n    [\n      %{field: :age, op: :==, value: 8},\n      %{field: :name, op: :ilike_and, value: \"George\"}\n    ]\n\n    iex> map_to_filter_params(\n    ...>   %{\"name\" => \"George\", \"age\" => 8, \"cat\" => true},\n    ...>   operators: %{name: :ilike_and, age: :  )\n    [\n      %{\"field\" => \"age\", \"op\" => :  params = map_to_filter_params(\n    ...>   %{s: \"George\", age: 8, species: nil},\n    ...>   rename: %{s: :name}\n    ...> )\n    iex> Enum.sort(params)\n    [\n      %{field: :age, op: :==, value: 8},\n      %{field: :name, op: :==, value: \"George\"}\n    ]\n\n    iex> map_to_filter_params(\n    ...>   %{\"s\" => \"George\", \"cat\" => true},\n    ...>   rename: %{s: :name, cat: :dog}\n    ...> )\n    [\n      %{\"field\" => \"dog\", \"op\" => :==, \"value\" => true},\n      %{\"field\" => \"name\", \"op\" => :==, \"value\" => \"George\"}\n    ]\n\nIf both a rename option and an operator are set for a field, the operator\noption needs to use the new field name.\n\n    iex> map_to_filter_params(\n    ...>   %{n: \"George\"},\n    ...>   rename: %{n: :name},\n    ...>   operators: %{name: :ilike_or}\n    ...> )\n    [%{field: :name, op: :ilike_or, value: \"George\"}]\n\nSee also `Flop.Filter.new/2`.","ref":"Flop.html#map_to_filter_params/2"},{"type":"function","title":"Flop.meta/3","doc":"Returns meta information for the given query and flop that can be used for\nbuilding the pagination links.\n\n    iex> Flop.meta(MyApp.Pet, %Flop{limit: 10}, repo: Flop.Repo)\n    %Flop.Meta{\n      current_offset: 0,\n      current_page: 1,\n      end_cursor: nil,\n      flop: %Flop{limit: 10},\n      has_next_page?: false,\n      has_previous_page?: false,\n      next_offset: nil,\n      next_page: nil,\n      opts: [repo: Flop.Repo],\n      page_size: 10,\n      previous_offset: nil,\n      previous_page: nil,\n      start_cursor: nil,\n      total_count: 0,\n      total_pages: 0\n    }\n\nThe function returns both the current offset and the current page, regardless\nof the pagination type. If the offset lies in between pages, the current page\nnumber is rounded up. This means that it is possible that the values for\n`current_page` and `next_page` can be identical. This can only occur if you\nuse offset/limit based pagination with arbitrary offsets, but in that case,\nyou will use the `previous_offset`, `current_offset` and `next_offset` values\nto render the pagination links anyway, so this shouldn't be a problem.\n\nUnless cursor-based pagination is used, this function will run a query to\nfigure get the total count of matching records.\n\nThis function does _not_ validate or apply default parameters to the given\nFlop struct. Be sure to validate any user-generated parameters with\n`validate/2` or `validate!/2` before passing them to this function.","ref":"Flop.html#meta/3"},{"type":"function","title":"Flop.named_bindings/3","doc":"Returns the names of the bindings that are required for the filters and order\nclauses of the given Flop.\n\nThe second argument is the schema module that derives `Flop.Schema`.\n\nFor example, your schema module might define a join field called `:owner_age`.\n\n    @derive {\n      Flop.Schema,\n      filterable: [:name, :owner_age],\n      sortable: [:name, :owner_age],\n      adapter_opts: [\n        join_fields: [owner_age: {:owner, :age}]\n      ]\n    }\n\nIf you pass a Flop with a filter on the `:owner_age` field, the returned list\nwill include the `:owner` binding.\n\n    iex> named_bindings(\n    ...>   %Flop{\n    ...>     filters: [%Flop.Filter{field: :owner_age, op: :==, value: 5}]\n    ...>   },\n    ...>   MyApp.Pet\n    ...> )\n    [:owner]\n\nIf on the other hand only normal fields or compound fields are used in the\nfilter and order options, or if the filter values are nil, an empty list will\nbe returned.\n\n    iex> named_bindings(\n    ...>   %Flop{\n    ...>     filters: [\n    ...>       %Flop.Filter{field: :name, op: :==, value: \"George\"},\n    ...>       %Flop.Filter{field: :owner_age, op: :==, value: nil}\n    ...>     ]\n    ...>   },\n    ...>   MyApp.Pet\n    ...> )\n    []\n\nIf a join field is part of a compound field, it will be returned.\n\n    iex> named_bindings(\n    ...>   %Flop{\n    ...>     filters: [\n    ...>       %Flop.Filter{field: :pet_and_owner_name, op: :==, value: \"Mae\"}\n    ...>     ]\n    ...>   },\n    ...>   MyApp.Pet\n    ...> )\n    [:owner]\n\nFor custom fields, you can set the `:bindings` option when you derive the\n`Flop.Schema` protocol.\n\nYou can use this to dynamically build the join clauses needed for the query.\nSee also `Flop.with_named_bindings/4`.\n\nFor more information about join fields, refer to the module documentation of\n`Flop.Schema`.","ref":"Flop.html#named_bindings/3"},{"type":"function","title":"Options - Flop.named_bindings/3","doc":"- `:order` - If `false`, only bindings needed for filtering are included.\n  Defaults to `true`.","ref":"Flop.html#named_bindings/3-options"},{"type":"function","title":"Flop.nest_filters/3","doc":"Converts key/value filter parameters at the root of a map, converts them into\na list of filter parameter maps and nests them under the `:filters` key.\n\nThis is useful in cases where you get some or all filter parameters as\nkey/value pairs instead of a full map with operators, for example when you\nexpose certain filters with fixed operators on an API, or if you want to\nreflect some or all filters in the URL as path parameters or simple query\nparameters (e.g. `/posts/{tag}` or `/posts?s=searchterm`).\n\nThe given map should have either string keys or atom keys. Passing a map with\nmixed keys will lead to unexpected results and will cause an Ecto error when\nthe return value is passed to one of the validation functions.\n\nThe second argument is a list of fields as atoms.\n\nThe `opts` argument is passed to `map_to_filter_params/2`.\n\nThe function returns a map that eventually needs to be passed to one of the\nFlop validation functions (any `Flop.validate*` function) before it can be\nused to make a query.","ref":"Flop.html#nest_filters/3"},{"type":"function","title":"Examples - Flop.nest_filters/3","doc":"Map with atom keys\n\n    iex> nest_filters(%{name: \"Peter\", page_size: 10}, [:name])\n    %{filters: [%{field: :name, op: :==, value: \"Peter\"}], page_size: 10}\n\nMap with string keys\n\n    iex> nest_filters(%{\"name\" => \"Peter\"}, [:name])\n    %{\"filters\" => [%{\"field\" => \"name\", \"op\" => :==, \"value\" =>  \"Peter\"}]}\n\nSpecifying operators\n\n    iex> nest_filters(%{name: \"Peter\"}, [:name], operators: %{name: :!=})\n    %{filters: [%{field: :name, op: :!=, value: \"Peter\"}]}\n\nRenaming fields\n\n    iex> nest_filters(%{nombre: \"Peter\", page_size: 10}, [:nombre],\n    ...>   rename: %{nombre: :name}\n    ...> )\n    %{filters: [%{field: :name, op: :==, value: \"Peter\"}], page_size: 10}\n\n    iex> nest_filters(%{\"nombre\" => \"Peter\"}, [:nombre],\n    ...>   rename: %{nombre: :name}\n    ...> )\n    %{\"filters\" => [%{\"field\" => \"name\", \"op\" => :==, \"value\" =>  \"Peter\"}]}\n\n    iex> nest_filters(%{\"nombre\" => \"Peter\"}, [:nombre],\n    ...>   rename: %{nombre: :name},\n    ...>   operators: %{name: :like}\n    ...> )\n    %{\"filters\" => [%{\"field\" => \"name\", \"op\" => :like, \"value\" =>  \"Peter\"}]}\n\nIf the map already has a `filters` key, the extracted filters are added to\nthe existing filters.\n\n    iex> nest_filters(%{name: \"Peter\", filters: [%{field: \"age\", op: \">\", value: 25}]}, [:name])\n    %{filters: [%{field: \"age\", op: \">\", value: 25}, %{field: :name, op: :==, value: \"Peter\"}]}\n\n    iex> nest_filters(%{\"name\" => \"Peter\", \"filters\" => [%{\"field\" => \"age\", \"op\" => \">\", \"value\" => 25}]}, [:name])\n    %{\"filters\" => [%{\"field\" => \"age\", \"op\" => \">\", \"value\" => 25}, %{\"field\" => \"name\", \"op\" => :==, \"value\" => \"Peter\"}]}\n\nIf the existing filters are formatted as a map with integer indexes as keys as\nproduced by a form, the map is converted to a list first.\n\n    iex> nest_filters(%{name: \"Peter\", filters: %{\"0\" => %{field: \"age\", op: \">\", value: 25}}}, [:name])\n    %{filters: [%{field: \"age\", op: \">\", value: 25}, %{field: :name, op: :==, value: \"Peter\"}]}\n\n    iex> nest_filters(%{\"name\" => \"Peter\", \"filters\" => %{\"0\" => %{\"field\" => \"age\", \"op\" => \">\", \"value\" => 25}}}, [:name])\n    %{\"filters\" => [%{\"field\" => \"age\", \"op\" => \">\", \"value\" => 25}, %{\"field\" => \"name\", \"op\" => :==, \"value\" => \"Peter\"}]}","ref":"Flop.html#nest_filters/3-examples"},{"type":"function","title":"Flop.order_by/3","doc":"Applies the `order_by` and `order_directions` parameters of a `t:Flop.t/0`\nto an `t:Ecto.Queryable.t/0`.\n\nUsed by `Flop.query/2`.\n\nThis function does _not_ validate or apply default parameters to the given\nFlop struct. Be sure to validate any user-generated parameters with\n`validate/2` or `validate!/2` before passing them to this function.\n\nNote that you will need to pass the `for` option in order for Flop to be\nable to find your join, compound, alias and custom field configuration.","ref":"Flop.html#order_by/3"},{"type":"function","title":"Flop.paginate/3","doc":"Applies the pagination parameters of a `t:Flop.t/0` to an\n`t:Ecto.Queryable.t/0`.\n\nThe function supports both `offset`/`limit` based pagination and\n`page`/`page_size` based pagination.\n\nIf you validated the `t:Flop.t/0` with `Flop.validate/1` before, you can be\nsure that the given `t:Flop.t/0` only has pagination parameters set for one\npagination method. If you pass an unvalidated `t:Flop.t/0` that has\npagination parameters set for multiple pagination methods, this function\nwill arbitrarily only apply one of the pagination methods.\n\nUsed by `Flop.query/2`.\n\nThis function does _not_ validate or apply default parameters to the given\nFlop struct. Be sure to validate any user-generated parameters with\n`validate/2` or `validate!/2` before passing them to this function.\n\nNote that you will need to pass the `for` option in order for Flop to be\nable to find your join, compound, alias and custom field configuration.","ref":"Flop.html#paginate/3"},{"type":"function","title":"Flop.push_order/3","doc":"Updates the `order_by` and `order_directions` values of a `Flop` struct.\n\n- If the field is not in the current `order_by` value, it will be prepended to\n  the list. By default, the order direction for the field will be set to\n  `:asc`.\n- If the field is already at the front of the `order_by` list, the order\n  direction will be reversed.\n- If the field is already in the list, but not at the front, it will be moved\n  to the front and the order direction will be set to `:asc` (or the custom\n  asc direction supplied in the `:directions` option).\n- If the `:directions` option——a 2-element tuple——is passed, the first and\n  second elements will be used as custom sort declarations for ascending and\n  descending, respectively.","ref":"Flop.html#push_order/3"},{"type":"function","title":"Examples - Flop.push_order/3","doc":"iex> flop = push_order(%Flop{}, :name)\n    iex> flop.order_by\n    [:name]\n    iex> flop.order_directions\n    [:asc]\n    iex> flop = push_order(flop, :age)\n    iex> flop.order_by\n    [:age, :name]\n    iex> flop.order_directions\n    [:asc, :asc]\n    iex> flop = push_order(flop, :age)\n    iex> flop.order_by\n    [:age, :name]\n    iex> flop.order_directions\n    [:desc, :asc]\n    iex> flop = push_order(flop, :species)\n    iex> flop.order_by\n    [:species, :age, :name]\n    iex> flop.order_directions\n    [:asc, :desc, :asc]\n    iex> flop = push_order(flop, :age)\n    iex> flop.order_by\n    [:age, :species, :name]\n    iex> flop.order_directions\n    [:asc, :asc, :asc]\n\nBy default, the function toggles between `:asc` and `:desc`. You can override\nthis with the `:directions` option.\n\n    iex> directions = {:asc_nulls_last, :desc_nulls_last}\n    iex> flop = push_order(%Flop{}, :ttfb, directions: directions)\n    iex> flop.order_by\n    [:ttfb]\n    iex> flop.order_directions\n    [:asc_nulls_last]\n    iex> flop = push_order(flop, :ttfb, directions: directions)\n    iex> flop.order_by\n    [:ttfb]\n    iex> flop.order_directions\n    [:desc_nulls_last]\n\nIf a string is passed as the second argument, it will be converted to an atom\nusing `String.to_existing_atom/1`. If the atom does not exist, the `Flop`\nstruct will be returned unchanged.\n\n    iex> flop = push_order(%Flop{}, \"name\")\n    iex> flop.order_by\n    [:name]\n    iex> flop = push_order(%Flop{}, \"this_atom_does_not_exist\")\n    iex> flop.order_by\n    nil\n\nSince the pagination cursor depends on the sort order, the `:before` and\n`:after` parameters are reset.\n\n    iex> push_order(%Flop{order_by: [:id], after: \"ABC\"}, :name)\n    %Flop{order_by: [:name, :id], order_directions: [:asc], after: nil}\n    iex> push_order(%Flop{order_by: [:id], before: \"DEF\"}, :name)\n    %Flop{order_by: [:name, :id], order_directions: [:asc], before: nil}","ref":"Flop.html#push_order/3-examples"},{"type":"function","title":"Flop.query/3","doc":"Adds clauses for filtering, ordering and pagination to a\n`t:Ecto.Queryable.t/0`.\n\nThe parameters are represented by the `t:Flop.t/0` type. Any `nil` values\nwill be ignored.\n\nThis function does _not_ validate or apply default parameters to the given\nFlop struct. Be sure to validate any user-generated parameters with\n`validate/2` or `validate!/2` before passing them to this function.","ref":"Flop.html#query/3"},{"type":"function","title":"Examples - Flop.query/3","doc":"iex> flop = %Flop{limit: 10, offset: 19}\n    iex> Flop.query(MyApp.Pet, flop)\n    #Ecto.Query \n\nOr enhance an already defined query:\n\n    iex> require Ecto.Query\n    iex> flop = %Flop{limit: 10}\n    iex> q = Ecto.Query.where(MyApp.Pet, species: \"dog\")\n    iex> Flop.query(q, flop, for: MyApp.Pet)\n    #Ecto.Query \n\nNote that when using cursor-based pagination, the applied limit will be\n`first + 1` or `last + 1`. The extra record is removed by `Flop.run/3`.\n\nAlso note that you will need to pass the `for` option in order for Flop to be\nable to find your join, compound, alias and custom field configuration.","ref":"Flop.html#query/3-examples"},{"type":"function","title":"Flop.reset_cursors/1","doc":"Removes the `after` and `before` cursors from a Flop struct.","ref":"Flop.html#reset_cursors/1"},{"type":"function","title":"Example - Flop.reset_cursors/1","doc":"iex> reset_cursors(%Flop{after: \"A\"})\n    %Flop{}\n\n    iex> reset_cursors(%Flop{before: \"A\"})\n    %Flop{}","ref":"Flop.html#reset_cursors/1-example"},{"type":"function","title":"Flop.reset_filters/1","doc":"Removes all filters from a Flop struct.","ref":"Flop.html#reset_filters/1"},{"type":"function","title":"Example - Flop.reset_filters/1","doc":"iex> reset_filters(%Flop{filters: [\n    ...>   %Flop.Filter{field: :name, value: \"Jim\"}\n    ...> ]})\n    %Flop{filters: []}","ref":"Flop.html#reset_filters/1-example"},{"type":"function","title":"Flop.reset_order/1","doc":"Removes the order parameters from a Flop struct.","ref":"Flop.html#reset_order/1"},{"type":"function","title":"Example - Flop.reset_order/1","doc":"iex> reset_order(%Flop{order_by: [:name], order_directions: [:asc]})\n    %Flop{order_by: nil, order_directions: nil}","ref":"Flop.html#reset_order/1-example"},{"type":"function","title":"Flop.run/3","doc":"Applies the given Flop to the given queryable, retrieves the data and the\nmeta data.\n\nThis function does _not_ validate or apply default parameters to the given\nflop parameters. You can validate the parameters with `Flop.validate/2` or\n`Flop.validate!/2`, or you can use `Flop.validate_and_run/3` or\n`Flop.validate_and_run!/3` instead of this function.\n\nNote that you will need to pass the `for` option to both to `validate/2`,\n`validate!/2` and `run/3`. Otherwise, Flop would not know how to find your\nfield configuration (join fields, alias fields, custom fields, compound\nfields).\n\n    iex> opts = [for: MyApp.Pet]\n    iex> flop = Flop.validate!(%{}, opts)\n    iex> {data, meta} = Flop.run(MyApp.Pet, flop, opts)\n    iex> data == []\n    true\n    iex> match?(%Flop.Meta{}, meta)\n    true\n\nSee the documentation for `Flop.validate_and_run/3` for supported options.","ref":"Flop.html#run/3"},{"type":"function","title":"Flop.set_cursor/2","doc":"Takes a `Flop.Meta` struct and returns a `Flop` struct with updated cursor\npagination params for going to either the previous or the next page.\n\nSee `to_previous_cursor/1` and `to_next_cursor/1` for details.","ref":"Flop.html#set_cursor/2"},{"type":"function","title":"Examples - Flop.set_cursor/2","doc":"iex> set_cursor(\n    ...>   %Flop.Meta{\n    ...>     flop: %Flop{first: 5, after: \"a\"},\n    ...>     has_previous_page?: true, start_cursor: \"b\"\n    ...>   },\n    ...>   :previous\n    ...> )\n    %Flop{last: 5, before: \"b\"}\n\n    iex> set_cursor(\n    ...>   %Flop.Meta{\n    ...>     flop: %Flop{first: 5, after: \"a\"},\n    ...>     has_next_page?: true, end_cursor: \"b\"\n    ...>   },\n    ...>   :next\n    ...> )\n    %Flop{first: 5, after: \"b\"}","ref":"Flop.html#set_cursor/2-examples"},{"type":"function","title":"Flop.set_offset/2","doc":"Sets the offset value of a `Flop` struct while also removing/converting\npagination parameters for other pagination types.\n\n    iex> set_offset(%Flop{limit: 10, offset: 10}, 20)\n    %Flop{offset: 20, limit: 10}\n\n    iex> set_offset(%Flop{page: 5, page_size: 10}, 20)\n    %Flop{limit: 10, offset: 20, page: nil, page_size: nil}\n\n    iex> set_offset(%Flop{limit: 10, offset: 10}, \"20\")\n    %Flop{offset: 20, limit: 10}\n\nThe offset will not be allowed to go below 0.\n\n    iex> set_offset(%Flop{}, -5)\n    %Flop{offset: 0}","ref":"Flop.html#set_offset/2"},{"type":"function","title":"Flop.set_page/2","doc":"Sets the page value of a `Flop` struct while also removing/converting\npagination parameters for other pagination types.\n\n    iex> set_page(%Flop{page: 2, page_size: 10}, 6)\n    %Flop{page: 6, page_size: 10}\n\n    iex> set_page(%Flop{limit: 10, offset: 20}, 8)\n    %Flop{limit: nil, offset: nil, page: 8, page_size: 10}\n\n    iex> set_page(%Flop{page: 2, page_size: 10}, \"6\")\n    %Flop{page: 6, page_size: 10}\n\nThe page number will not be allowed to go below 1.\n\n    iex> set_page(%Flop{}, -5)\n    %Flop{page: 1}","ref":"Flop.html#set_page/2"},{"type":"function","title":"Flop.to_next_cursor/1","doc":"Takes a `Flop.Meta` struct and returns a `Flop` struct with updated cursor\npagination params for going to the next page.","ref":"Flop.html#to_next_cursor/1"},{"type":"function","title":"Examples - Flop.to_next_cursor/1","doc":"iex> to_next_cursor(\n    ...>   %Flop.Meta{\n    ...>     flop: %Flop{first: 5, after: \"a\"},\n    ...>     has_next_page?: true, end_cursor: \"b\"\n    ...>   }\n    ...> )\n    %Flop{first: 5, after: \"b\"}\n\n    iex> to_next_cursor(\n    ...>   %Flop.Meta{\n    ...>     flop: %Flop{last: 5, before: \"b\"},\n    ...>     has_next_page?: true, end_cursor: \"a\"\n    ...>   }\n    ...> )\n    %Flop{first: 5, after: \"a\"}\n\nIf there is no next page, the `Flop` struct is returned unchanged.\n\n    iex> to_next_cursor(\n    ...>   %Flop.Meta{\n    ...>     flop: %Flop{first: 5, after: \"a\"},\n    ...>     has_next_page?: false, start_cursor: \"b\"\n    ...>   }\n    ...> )\n    %Flop{first: 5, after: \"a\"}\n\nIt can happen that `has_next_page?` is `true`, but no end cursor is set,\nfor example if the user is on the second page and switches to the previous\npage right after the only item on the first page is deleted. Since the result\nset is empty and there is no end cursor in this case, this function sets the\nnew `after` parameter to `nil`. By doing this, the user is sent to the first\npage without skipping any items.\n\n    iex> to_next_cursor(\n    ...>   %Flop.Meta{\n    ...>     flop: %Flop{first: 5, after: \"a\"},\n    ...>     has_next_page?: true, end_cursor: nil\n    ...>   }\n    ...> )\n    %Flop{first: 5, after: nil}\n\n    iex> to_next_cursor(\n    ...>   %Flop.Meta{\n    ...>     flop: %Flop{last: 5, before: \"b\"},\n    ...>     has_next_page?: true, end_cursor: nil\n    ...>   }\n    ...> )\n    %Flop{first: 5, after: nil}","ref":"Flop.html#to_next_cursor/1-examples"},{"type":"function","title":"Flop.to_next_offset/2","doc":"Sets the offset of a Flop struct to the next page depending on the limit.\n\nIf the total count is given as the second argument, the offset will not be\nincreased if the last page has already been reached. You can get the total\ncount from the `Flop.Meta` struct. If the Flop has an offset beyond the total\ncount, the offset will be set to the last page.","ref":"Flop.html#to_next_offset/2"},{"type":"function","title":"Examples - Flop.to_next_offset/2","doc":"iex> to_next_offset(%Flop{offset: 10, limit: 5})\n    %Flop{offset: 15, limit: 5}\n\n    iex> to_next_offset(%Flop{offset: 15, limit: 5}, 21)\n    %Flop{offset: 20, limit: 5}\n\n    iex> to_next_offset(%Flop{offset: 15, limit: 5}, 20)\n    %Flop{offset: 15, limit: 5}\n\n    iex> to_next_offset(%Flop{offset: 28, limit: 5}, 22)\n    %Flop{offset: 20, limit: 5}\n\n    iex> to_next_offset(%Flop{offset: -5, limit: 20})\n    %Flop{offset: 0, limit: 20}","ref":"Flop.html#to_next_offset/2-examples"},{"type":"function","title":"Flop.to_next_page/2","doc":"Sets the page of a Flop struct to the next page.\n\nIf the total number of pages is given as the second argument, the page number\nwill not be increased if the last page has already been reached. You can get\nthe total number of pages from the `Flop.Meta` struct.","ref":"Flop.html#to_next_page/2"},{"type":"function","title":"Examples - Flop.to_next_page/2","doc":"iex> to_next_page(%Flop{page: 5})\n    %Flop{page: 6}\n\n    iex> to_next_page(%Flop{page: 5}, 6)\n    %Flop{page: 6}\n\n    iex> to_next_page(%Flop{page: 6}, 6)\n    %Flop{page: 6}\n\n    iex> to_next_page(%Flop{page: 7}, 6)\n    %Flop{page: 6}\n\n    iex> to_next_page(%Flop{page: -5})\n    %Flop{page: 1}","ref":"Flop.html#to_next_page/2-examples"},{"type":"function","title":"Flop.to_previous_cursor/1","doc":"Takes a `Flop.Meta` struct and returns a `Flop` struct with updated cursor\npagination params for going to the previous page.","ref":"Flop.html#to_previous_cursor/1"},{"type":"function","title":"Examples - Flop.to_previous_cursor/1","doc":"iex> to_previous_cursor(\n    ...>   %Flop.Meta{\n    ...>     flop: %Flop{first: 5, after: \"a\"},\n    ...>     has_previous_page?: true, start_cursor: \"b\"\n    ...>   }\n    ...> )\n    %Flop{last: 5, before: \"b\"}\n\n    iex> to_previous_cursor(\n    ...>   %Flop.Meta{\n    ...>     flop: %Flop{last: 5, before: \"b\"},\n    ...>     has_previous_page?: true, start_cursor: \"a\"\n    ...>   }\n    ...> )\n    %Flop{last: 5, before: \"a\"}\n\nIf there is no previous page, the `Flop` struct is returned unchanged.\n\n    iex> to_previous_cursor(\n    ...>   %Flop.Meta{\n    ...>     flop: %Flop{first: 5, after: \"b\"},\n    ...>     has_previous_page?: false, start_cursor: \"a\"\n    ...>   }\n    ...> )\n    %Flop{first: 5, after: \"b\"}\n\nIt can happen that `has_previous_page?` is `true`, but no start cursor is set,\nfor example if the user is on the next-to-last page and switches to the next\npage right after the only item on the last page is deleted. Since the result\nset would be empty and there is no start cursor in this case, this function\nsets the new `before` parameter to `nil`. By doing this, the user is sent to\nthe actual last page without skipping any items.\n\n    iex> to_previous_cursor(\n    ...>   %Flop.Meta{\n    ...>     flop: %Flop{first: 5, after: \"a\"},\n    ...>     has_previous_page?: true, start_cursor: nil\n    ...>   }\n    ...> )\n    %Flop{last: 5, before: nil}\n\n    iex> to_previous_cursor(\n    ...>   %Flop.Meta{\n    ...>     flop: %Flop{last: 5, before: \"b\"},\n    ...>     has_previous_page?: true, start_cursor: nil\n    ...>   }\n    ...> )\n    %Flop{last: 5, before: nil}","ref":"Flop.html#to_previous_cursor/1-examples"},{"type":"function","title":"Flop.to_previous_offset/1","doc":"Sets the offset of a Flop struct to the page depending on the limit.","ref":"Flop.html#to_previous_offset/1"},{"type":"function","title":"Examples - Flop.to_previous_offset/1","doc":"iex> to_previous_offset(%Flop{offset: 20, limit: 10})\n    %Flop{offset: 10, limit: 10}\n\n    iex> to_previous_offset(%Flop{offset: 5, limit: 10})\n    %Flop{offset: 0, limit: 10}\n\n    iex> to_previous_offset(%Flop{offset: 0, limit: 10})\n    %Flop{offset: 0, limit: 10}\n\n    iex> to_previous_offset(%Flop{offset: -2, limit: 10})\n    %Flop{offset: 0, limit: 10}","ref":"Flop.html#to_previous_offset/1-examples"},{"type":"function","title":"Flop.to_previous_page/1","doc":"Sets the page of a Flop struct to the previous page, but not less than 1.","ref":"Flop.html#to_previous_page/1"},{"type":"function","title":"Examples - Flop.to_previous_page/1","doc":"iex> to_previous_page(%Flop{page: 5})\n    %Flop{page: 4}\n\n    iex> to_previous_page(%Flop{page: 1})\n    %Flop{page: 1}\n\n    iex> to_previous_page(%Flop{page: -2})\n    %Flop{page: 1}","ref":"Flop.html#to_previous_page/1-examples"},{"type":"function","title":"Flop.unnest_filters/3","doc":"Takes a Flop, converts it to a map and unnests the filters for the given\nfields.\n\nThis is the reverse operation of `nest_filters/3`, with the caveat that the\nresult of `nest_filters/3` needs to be validated to convert it to a `Flop`\nstruct before it can be passed back to `unnest_filters/3`.\n\nThe function returns a map with `nil` values removed.","ref":"Flop.html#unnest_filters/3"},{"type":"function","title":"Examples - Flop.unnest_filters/3","doc":"iex> unnest_filters(\n    ...>   %Flop{\n    ...>     filters: [%Flop.Filter{field: :name, op: :==, value: \"Peter\"}],\n    ...>     page_size: 10\n    ...>   },\n    ...>   [:name]\n    ...> )\n    %{name: \"Peter\", page_size: 10}\n\n    iex> unnest_filters(\n    ...>   %Flop{\n    ...>     filters: [%Flop.Filter{field: :name, op: :==, value: nil}],\n    ...>     page_size: 10\n    ...>   },\n    ...>   [:name]\n    ...> )\n    %{page_size: 10}\n\nTo rename fields, you can pass a map, where the keys are the field names in\nthe unnested map and the values are the field names in the `Flop.Filter`\nstruct. You can pass the exact same rename map to `nest_filters/3` and\n`unnest_filters/3` to revert the nesting/unnesting.\n\n    iex> unnest_filters(\n    ...>   %Flop{\n    ...>     filters: [%Flop.Filter{field: :name, op: :==, value: \"Peter\"}],\n    ...>     page_size: 10\n    ...>   },\n    ...>   [:name],\n    ...>   rename: %{nombre: :name}\n    ...> )\n    %{nombre: \"Peter\", page_size: 10}","ref":"Flop.html#unnest_filters/3-examples"},{"type":"function","title":"Flop.validate/2","doc":"Validates a `t:Flop.t/0`.","ref":"Flop.html#validate/2"},{"type":"function","title":"Examples - Flop.validate/2","doc":"iex> params = %{\"limit\" => 10, \"offset\" => 0, \"texture\" => \"fluffy\"}\n    iex> Flop.validate(params)\n    {:ok,\n     %Flop{\n       filters: [],\n       limit: 10,\n       offset: 0,\n       order_by: nil,\n       order_directions: nil,\n       page: nil,\n       page_size: nil\n     }}\n\n    iex> flop = %Flop{offset: -1}\n    iex> {:error, %Flop.Meta{} = meta} = Flop.validate(flop)\n    iex> meta.errors\n    [\n      offset: [\n        {\"must be greater than or equal to %{number}\",\n         [validation: :number, kind: :greater_than_or_equal_to, number: 0]}\n      ]\n    ]\n\nIt also makes sure that only one pagination method is used.\n\n    iex> params = %{limit: 10, offset: 0, page: 5, page_size: 10}\n    iex> {:error, %Flop.Meta{} = meta} = Flop.validate(params)\n    iex> meta.errors\n    [limit: [{\"cannot combine multiple pagination types\", []}]]\n\nIf you derived `Flop.Schema` in your Ecto schema to define the filterable\nand sortable fields, you can pass the module name to the function to validate\nthat only allowed fields are used. The function will also apply any default\nvalues set for the schema.\n\n    iex> params = %{\"order_by\" => [\"species\"]}\n    iex> {:error, %Flop.Meta{} = meta} = Flop.validate(params, for: MyApp.Pet)\n    iex> [order_by: [{msg, [_, {_, enum}]}]] = meta.errors\n    iex> msg\n    \"has an invalid entry\"\n    iex> enum\n    [:name, :age, :owner_name, :owner_age]\n\nNote that currently, trying to use an existing field that is not allowed as\nseen above will result in the error message `has an invalid entry`, while\ntrying to use a field name that does not exist in the schema (or more\nprecisely: a field name that doesn't exist as an atom) will result in\nthe error message `is invalid`. This might change in the future.","ref":"Flop.html#validate/2-examples"},{"type":"function","title":"Flop.validate!/2","doc":"Same as `Flop.validate/2`, but raises an `Ecto.InvalidChangesetError` if the\nparameters are invalid.","ref":"Flop.html#validate!/2"},{"type":"function","title":"Flop.validate_and_run/3","doc":"Validates the given flop parameters and retrieves the data and meta data on\nsuccess.\n\n    iex> {:ok, {[], %Flop.Meta{}}} =\n    ...>   Flop.validate_and_run(MyApp.Pet, %Flop{}, for: MyApp.Pet)\n    iex> {:error, %Flop.Meta{} = meta} =\n    ...>   Flop.validate_and_run(MyApp.Pet, %Flop{limit: -1})\n    iex> meta.errors\n    [\n      limit: [\n        {\"must be greater than %{number}\",\n         [validation: :number, kind: :greater_than, number: 0]}\n      ]\n    ]","ref":"Flop.html#validate_and_run/3"},{"type":"function","title":"Options - Flop.validate_and_run/3","doc":"- `for`: Passed to `Flop.validate/2`. Also used to look up the join, alias,\n  compound and custom field configuration.\n- `repo`: The `Ecto.Repo` module. Required if no default repo is configured.\n- `cursor_value_func`: An arity-2 function to be used to retrieve an\n  unencoded cursor value from a query result item and the `order_by` fields.\n  Defaults to `Flop.Cursor.get_cursor_from_node/2`.\n- `count_query`: Lets you override the base query for counting, e.g. if you\n  don't want to include unnecessary joins. The filter parameters are applied\n  to the given query. See also `Flop.count/3`.","ref":"Flop.html#validate_and_run/3-options"},{"type":"function","title":"Flop.validate_and_run!/3","doc":"Same as `Flop.validate_and_run/3`, but raises on error.","ref":"Flop.html#validate_and_run!/3"},{"type":"function","title":"Flop.with_named_bindings/4","doc":"Applies a callback function to a query for all named bindings that are\nnecessary for the given `Flop` parameters.\n\nThe callback function must accept a queryable and the name of the binding and\nreturn a query that includes the given binding. This is the same as\n`Ecto.Query.with_named_binding/3`, except that the callback function _must_\ntake the second argument.","ref":"Flop.html#with_named_bindings/4"},{"type":"function","title":"Options - Flop.with_named_bindings/4","doc":"- `:for` (required) - The schema module that derives `Flop.Schema`.\n- `:order` - If `false`, only bindings needed for filtering are included.\n  Defaults to `true`.","ref":"Flop.html#with_named_bindings/4-options"},{"type":"function","title":"Example - Flop.with_named_bindings/4","doc":"def list_pets(params) do\n      opts = [for: Pet]\n\n      with {:ok, flop}   Flop.with_named_bindings(flop, &join_pet_assocs/2, opts)\n        |> Flop.run(flop, opts)\n      end\n    end\n\n    defp join_pet_assocs(query, :owner) do\n      join(query, [p], o in assoc(p, :owner), as: :owner)\n    end\n\n    defp join_pet_assocs(query, :toys) do\n      join(query, [p], t in assoc(p, :toys), as: :toys)\n    end\n\nSince the callback function has the same arguments and return value as the one\npassed to `Ecto.Query.with_named_binding/3`, you can reuse the function to add\nany other bindings you may need for the query besides the ones for Flop\nfilters.\n\nThis also means you can use `Ecto.Query.with_named_binding/3` to recursively\nadd bindings in case you need intermediate joins to get to a nested\nassociation.\n\n    def list_owners(params) do\n      opts = [for: Owner]\n\n      with {:ok, flop}   Flop.with_named_bindings(flop, &join_owner_assocs/2, opts)\n        |> Flop.run(flop, opts)\n      end\n    end\n\n    defp join_owner_assocs(query, :pets) do\n      join(query, [o], p in assoc(o, :pets), as: :pets)\n    end\n\n    defp join_owner_assocs(query, :toys) do\n      query\n      |> Ecto.Query.with_named_binding(:pets, &join_owner_assocs/2)\n      |> join(query, [pets: p], t in assoc(p, :toys), as: :toys)\n    end","ref":"Flop.html#with_named_bindings/4-example"},{"type":"type","title":"Flop.adapter_option/0","doc":"","ref":"Flop.html#t:adapter_option/0"},{"type":"type","title":"Flop.default_order/0","doc":"A map with the order fields and the order directions.\n\nEach atom in `:order_by` corresponds to a field by which the data is ordered.\nThe list of order fields and order directions is zipped when the parameters\nare applied. If no order directions are given or the list of order directions\nis shorter than the list of order fields, `:asc` is used as a default order\ndirection.","ref":"Flop.html#t:default_order/0"},{"type":"type","title":"Flop.option/0","doc":"These options can be passed to most functions or configured via the\napplication environment.","ref":"Flop.html#t:option/0"},{"type":"type","title":"Options - Flop.option/0","doc":"#","ref":"Flop.html#t:option/0-options"},{"type":"type","title":"General - Flop.option/0","doc":"- `:for` - The Ecto schema module for validation and query building.\n  `Flop.Schema` must be derived for this module.\n- `:cursor_value_func` - A function used to extract the cursor value from a\n  record. It takes the record and the list of fields used in the `ORDER BY`\n  clause as arguments, and returns a map with the order fields as keys and the\n  corresponding record values as values. Default is\n  `Flop.Cursor.get_cursor_from_node/2`.\n- `:replace_invalid_params` - If set to `true`, invalid parameters are\n  replaced with default values or removed instead of causing errors. Default\n  is `false`.\n\n#","ref":"Flop.html#t:option/0-general"},{"type":"type","title":"Defaults - Flop.option/0","doc":"- `:default_limit` - The default limit for queries. Used when no specific\n  limit is set in the parameters or schema.  Set to `false` to not set any\n  default limit. Default is `50`.\n- `:default_order` - The default ordering for a query when no order is\n  specified in the parameters, or if ordering is disabled. Can be set in the\n  schema or in the options passed to the query functions.\n- `:default_pagination_type` - The default pagination type when it cannot be\n  inferred from the parameters.\n- `:max_limit` - The maximum limit for queries. Used when no maximum limit is\n  set in the parameters or schema. Set to `false` to not set any maximum\n  limit. Default is `1000`.\n\n#","ref":"Flop.html#t:option/0-defaults"},{"type":"type","title":"Modifying counts - Flop.option/0","doc":"- `:count_query` - A separate base query for counting. Can only be passed as\n  an option to one of the query functions. See `Flop.validate_and_run/3` and\n  `Flop.count/3`.\n- `:count` - A precomputed count. Useful when the count is already known.\n\n#","ref":"Flop.html#t:option/0-modifying-counts"},{"type":"type","title":"Disabling features - Flop.option/0","doc":"- `:filtering` (boolean) - Enables or disables filtering. When set to `false`,\n  filter parameters are ignored.\n- `:ordering` (boolean) - Enables or disables ordering. When set to `false`,\n  order parameters are ignored, but the default order is still applied.\n- `:pagination` (boolean) - Enables or disables pagination. When set to\n  `false`, pagination parameters are ignored.\n- `:pagination_types` - The allowed pagination types. Parameters for\n  disallowed pagination types will not be cast. By default, all types are\n  allowed. See also `t:Flop.pagination_type/0`.\n\n#","ref":"Flop.html#t:option/0-disabling-features"},{"type":"type","title":"Additional options - Flop.option/0","doc":"- `:extra_opts` (keyword list) - Extra options for custom fields.\n- `:adapter_opts` - Adapter-specific options. For backward compatibility,\n  options for the Ecto adapter can be set directly at the root level.","ref":"Flop.html#t:option/0-additional-options"},{"type":"type","title":"Look-up order - Flop.option/0","doc":"Options are looked up in the following order:\n\n1. Function arguments\n2. Schema-level options\n3. Module-level options in the config (backend) module\n4. Global options in the application environment\n5. Library defaults","ref":"Flop.html#t:option/0-look-up-order"},{"type":"type","title":"Flop.order_direction/0","doc":"Represents the supported order direction values.","ref":"Flop.html#t:order_direction/0"},{"type":"type","title":"Flop.pagination_type/0","doc":"Represents the pagination type.\n\n- `:offset` - pagination using the `offset` and `limit` parameters\n- `:page` - pagination using the `page` and `page_size` parameters\n- `:first` - cursor-based pagination using the `first` and `after` parameters\n- `:last` - cursor-based pagination using the `last` and `before` parameters","ref":"Flop.html#t:pagination_type/0"},{"type":"type","title":"Flop.t/0","doc":"Represents the query parameters for filtering, ordering and pagination.\n\n#","ref":"Flop.html#t:t/0"},{"type":"type","title":"Fields - Flop.t/0","doc":"- `after`: Used for cursor-based pagination. Must be used with `first` or a\n  default limit.\n- `before`: Used for cursor-based pagination. Must be used with `last` or a\n  default limit.\n- `decoded_cursor`: Used internally to hold on to the decoded cursor between\n  validation and query execution. Value is discarded when meta is built.\n- `limit`, `offset`: Used for offset-based pagination.\n- `first`: Used for cursor-based pagination. Can be used alone to begin\n  pagination from the start or in conjunction with `after`.\n- `last`: Used for cursor-based pagination.\n- `page`, `page_size`: Used for offset-based pagination as an alternative to\n  `offset` and `limit`.\n- `order_by`: List of fields to order by. Fields can be restricted by\n  deriving `Flop.Schema` in your Ecto schema.\n- `order_directions`: List of order directions applied to the fields defined\n  in `order_by`. If empty or the list is shorter than the `order_by` list,\n  `:asc` will be used as a default for each missing order direction.\n- `filters`: List of filters, see `t:Flop.Filter.t/0`. Each `Filter.t()`\n  represents a filter operation on a specific field.\n\nNote: Pagination fields are mutually exclusive.","ref":"Flop.html#t:t/0-fields"},{"type":"module","title":"Flop.Cursor","doc":"Functions for encoding, decoding and extracting cursor values.","ref":"Flop.Cursor.html"},{"type":"function","title":"Flop.Cursor.decode/1","doc":"Decodes a cursor value.\n\nReturns `:error` if the cursor cannot be decoded or the decoded term is not a\nmap with atom keys.\n\n    iex> Flop.Cursor.decode(\"g3QAAAABZAACaWRiAAACDg==\")\n    {:ok, %{id: 526}}\n\n    iex> Flop.Cursor.decode(\"AAAH\")\n    :error\n\n    iex> f = fn a -> a + 1 end\n    iex> cursor = Flop.Cursor.encode(%{a: f})\n    iex> Flop.Cursor.decode(cursor)\n    :error\n\n    iex> cursor = Flop.Cursor.encode(a: \"b\")\n    iex> Flop.Cursor.decode(cursor)\n    :error\n\n    iex> cursor = Flop.Cursor.encode(%{\"a\" => \"b\"})\n    iex> Flop.Cursor.decode(cursor)\n    :error\n\nTrying to decode a cursor that contains non-existent atoms also results in an\nerror.\n\n    iex> Flop.Cursor.decode(\"g3QAAAABZAAGYmFybmV5ZAAGcnViYmVs\")\n    :error","ref":"Flop.Cursor.html#decode/1"},{"type":"function","title":"Flop.Cursor.decode!/1","doc":"Same as `Flop.Cursor.decode/1`, but raises an error if the cursor is invalid.\n\n    iex> Flop.Cursor.decode!(\"g3QAAAABZAACaWRiAAACDg==\")\n    %{id: 526}","ref":"Flop.Cursor.html#decode!/1"},{"type":"function","title":"Flop.Cursor.encode/1","doc":"Encodes a cursor value.\n\n    Flop.Cursor.encode(%{email: \"peter@mail\", name: \"Peter\"})\n    \"g3QAAAACdwRuYW1lbQAAAAVQZXRlcncFZW1haWxtAAAACnBldGVyQG1haWw=\"","ref":"Flop.Cursor.html#encode/1"},{"type":"function","title":"Flop.Cursor.get_cursor_from_edge/2","doc":"Takes a tuple with the node and the edge and the `order_by` field list and\nreturns the cursor value derived from the edge map.\n\nIf a map is passed instead of a tuple, it retrieves the cursor value from that\nmap.\n\nThis function can be used for the `:cursor_value_func` option. See also\n`Flop.Cursor.get_cursor_from_node/2`.\n\n    iex> record = %{id: 20, name: \"George\", age: 62}\n    iex> edge = %{id: 25, relation: \"sibling\"}\n    iex>\n    iex> Flop.Cursor.get_cursor_from_edge({record, edge}, [:id])\n    %{id: 25}\n    iex> Flop.Cursor.get_cursor_from_edge({record, edge}, [:id, :relation])\n    %{id: 25, relation: \"sibling\"}\n    iex> Flop.Cursor.get_cursor_from_edge(record, [:id])\n    %{id: 20}\n\nIf the edge is a struct that derives `Flop.Schema`, join and compound fields\nare resolved according to the configuration.\n\n    iex> record = %{id: 25, relation: \"sibling\"}\n    iex> edge = %MyApp.Pet{\n    ...>   name: \"George\",\n    ...>   owner: %MyApp.Owner{name: \"Carl\"}\n    ...> }\n    iex>\n    iex> Flop.Cursor.get_cursor_from_edge({record, edge}, [:owner_name])\n    %{owner_name: \"Carl\"}\n    iex> Flop.Cursor.get_cursor_from_edge(edge, [:owner_name])\n    %{owner_name: \"Carl\"}\n\n    iex> record = %{id: 25, relation: \"sibling\"}\n    iex> edge = %MyApp.Pet{\n    ...>   given_name: \"George\",\n    ...>   family_name: \"Gooney\"\n    ...> }\n    iex> Flop.Cursor.get_cursor_from_edge({record, edge}, [:full_name])\n    %{full_name: \"Gooney George\"}\n    iex> Flop.Cursor.get_cursor_from_edge(edge, [:full_name])\n    %{full_name: \"Gooney George\"}","ref":"Flop.Cursor.html#get_cursor_from_edge/2"},{"type":"function","title":"Flop.Cursor.get_cursor_from_node/2","doc":"Takes a tuple with the node and the edge and the `order_by` field list and\nreturns the cursor value derived from the node map.\n\nIf a map is passed instead of a tuple, it retrieves the cursor value from that\nmap.\n\nThis function is used as a default if no `:cursor_value_func` option is\nset. See also `Flop.Cursor.get_cursor_from_edge/2`.\n\n    iex> record = %{id: 20, name: \"George\", age: 62}\n    iex> edge = %{id: 25, relation: \"sibling\"}\n    iex>\n    iex> Flop.Cursor.get_cursor_from_node({record, edge}, [:id])\n    %{id: 20}\n    iex> Flop.Cursor.get_cursor_from_node({record, edge}, [:id, :name])\n    %{id: 20, name: \"George\"}\n    iex> Flop.Cursor.get_cursor_from_node(record, [:id])\n    %{id: 20}\n\nIf the node is a struct that derives `Flop.Schema`, join and compound fields\nare resolved according to the configuration.\n\n    iex> record = %MyApp.Pet{\n    ...>   name: \"George\",\n    ...>   owner: %MyApp.Owner{name: \"Carl\"}\n    ...> }\n    iex> edge = %{id: 25, relation: \"sibling\"}\n    iex>\n    iex> Flop.Cursor.get_cursor_from_node({record, edge}, [:owner_name])\n    %{owner_name: \"Carl\"}\n    iex> Flop.Cursor.get_cursor_from_node(record, [:owner_name])\n    %{owner_name: \"Carl\"}\n\n    iex> record = %MyApp.Pet{\n    ...>   given_name: \"George\",\n    ...>   family_name: \"Gooney\"\n    ...> }\n    iex> edge = %{id: 25, relation: \"sibling\"}\n    iex> Flop.Cursor.get_cursor_from_node({record, edge}, [:full_name])\n    %{full_name: \"Gooney George\"}\n    iex> Flop.Cursor.get_cursor_from_node(record, [:full_name])\n    %{full_name: \"Gooney George\"}","ref":"Flop.Cursor.html#get_cursor_from_node/2"},{"type":"function","title":"Flop.Cursor.get_cursors/3","doc":"Retrieves the start and end cursors from a query result.\n\n    iex> results = [%{name: \"Mary\"}, %{name: \"Paul\"}, %{name: \"Peter\"}]\n    iex> order_by = [:name]\n    iex>\n    iex> {start_cursor, end_cursor} =\n    ...>   Flop.Cursor.get_cursors(results, order_by)\n    {\"g3QAAAABdwRuYW1lbQAAAARNYXJ5\", \"g3QAAAABdwRuYW1lbQAAAAVQZXRlcg==\"}\n    iex>\n    iex> Flop.Cursor.decode(start_cursor)\n    {:ok, %{name: \"Mary\"}}\n    iex> Flop.Cursor.decode(end_cursor)\n    {:ok, %{name: \"Peter\"}}\n\nIf the result set is empty, the cursor values will be `nil`.\n\n    iex> Flop.Cursor.get_cursors([], [:id])\n    {nil, nil}\n\nThe default function to retrieve the cursor value from the query result is\n`Flop.Cursor.get_cursor_from_node/2`, which expects the query result to be a\nmap or a 2-tuple. You can set the `cursor_value_func` option to use\nanother function. Flop also comes with `Flop.Cursor.get_cursor_from_edge/2`.\n\nIf the records in the result set are not maps, you can define a custom cursor\nvalue function like this:\n\n    iex> results = [{\"Mary\", 1936}, {\"Paul\", 1937}, {\"Peter\", 1938}]\n    iex> cursor_func = fn {name, year}, order_fields ->\n    ...>   Enum.into(order_fields, %{}, fn\n    ...>     :name -> {:name, name}\n    ...>     :year -> {:year, year}\n    ...>   end)\n    ...> end\n    iex> opts = [cursor_value_func: cursor_func]\n    iex>\n    iex> {start_cursor, end_cursor} =\n    ...>   Flop.Cursor.get_cursors(results, [:name, :year], opts)\n    {\"g3QAAAACdwRuYW1lbQAAAARNYXJ5dwR5ZWFyYgAAB5A=\",\n      \"g3QAAAACdwRuYW1lbQAAAAVQZXRlcncEeWVhcmIAAAeS\"}\n    iex>\n    iex> Flop.Cursor.decode(start_cursor)\n    {:ok, %{name: \"Mary\", year: 1936}}\n    iex> Flop.Cursor.decode(end_cursor)\n    {:ok, %{name: \"Peter\", year: 1938}}","ref":"Flop.Cursor.html#get_cursors/3"},{"type":"module","title":"Flop.FieldInfo","doc":"Defines a struct that holds the information about a schema field.\n\nThis struct is mainly for use by adapters.","ref":"Flop.FieldInfo.html"},{"type":"type","title":"Flop.FieldInfo.t/0","doc":"Contains the information about a schema field.\n\n- `ecto_type` - The Ecto type of the field. This value is used to determine\n  which operators can be used on the field and to determine how to cast\n  filter values.\n- `operators` - The allowed filter operators on this field. If `nil`, the\n  allowed operators are determined based on the `ecto_type`. If set, the\n  given operator list is used instead.\n- `extra` - A map with additional configuration for the field. The contents\n  depend on the specific adapter.","ref":"Flop.FieldInfo.html#t:t/0"},{"type":"module","title":"Flop.Filter","doc":"Defines a filter.","ref":"Flop.Filter.html"},{"type":"function","title":"Flop.Filter.allowed_operators/1","doc":"Returns the allowed operators for the given Ecto type.\n\nIf the given value is not a native Ecto type, a list with all operators is\nreturned.\n\n    iex> allowed_operators(:integer)\n    [:==, :!=, :empty, :not_empty, : =, :>, :in, :not_in]","ref":"Flop.Filter.html#allowed_operators/1"},{"type":"function","title":"Flop.Filter.allowed_operators/2","doc":"Returns the allowed operators for the given schema module and field.\n\nFor regular Ecto schema fields, the type is derived via schema reflection.\n\nIf the given schema module derives `Flop.Schema`, the type of join and\ncustom fields is determined via the `ecto_type` option. Compound files are\nalways handled as string fields, minus unsupported operators.\n\nIf the type cannot be determined or if the type is not a native Ecto type, a\nlist with all operators is returned.\n\n    iex> allowed_operators(Pet, :age)\n    [:==, :!=, :empty, :not_empty, : =, :>, :in, :not_in]","ref":"Flop.Filter.html#allowed_operators/2"},{"type":"function","title":"Flop.Filter.delete/2","doc":"Deletes the filters for the given field from a list of filters.","ref":"Flop.Filter.html#delete/2"},{"type":"function","title":"Examples - Flop.Filter.delete/2","doc":"#","ref":"Flop.Filter.html#delete/2-examples"},{"type":"function","title":"Flop.Filter struct - Flop.Filter.delete/2","doc":"iex> delete(\n    ...>   [\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n    ...>     %Flop.Filter{field: :age, op: :>, value: 8}\n    ...>   ],\n    ...>   :name\n    ...> )\n    [%Flop.Filter{field: :age, op: :>, value: 8}]\n\n    iex> delete(\n    ...>   [\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n    ...>     %Flop.Filter{field: :age, op: :>, value: 8},\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n    ...>   ],\n    ...>   :name\n    ...> )\n    [%Flop.Filter{field: :age, op: :>, value: 8}]\n\n    iex> delete([%Flop.Filter{field: :name, op: :==, value: \"Joe\"}], :age)\n    [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}]\n\n#","ref":"Flop.Filter.html#delete/2-flop-filter-struct"},{"type":"function","title":"Map with atom keys - Flop.Filter.delete/2","doc":"iex> delete(\n    ...>   [\n    ...>     %{field: :name, op: :==, value: \"Joe\"},\n    ...>     %{field: :age, op: :>, value: 8}\n    ...>   ],\n    ...>   :name\n    ...> )\n    [%{field: :age, op: :>, value: 8}]\n\n#","ref":"Flop.Filter.html#delete/2-map-with-atom-keys"},{"type":"function","title":"Map with string keys - Flop.Filter.delete/2","doc":"iex> delete(\n    ...>   [\n    ...>     %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n    ...>     %{\"field\" => \"age\", \"op\" => \">\", \"value\" => \"8\"}\n    ...>   ],\n    ...>   :name\n    ...> )\n    [%{\"field\" => \"age\", \"op\" => \">\", \"value\" => \"8\"}]\n\n#","ref":"Flop.Filter.html#delete/2-map-with-string-keys"},{"type":"function","title":"Indexed map - Flop.Filter.delete/2","doc":"Filters passed as an indexed map will be converted to a list, even if no\nmatching filter exists.\n\n    iex> delete(\n    ...>   %{\n    ...>     0 => %{field: \"name\", op: \"==\", value: \"Joe\"},\n    ...>     1 => %{field: \"age\", op: \">\", value: \"8\"}\n    ...>   },\n    ...>   :name\n    ...> )\n    [%{field: \"age\", op: \">\", value: \"8\"}]\n\n    iex> delete(\n    ...>   %{\n    ...>     \"0\" => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n    ...>     \"1\" => %{\"field\" => \"age\", \"op\" => \">\", \"value\" => \"8\"}\n    ...>   },\n    ...>   :name\n    ...> )\n    [%{\"field\" => \"age\", \"op\" => \">\", \"value\" => \"8\"}]","ref":"Flop.Filter.html#delete/2-indexed-map"},{"type":"function","title":"Flop.Filter.delete_first/2","doc":"Deletes the first filter in list of filters for the given field.","ref":"Flop.Filter.html#delete_first/2"},{"type":"function","title":"Examples - Flop.Filter.delete_first/2","doc":"#","ref":"Flop.Filter.html#delete_first/2-examples"},{"type":"function","title":"Flop.Filter struct - Flop.Filter.delete_first/2","doc":"iex> delete_first(\n    ...>   [\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n    ...>     %Flop.Filter{field: :age, op: :>, value: 8},\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n    ...>   ],\n    ...>   :name\n    ...> )\n    [\n      %Flop.Filter{field: :age, op: :>, value: 8},\n      %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n    ]\n\n    iex> delete_first([%Flop.Filter{field: :name, op: :==, value: \"Joe\"}], :age)\n    [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}]\n\n#","ref":"Flop.Filter.html#delete_first/2-flop-filter-struct"},{"type":"function","title":"Map with atom keys - Flop.Filter.delete_first/2","doc":"iex> delete_first(\n    ...>   [\n    ...>     %{field: :name, op: :==, value: \"Joe\"},\n    ...>     %{field: :age, op: :>, value: 8},\n    ...>     %{field: :name, op: :==, value: \"Jim\"}\n    ...>   ],\n    ...>   :name\n    ...> )\n    [\n      %{field: :age, op: :>, value: 8},\n      %{field: :name, op: :==, value: \"Jim\"}\n    ]\n\n#","ref":"Flop.Filter.html#delete_first/2-map-with-atom-keys"},{"type":"function","title":"Map with string keys - Flop.Filter.delete_first/2","doc":"iex> delete_first(\n    ...>   [\n    ...>     %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n    ...>     %{\"field\" => \"age\", \"op\" => \">\", \"value\" => 8},\n    ...>     %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Jim\"}\n    ...>   ],\n    ...>   :name\n    ...> )\n    [\n      %{\"field\" => \"age\", \"op\" => \">\", \"value\" => 8},\n      %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Jim\"}\n    ]\n\n    iex> delete_first(\n    ...>   [\n    ...>     %{\"field\" => :name, \"op\" => :==, \"value\" => \"Joe\"},\n    ...>     %{\"field\" => :age, \"op\" => :>, \"value\" => 8},\n    ...>     %{\"field\" => :name, \"op\" => :==, \"value\" => \"Jim\"}\n    ...>   ],\n    ...>   :name\n    ...> )\n    [\n      %{\"field\" => :age, \"op\" => :>, \"value\" => 8},\n      %{\"field\" => :name, \"op\" => :==, \"value\" => \"Jim\"}\n    ]\n\n#","ref":"Flop.Filter.html#delete_first/2-map-with-string-keys"},{"type":"function","title":"Indexed map - Flop.Filter.delete_first/2","doc":"Filters passed as an indexed map will be converted to a list, even if no\nmatching filter exists.\n\n    iex> delete_first(\n    ...>   %{\n    ...>     0 => %{field: \"name\", op: \"==\", value: \"Joe\"},\n    ...>     1 => %{field: \"age\", op: \">\", value: 8},\n    ...>     2 => %{field: \"name\", op: \"==\", value: \"Jim\"}\n    ...>   },\n    ...>   :name\n    ...> )\n    [\n      %{field: \"age\", op: \">\", value: 8},\n      %{field: \"name\", op: \"==\", value: \"Jim\"}\n    ]\n\n    iex> delete_first(\n    ...>   %{\n    ...>     \"0\" => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n    ...>     \"1\" => %{\"field\" => \"age\", \"op\" => \">\", \"value\" => 8},\n    ...>     \"2\" => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Jim\"}\n    ...>   },\n    ...>   :name\n    ...> )\n    [\n      %{\"field\" => \"age\", \"op\" => \">\", \"value\" => 8},\n      %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Jim\"}\n    ]","ref":"Flop.Filter.html#delete_first/2-indexed-map"},{"type":"function","title":"Flop.Filter.drop/2","doc":"Removes the filters for the given fields from a list of filters.","ref":"Flop.Filter.html#drop/2"},{"type":"function","title":"Examples - Flop.Filter.drop/2","doc":"#","ref":"Flop.Filter.html#drop/2-examples"},{"type":"function","title":"Flop.Filter struct - Flop.Filter.drop/2","doc":"iex> drop(\n    ...>   [\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n    ...>     %Flop.Filter{field: :age, op: :>, value: 8},\n    ...>     %Flop.Filter{field: :color, op: :==, value: \"blue\"}\n    ...>   ],\n    ...>   [:name, :color]\n    ...> )\n    [%Flop.Filter{field: :age, op: :>, value: 8}]\n\n    iex> drop(\n    ...>   [\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n    ...>     %Flop.Filter{field: :age, op: :>, value: 8},\n    ...>     %Flop.Filter{field: :color, op: :==, value: \"blue\"},\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n    ...>   ],\n    ...>   [:name, :species]\n    ...> )\n    [\n      %Flop.Filter{field: :age, op: :>, value: 8},\n      %Flop.Filter{field: :color, op: :==, value: \"blue\"}\n    ]\n\n#","ref":"Flop.Filter.html#drop/2-flop-filter-struct"},{"type":"function","title":"Map with atom keys - Flop.Filter.drop/2","doc":"iex> drop(\n    ...>   [\n    ...>     %{field: :name, op: :==, value: \"Joe\"},\n    ...>     %{field: :age, op: :>, value: 8},\n    ...>     %{field: :color, op: :==, value: \"blue\"}\n    ...>   ],\n    ...>   [:name, :color]\n    ...> )\n    [%{field: :age, op: :>, value: 8}]\n\n#","ref":"Flop.Filter.html#drop/2-map-with-atom-keys"},{"type":"function","title":"Map with string keys - Flop.Filter.drop/2","doc":"iex> drop(\n    ...>   [\n    ...>     %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n    ...>     %{\"field\" => \"age\", \"op\" => \">\", \"value\" => \"8\"},\n    ...>     %{\"field\" => \"color\", \"op\" => \"==\", \"value\" => \"blue\"}\n    ...>   ],\n    ...>   [:name, :color]\n    ...> )\n    [%{\"field\" => \"age\", \"op\" => \">\", \"value\" => \"8\"}]\n\n    iex> drop(\n    ...>   [\n    ...>     %{\"field\" => :name, \"op\" => :==, \"value\" => \"Joe\"},\n    ...>     %{\"field\" => :age, \"op\" => :>, \"value\" => \"8\"},\n    ...>     %{\"field\" => :color, \"op\" => :==, \"value\" => \"blue\"}\n    ...>   ],\n    ...>   [:name, :color]\n    ...> )\n    [%{\"field\" => :age, \"op\" => :>, \"value\" => \"8\"}]\n\n\n#","ref":"Flop.Filter.html#drop/2-map-with-string-keys"},{"type":"function","title":"Indexed map - Flop.Filter.drop/2","doc":"Filters passed as an indexed map will be converted to a list, even if no\nmatching filter exists.\n\n    iex> drop(\n    ...>   %{\n    ...>     0 => %{field: \"name\", op: \"==\", value: \"Joe\"},\n    ...>     1 => %{field: \"age\", op: \">\", value: \"8\"},\n    ...>     2 => %{field: \"color\", op: \"==\", value: \"blue\"}\n    ...>   },\n    ...>   [:name, :color]\n    ...> )\n    [%{field: \"age\", op: \">\", value: \"8\"}]\n\n    iex> drop(\n    ...>   %{\n    ...>     \"0\" => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n    ...>     \"1\" => %{\"field\" => \"age\", \"op\" => \">\", \"value\" => \"8\"},\n    ...>     \"2\" => %{\"field\" => \"color\", \"op\" => \"==\", \"value\" => \"blue\"}\n    ...>   },\n    ...>   [:name, :color]\n    ...> )\n    [%{\"field\" => \"age\", \"op\" => \">\", \"value\" => \"8\"}]","ref":"Flop.Filter.html#drop/2-indexed-map"},{"type":"function","title":"Flop.Filter.fetch/2","doc":"Fetches the first filter for the given field and returns it in a tuple.","ref":"Flop.Filter.html#fetch/2"},{"type":"function","title":"Examples - Flop.Filter.fetch/2","doc":"#","ref":"Flop.Filter.html#fetch/2-examples"},{"type":"function","title":"Flop.Filter struct - Flop.Filter.fetch/2","doc":"iex> fetch([], :name)\n    :error\n\n    iex> fetch([%Flop.Filter{field: :name, op: :==, value: \"Joe\"}], :name)\n    {:ok, %Flop.Filter{field: :name, op: :==, value: \"Joe\"}}\n\n    iex> fetch([%Flop.Filter{field: :name, op: :==, value: \"Joe\"}], :age)\n    :error\n\n    iex> fetch(\n    ...>   [\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n    ...>   ],\n    ...>   :name\n    ...> )\n    {:ok, %Flop.Filter{field: :name, op: :==, value: \"Joe\"}}\n\n#","ref":"Flop.Filter.html#fetch/2-flop-filter-struct"},{"type":"function","title":"Map with atom keys - Flop.Filter.fetch/2","doc":"iex> fetch([%{field: :name, op: :==, value: \"Joe\"}], :name)\n    {:ok, %{field: :name, op: :==, value: \"Joe\"}}\n\n#","ref":"Flop.Filter.html#fetch/2-map-with-atom-keys"},{"type":"function","title":"Map with string keys - Flop.Filter.fetch/2","doc":"iex> fetch([%{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"}], :name)\n    {:ok, %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"}}\n\n#","ref":"Flop.Filter.html#fetch/2-map-with-string-keys"},{"type":"function","title":"Indexed map - Flop.Filter.fetch/2","doc":"iex> fetch(\n    ...>   %{0 => %{field: \"name\", op: \"==\", value: \"Joe\"}},\n    ...>   :name\n    ...> )\n    {:ok, %{field: \"name\", op: \"==\", value: \"Joe\"}}\n\n    iex> fetch(\n    ...>   %{\"0\" => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"}},\n    ...>   :name\n    ...> )\n    {:ok, %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"}}","ref":"Flop.Filter.html#fetch/2-indexed-map"},{"type":"function","title":"Flop.Filter.fetch_value/2","doc":"Fetches the first filter value for the given field and returns it in a tuple.","ref":"Flop.Filter.html#fetch_value/2"},{"type":"function","title":"Examples - Flop.Filter.fetch_value/2","doc":"#","ref":"Flop.Filter.html#fetch_value/2-examples"},{"type":"function","title":"Flop.Filter struct - Flop.Filter.fetch_value/2","doc":"iex> fetch_value([], :name)\n    :error\n\n    iex> fetch_value([%Flop.Filter{field: :name, op: :==, value: \"Joe\"}], :name)\n    {:ok, \"Joe\"}\n\n    iex> fetch_value([%Flop.Filter{field: :name, op: :==, value: \"Joe\"}], :age)\n    :error\n\n    iex> fetch_value(\n    ...>   [\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n    ...>   ],\n    ...>   :name\n    ...> )\n    {:ok, \"Joe\"}\n\n#","ref":"Flop.Filter.html#fetch_value/2-flop-filter-struct"},{"type":"function","title":"Map with atom keys - Flop.Filter.fetch_value/2","doc":"iex> fetch_value([%{field: :name, op: :==, value: \"Joe\"}], :name)\n    {:ok, \"Joe\"}\n\n#","ref":"Flop.Filter.html#fetch_value/2-map-with-atom-keys"},{"type":"function","title":"Map with string keys - Flop.Filter.fetch_value/2","doc":"iex> fetch_value(\n    ...>   [%{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"}],\n    ...>   :name\n    ...> )\n    {:ok, \"Joe\"}\n\n#","ref":"Flop.Filter.html#fetch_value/2-map-with-string-keys"},{"type":"function","title":"Indexed map - Flop.Filter.fetch_value/2","doc":"iex> fetch_value(\n    ...>   %{0 => %{field: \"name\", op: \"==\", value: \"Joe\"}},\n    ...>   :name\n    ...> )\n    {:ok, \"Joe\"}\n\n    iex> fetch_value(\n    ...>   %{\"0\" => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"}},\n    ...>   :name\n    ...> )\n    {:ok, \"Joe\"}","ref":"Flop.Filter.html#fetch_value/2-indexed-map"},{"type":"function","title":"Flop.Filter.get/2","doc":"Returns the first filter for the given field.","ref":"Flop.Filter.html#get/2"},{"type":"function","title":"Examples - Flop.Filter.get/2","doc":"#","ref":"Flop.Filter.html#get/2-examples"},{"type":"function","title":"Flop.Filter struct - Flop.Filter.get/2","doc":"iex> get([], :name)\n    nil\n\n    iex> get([%Flop.Filter{field: :name, op: :==, value: \"Joe\"}], :name)\n    %Flop.Filter{field: :name, op: :==, value: \"Joe\"}\n\n    iex> get([%Flop.Filter{field: :name, op: :==, value: \"Joe\"}], :age)\n    nil\n\n    iex> get(\n    ...>   [\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n    ...>   ],\n    ...>   :name\n    ...> )\n    %Flop.Filter{field: :name, op: :==, value: \"Joe\"}\n\n#","ref":"Flop.Filter.html#get/2-flop-filter-struct"},{"type":"function","title":"Map with atom keys - Flop.Filter.get/2","doc":"iex> get([%{field: :name, op: :==, value: \"Joe\"}], :name)\n    %{field: :name, op: :==, value: \"Joe\"}\n\n#","ref":"Flop.Filter.html#get/2-map-with-atom-keys"},{"type":"function","title":"Map with string keys - Flop.Filter.get/2","doc":"iex> get([%{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"}], :name)\n    %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"}\n\n    iex> get([%{\"field\" => :name, \"op\" => \"==\", \"value\" => \"Joe\"}], :name)\n    %{\"field\" => :name, \"op\" => \"==\", \"value\" => \"Joe\"}\n\n#","ref":"Flop.Filter.html#get/2-map-with-string-keys"},{"type":"function","title":"Indexed map - Flop.Filter.get/2","doc":"iex> get(\n    ...>   %{0 => %{field: \"name\", op: \"==\", value: \"Joe\"}},\n    ...>   :name\n    ...> )\n    %{field: \"name\", op: \"==\", value: \"Joe\"}\n\n    iex> get(\n    ...>   %{0 => %{field: :name, op: \"==\", value: \"Joe\"}},\n    ...>   :name\n    ...> )\n    %{field: :name, op: \"==\", value: \"Joe\"}\n\n    iex> get(\n    ...>   %{\"0\" => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"}},\n    ...>   :name\n    ...> )\n    %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"}\n\n    iex> get(\n    ...>   %{\"0\" => %{\"field\" => :name, \"op\" => \"==\", \"value\" => \"Joe\"}},\n    ...>   :name\n    ...> )\n    %{\"field\" => :name, \"op\" => \"==\", \"value\" => \"Joe\"}","ref":"Flop.Filter.html#get/2-indexed-map"},{"type":"function","title":"Flop.Filter.get_all/2","doc":"Returns the all filters for the given field.","ref":"Flop.Filter.html#get_all/2"},{"type":"function","title":"Examples - Flop.Filter.get_all/2","doc":"#","ref":"Flop.Filter.html#get_all/2-examples"},{"type":"function","title":"Flop.Filter struct - Flop.Filter.get_all/2","doc":"iex> get_all([], :name)\n    []\n\n    iex> get_all([%Flop.Filter{field: :name, op: :==, value: \"Joe\"}], :name)\n    [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}]\n\n    iex> get_all([%Flop.Filter{field: :name, op: :==, value: \"Joe\"}], :age)\n    []\n\n    iex> get_all(\n    ...>   [\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n    ...>     %Flop.Filter{field: :age, op: :>, value: 8},\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n    ...>   ],\n    ...>   :name\n    ...> )\n    [\n      %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n      %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n    ]\n\n#","ref":"Flop.Filter.html#get_all/2-flop-filter-struct"},{"type":"function","title":"Map with atom keys - Flop.Filter.get_all/2","doc":"iex> get_all(\n    ...>   [\n    ...>     %{field: :name, op: :==, value: \"Joe\"},\n    ...>     %{field: :age, op: :>, value: 8},\n    ...>     %{field: :name, op: :==, value: \"Jim\"}\n    ...>   ],\n    ...>   :name\n    ...> )\n    [\n      %{field: :name, op: :==, value: \"Joe\"},\n      %{field: :name, op: :==, value: \"Jim\"}\n    ]\n\n#","ref":"Flop.Filter.html#get_all/2-map-with-atom-keys"},{"type":"function","title":"Map with string keys - Flop.Filter.get_all/2","doc":"iex> get_all(\n    ...>   [\n    ...>     %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n    ...>     %{\"field\" => \"age\", \"op\" => \">\", \"value\" => 8},\n    ...>     %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Jim\"}\n    ...>   ],\n    ...>   :name\n    ...> )\n    [\n      %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n      %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Jim\"}\n    ]\n\n#","ref":"Flop.Filter.html#get_all/2-map-with-string-keys"},{"type":"function","title":"Indexed map - Flop.Filter.get_all/2","doc":"iex> get_all(\n    ...>   %{\n    ...>     0 => %{field: \"name\", op: \"==\", value: \"Joe\"},\n    ...>     1 => %{field: \"age\", op: \">\", value: 8},\n    ...>     2 => %{field: \"name\", op: \"==\", value: \"Jim\"}\n    ...>   },\n    ...>   :name\n    ...> )\n    [\n      %{field: \"name\", op: \"==\", value: \"Joe\"},\n      %{field: \"name\", op: \"==\", value: \"Jim\"}\n    ]\n\n    iex> get_all(\n    ...>   %{\n    ...>     \"0\" => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n    ...>     \"1\" => %{\"field\" => \"age\", \"op\" => \">\", \"value\" => 8},\n    ...>     \"2\" => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Jim\"}\n    ...>   },\n    ...>   :name\n    ...> )\n    [\n      %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n      %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Jim\"}\n    ]","ref":"Flop.Filter.html#get_all/2-indexed-map"},{"type":"function","title":"Flop.Filter.get_value/2","doc":"Returns the first filter value for the given field.","ref":"Flop.Filter.html#get_value/2"},{"type":"function","title":"Examples - Flop.Filter.get_value/2","doc":"#","ref":"Flop.Filter.html#get_value/2-examples"},{"type":"function","title":"Flop.Filter struct - Flop.Filter.get_value/2","doc":"iex> get_value([], :name)\n    nil\n\n    iex> get_value([%Flop.Filter{field: :name, op: :==, value: \"Joe\"}], :name)\n    \"Joe\"\n\n    iex> get_value([%Flop.Filter{field: :name, op: :==, value: \"Joe\"}], :age)\n    nil\n\n    iex> get_value(\n    ...>   [\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n    ...>   ],\n    ...>   :name\n    ...> )\n    \"Joe\"\n\n    iex> get_value([%Flop.Filter{field: :ok, op: :empty, value: false}], :ok)\n    false\n\n#","ref":"Flop.Filter.html#get_value/2-flop-filter-struct"},{"type":"function","title":"Map with atom keys - Flop.Filter.get_value/2","doc":"iex> get_value([%{field: :name, op: :==, value: \"Joe\"}], :name)\n    \"Joe\"\n\n#","ref":"Flop.Filter.html#get_value/2-map-with-atom-keys"},{"type":"function","title":"Map with string keys - Flop.Filter.get_value/2","doc":"iex> get_value(\n    ...>   [%{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"}],\n    ...>   :name\n    ...> )\n    \"Joe\"\n\n#","ref":"Flop.Filter.html#get_value/2-map-with-string-keys"},{"type":"function","title":"Indexed map - Flop.Filter.get_value/2","doc":"iex> get_value(\n    ...>   %{0 => %{field: \"name\", op: \"==\", value: \"Joe\"}},\n    ...>   :name\n    ...> )\n    \"Joe\"\n\n    iex> get_value(\n    ...>   %{\"0\" => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"}},\n    ...>   :name\n    ...> )\n    \"Joe\"","ref":"Flop.Filter.html#get_value/2-indexed-map"},{"type":"function","title":"Flop.Filter.new/2","doc":"Creates a list of filters from an enumerable.\n\nThe default operator is `:==`.\n\n    iex> filters = new(%{name: \"George\", age: 8})\n    iex> Enum.sort(filters)\n    [\n      %Flop.Filter{field: :age, op: :==, value: 8},\n      %Flop.Filter{field: :name, op: :==, value: \"George\"}\n    ]\n\n    iex> filters = new([name: \"George\", age: 8])\n    iex> Enum.sort(filters)\n    [\n      %Flop.Filter{field: :age, op: :==, value: 8},\n      %Flop.Filter{field: :name, op: :==, value: \"George\"}\n    ]\n\nYou can optionally pass a mapping from field names to operators as a map\nwith atom keys.\n\n    iex> filters = new(\n    ...>   %{name: \"George\", age: 8},\n    ...>   operators: %{name: :ilike_and}\n    ...> )\n    iex> Enum.sort(filters)\n    [\n      %Flop.Filter{field: :age, op: :==, value: 8},\n      %Flop.Filter{field: :name, op: :ilike_and, value: \"George\"}\n    ]\n\nYou can also pass a map to rename fields.\n\n    iex> filters = new(\n    ...>   %{s: \"George\", age: 8},\n    ...>   rename: %{s: :name}\n    ...> )\n    iex> Enum.sort(filters)\n    [\n      %Flop.Filter{field: :age, op: :==, value: 8},\n      %Flop.Filter{field: :name, op: :==, value: \"George\"}\n    ]\n\n    iex> filters = new(\n    ...>   %{s: \"George\", cat: true},\n    ...>   rename: %{s: :name, cat: :dog}\n    ...> )\n    iex> Enum.sort(filters)\n    [\n      %Flop.Filter{field: :dog, op: :==, value: true},\n      %Flop.Filter{field: :name, op: :==, value: \"George\"}\n    ]\n\nIf both a rename option and an operator are set for a field, the operator\noption needs to use the new field name.\n\n    iex> new(\n    ...>   %{n: \"George\"},\n    ...>   rename: %{n: :name},\n    ...>   operators: %{name: :ilike_or}\n    ...> )\n    [%Flop.Filter{field: :name, op: :ilike_or, value: \"George\"}]\n\nIf the enumerable uses string keys as field names, the function attempts to\nconvert them to existing atoms. If the atom does not exist, the filter is\nremoved from the list.\n\n    iex> new(%{\"name\" => \"George\", \"age\" => 8})\n    [\n      %Flop.Filter{field: :age, op: :==, value: 8},\n      %Flop.Filter{field: :name, op: :==, value: \"George\"}\n    ]\n\n    iex> new(%{\"name\" => \"George\", \"doesnotexist\" => 8})\n    [\n      %Flop.Filter{field: :name, op: :==, value: \"George\"}\n    ]\n\nOther key types are also removed.\n\n    iex> new(%{\"name\" => \"George\", 2 => 8})\n    [\n      %Flop.Filter{field: :name, op: :==, value: \"George\"}\n    ]","ref":"Flop.Filter.html#new/2"},{"type":"function","title":"Flop.Filter.pop/3","doc":"Returns the first filter for the given field and removes all other filters for\nthe same field from the filter list.\n\nReturns a tuple with the first matching filter for `key` and the remaining\nfilter list. If there is no filter for the field in the list, the default\nvalue is returned as the first tuple element.\n\nSee also `Flop.Filter.pop_first/3`, `Flop.Filter.pop_value/3` and\n`Flop.Filter.pop_first_value/3`.","ref":"Flop.Filter.html#pop/3"},{"type":"function","title":"Examples - Flop.Filter.pop/3","doc":"#","ref":"Flop.Filter.html#pop/3-examples"},{"type":"function","title":"Flop.Filter struct - Flop.Filter.pop/3","doc":"iex> pop([%Flop.Filter{field: :name, op: :==, value: \"Joe\"}], :name)\n    {%Flop.Filter{field: :name, op: :==, value: \"Joe\"}, []}\n\n    iex> pop([%Flop.Filter{field: :name, op: :==, value: \"Joe\"}], :age)\n    {nil, [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}]}\n\n    iex> pop(\n    ...>   [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}],\n    ...>   :age,\n    ...>   %Flop.Filter{field: :age, op: :>, value: 8}\n    ...> )\n    {\n      %Flop.Filter{field: :age, op: :>, value: 8},\n      [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}]\n    }\n\n    iex> pop(\n    ...>   [\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n    ...>     %Flop.Filter{field: :age, op: :>, value: 8},\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Jim\"},\n    ...>   ],\n    ...>   :name\n    ...> )\n    {\n      %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n      [%Flop.Filter{field: :age, op: :>, value: 8}]\n    }\n\n#","ref":"Flop.Filter.html#pop/3-flop-filter-struct"},{"type":"function","title":"Map with atom keys - Flop.Filter.pop/3","doc":"iex> pop([%{field: :name, op: :==, value: \"Joe\"}], :name)\n    {%{field: :name, op: :==, value: \"Joe\"}, []}\n\n#","ref":"Flop.Filter.html#pop/3-map-with-atom-keys"},{"type":"function","title":"Map with string keys - Flop.Filter.pop/3","doc":"iex> pop([%{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"}], :name)\n    {%{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"}, []}\n\n#","ref":"Flop.Filter.html#pop/3-map-with-string-keys"},{"type":"function","title":"Indexed map - Flop.Filter.pop/3","doc":"Filters passed as an indexed map will be converted to a list, even if no\nmatching filter exists.\n\n    iex> pop(\n    ...>   %{\n    ...>     0 => %{field: \"name\", op: \"==\", value: \"Joe\"},\n    ...>     1 => %{field: \"age\", op: \">\", value: \"8\"},\n    ...>     2 => %{field: \"name\", op: \"==\", value: \"Jim\"},\n    ...>   },\n    ...>   :name\n    ...> )\n    {\n      %{field: \"name\", op: \"==\", value: \"Joe\"},\n      [%{field: \"age\", op: \">\", value: \"8\"}]\n    }\n\n    iex> pop(\n    ...>   %{\n    ...>     0 => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n    ...>     1 => %{\"field\" => \"age\", \"op\" => \">\", \"value\" => \"8\"},\n    ...>     2 => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Jim\"},\n    ...>   },\n    ...>   :name\n    ...> )\n    {\n      %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n      [%{\"field\" => \"age\", \"op\" => \">\", \"value\" => \"8\"}]\n    }","ref":"Flop.Filter.html#pop/3-indexed-map"},{"type":"function","title":"Flop.Filter.pop_first/3","doc":"Returns the first filter for the given field and a filter list with all\nremaining filters.\n\nReturns a tuple with the first matching filter for `key` and the\nremaining filter list. If there is no filter for the field in the list, the\ndefault value is returned as the first tuple element.\n\nSee also `Flop.Filter.pop/3`, `Flop.Filter.pop_value/3` and\n`Flop.Filter.pop_first_value/3`.","ref":"Flop.Filter.html#pop_first/3"},{"type":"function","title":"Examples - Flop.Filter.pop_first/3","doc":"#","ref":"Flop.Filter.html#pop_first/3-examples"},{"type":"function","title":"Flop.Filter struct - Flop.Filter.pop_first/3","doc":"iex> pop_first([%Flop.Filter{field: :name, op: :==, value: \"Joe\"}], :name)\n    {%Flop.Filter{field: :name, op: :==, value: \"Joe\"}, []}\n\n    iex> pop_first([%Flop.Filter{field: :name, op: :==, value: \"Joe\"}], :age)\n    {nil, [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}]}\n\n    iex> pop_first(\n    ...>   [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}],\n    ...>   :age,\n    ...>   %Flop.Filter{field: :age, op: :>, value: 8}\n    ...> )\n    {\n      %Flop.Filter{field: :age, op: :>, value: 8},\n      [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}]\n    }\n\n    iex> pop_first(\n    ...>   [\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n    ...>     %Flop.Filter{field: :age, op: :>, value: 8},\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Jim\"},\n    ...>   ],\n    ...>   :name\n    ...> )\n    {\n      %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n      [\n        %Flop.Filter{field: :age, op: :>, value: 8},\n        %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n      ]\n    }\n\n#","ref":"Flop.Filter.html#pop_first/3-flop-filter-struct"},{"type":"function","title":"Map with atom keys - Flop.Filter.pop_first/3","doc":"iex> pop_first([%{field: :name, op: :==, value: \"Joe\"}], :name)\n    {%{field: :name, op: :==, value: \"Joe\"}, []}\n\n#","ref":"Flop.Filter.html#pop_first/3-map-with-atom-keys"},{"type":"function","title":"Map with string keys - Flop.Filter.pop_first/3","doc":"iex> pop_first(\n    ...>   [%{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"}],\n    ...>   :name\n    ...> )\n    {%{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"}, []}\n\n#","ref":"Flop.Filter.html#pop_first/3-map-with-string-keys"},{"type":"function","title":"Indexed map - Flop.Filter.pop_first/3","doc":"Filters passed as an indexed map will be converted to a list, even if no\nmatching filter exists.\n\n    iex> pop_first(\n    ...>   %{\n    ...>     0 => %{field: \"name\", op: \"==\", value: \"Joe\"},\n    ...>     1 => %{field: \"age\", op: \">\", value: 8},\n    ...>     2 => %{field: \"name\", op: \"==\", value: \"Jim\"},\n    ...>   },\n    ...>   :name\n    ...> )\n    {\n      %{field: \"name\", op: \"==\", value: \"Joe\"},\n      [\n        %{field: \"age\", op: \">\", value: 8},\n        %{field: \"name\", op: \"==\", value: \"Jim\"}\n      ]\n    }\n\n    iex> pop_first(\n    ...>   %{\n    ...>     \"0\" => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n    ...>     \"1\" => %{\"field\" => \"age\", \"op\" => \">\", \"value\" => 8},\n    ...>     \"2\" => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Jim\"},\n    ...>   },\n    ...>   :name\n    ...> )\n    {\n      %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n      [\n        %{\"field\" => \"age\", \"op\" => \">\", \"value\" => 8},\n        %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Jim\"}\n      ]\n    }","ref":"Flop.Filter.html#pop_first/3-indexed-map"},{"type":"function","title":"Flop.Filter.pop_first_value/3","doc":"Returns the first filter for the given field and a filter list with all\nremaining filters.\n\nReturns a tuple with the first matching filter value for `key` and the\nremaining filter list. If there is no filter for the field in the list, the\ndefault value is returned as the first tuple element.\n\nSee also `Flop.Filter.pop/3`, `Flop.Filter.pop_value/3` and\n`Flop.Filter.pop_first/3`.","ref":"Flop.Filter.html#pop_first_value/3"},{"type":"function","title":"Examples - Flop.Filter.pop_first_value/3","doc":"#","ref":"Flop.Filter.html#pop_first_value/3-examples"},{"type":"function","title":"Flop.Filter struct - Flop.Filter.pop_first_value/3","doc":"iex> pop_first_value(\n    ...>   [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}],\n    ...>   :name\n    ...> )\n    {\"Joe\", []}\n\n    iex> pop_first_value(\n    ...>   [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}],\n    ...>   :age\n    ...> )\n    {nil, [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}]}\n\n    iex> pop_first_value(\n    ...>   [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}],\n    ...>   :age,\n    ...>   8\n    ...> )\n    {8, [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}]}\n\n    iex> pop_first_value(\n    ...>   [\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n    ...>     %Flop.Filter{field: :age, op: :>, value: 8},\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Jim\"},\n    ...>   ],\n    ...>   :name\n    ...> )\n    {\n      \"Joe\",\n      [\n        %Flop.Filter{field: :age, op: :>, value: 8},\n        %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n      ]\n    }\n\n#","ref":"Flop.Filter.html#pop_first_value/3-flop-filter-struct"},{"type":"function","title":"Map with atom keys - Flop.Filter.pop_first_value/3","doc":"iex> pop_first_value([%{field: :name, op: :==, value: \"Joe\"}], :name)\n    {\"Joe\", []}\n\n#","ref":"Flop.Filter.html#pop_first_value/3-map-with-atom-keys"},{"type":"function","title":"Map with string keys - Flop.Filter.pop_first_value/3","doc":"iex> pop_first_value(\n    ...>   [%{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"}],\n    ...>   :name\n    ...> )\n    {\"Joe\", []}\n\n#","ref":"Flop.Filter.html#pop_first_value/3-map-with-string-keys"},{"type":"function","title":"Indexed map - Flop.Filter.pop_first_value/3","doc":"Filters passed as an indexed map will be converted to a list, even if no\nmatching filter exists.\n\n    iex> pop_first_value(\n    ...>   %{\n    ...>     0 => %{field: \"name\", op: \"==\", value: \"Joe\"},\n    ...>     1 => %{field: \"age\", op: \">\", value: 8},\n    ...>     2 => %{field: \"name\", op: \"==\", value: \"Jim\"},\n    ...>   },\n    ...>   :name\n    ...> )\n    {\n      \"Joe\",\n      [\n        %{field: \"age\", op: \">\", value: 8},\n        %{field: \"name\", op: \"==\", value: \"Jim\"}\n      ]\n    }\n\n    iex> pop_first_value(\n    ...>   %{\n    ...>     \"0\" => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n    ...>     \"1\" => %{\"field\" => \"age\", \"op\" => \">\", \"value\" => 8},\n    ...>     \"2\" => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Jim\"},\n    ...>   },\n    ...>   :name\n    ...> )\n    {\n      \"Joe\",\n      [\n        %{\"field\" => \"age\", \"op\" => \">\", \"value\" => 8},\n        %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Jim\"}\n      ]\n    }","ref":"Flop.Filter.html#pop_first_value/3-indexed-map"},{"type":"function","title":"Flop.Filter.pop_value/3","doc":"Returns the first filter value for the given field and removes all other\nfilters for the same field from the filter list.\n\nReturns a tuple with the value of the first matching filter for `key` and the\nremaining filter list. If there is no filter for the field in the list, the\ndefault value is returned as the first tuple element.\n\nSee also `Flop.Filter.pop/3`, `Flop.Filter.pop_first/3` and\n`Flop.Filter.pop_first_value/3`","ref":"Flop.Filter.html#pop_value/3"},{"type":"function","title":"Examples - Flop.Filter.pop_value/3","doc":"#","ref":"Flop.Filter.html#pop_value/3-examples"},{"type":"function","title":"Flop.Filter struct - Flop.Filter.pop_value/3","doc":"iex> pop_value([%Flop.Filter{field: :name, op: :==, value: \"Joe\"}], :name)\n    {\"Joe\", []}\n\n    iex> pop_value([%Flop.Filter{field: :name, op: :==, value: \"Joe\"}], :age)\n    {nil, [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}]}\n\n    iex> pop_value(\n    ...>   [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}],\n    ...>   :age,\n    ...>   8\n    ...> )\n    {8, [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}]}\n\n    iex> pop_value(\n    ...>   [\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n    ...>     %Flop.Filter{field: :age, op: :>, value: 8},\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Jim\"},\n    ...>   ],\n    ...>   :name\n    ...> )\n    {\"Joe\", [%Flop.Filter{field: :age, op: :>, value: 8}]}\n\n#","ref":"Flop.Filter.html#pop_value/3-flop-filter-struct"},{"type":"function","title":"Map with atom keys - Flop.Filter.pop_value/3","doc":"iex> pop_value([%{field: :name, op: :==, value: \"Joe\"}], :name)\n    {\"Joe\", []}\n\n#","ref":"Flop.Filter.html#pop_value/3-map-with-atom-keys"},{"type":"function","title":"Map with string keys - Flop.Filter.pop_value/3","doc":"iex> pop_value([%{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"}], :name)\n    {\"Joe\", []}\n\n#","ref":"Flop.Filter.html#pop_value/3-map-with-string-keys"},{"type":"function","title":"Indexed map - Flop.Filter.pop_value/3","doc":"Filters passed as an indexed map will be converted to a list, even if no\nmatching filter exists.\n\n    iex> pop_value(\n    ...>   %{\n    ...>     0 => %{field: \"name\", op: \"==\", value: \"Joe\"},\n    ...>     1 => %{field: \"age\", op: \">\", value: \"8\"},\n    ...>     2 => %{field: \"name\", op: \"==\", value: \"Jim\"},\n    ...>   },\n    ...>   :name\n    ...> )\n    {\"Joe\", [%{field: \"age\", op: \">\", value: \"8\"}]}\n\n    iex> pop_value(\n    ...>   %{\n    ...>     0 => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n    ...>     1 => %{\"field\" => \"age\", \"op\" => \">\", \"value\" => \"8\"},\n    ...>     2 => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Jim\"},\n    ...>   },\n    ...>   :name\n    ...> )\n    {\"Joe\", [%{\"field\" => \"age\", \"op\" => \">\", \"value\" => \"8\"}]}","ref":"Flop.Filter.html#pop_value/3-indexed-map"},{"type":"function","title":"Flop.Filter.put/2","doc":"Adds the given filter to the filter list and removes all existing filters for\nthe same field from the list.","ref":"Flop.Filter.html#put/2"},{"type":"function","title":"Examples - Flop.Filter.put/2","doc":"iex> put(\n    ...>   [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}],\n    ...>   %Flop.Filter{field: :age, op: :>, value: 8}\n    ...> )\n    [\n      %Flop.Filter{field: :age, op: :>, value: 8},\n      %Flop.Filter{field: :name, op: :==, value: \"Joe\"}\n    ]\n\n    iex> put(\n    ...>   [\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n    ...>     %Flop.Filter{field: :age, op: :>, value: 8},\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n    ...>   ],\n    ...>   %Flop.Filter{field: :name, op: :==, value: \"Jane\"}\n    ...> )\n    [\n      %Flop.Filter{field: :name, op: :==, value: \"Jane\"},\n      %Flop.Filter{field: :age, op: :>, value: 8}\n    ]","ref":"Flop.Filter.html#put/2-examples"},{"type":"function","title":"Flop.Filter.put_new/2","doc":"Adds the given filter to the filter list only if no filter for the field\nexists yet.","ref":"Flop.Filter.html#put_new/2"},{"type":"function","title":"Examples - Flop.Filter.put_new/2","doc":"iex> put_new(\n    ...>   [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}],\n    ...>   %Flop.Filter{field: :age, op: :>, value: 8}\n    ...> )\n    [\n      %Flop.Filter{field: :age, op: :>, value: 8},\n      %Flop.Filter{field: :name, op: :==, value: \"Joe\"}\n    ]\n\n    iex> put_new(\n    ...>   [\n    ...>     %Flop.Filter{field: :age, op: :>, value: 8},\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n    ...>   ],\n    ...>   %Flop.Filter{field: :name, op: :==, value: \"Jane\"}\n    ...> )\n    [\n      %Flop.Filter{field: :age, op: :>, value: 8},\n      %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n    ]","ref":"Flop.Filter.html#put_new/2-examples"},{"type":"function","title":"Flop.Filter.put_new_value/4","doc":"Adds the given filter value to the filter list only if no filter for the field\nexists yet.","ref":"Flop.Filter.html#put_new_value/4"},{"type":"function","title":"Examples - Flop.Filter.put_new_value/4","doc":"iex> put_new_value(\n    ...>   [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}],\n    ...>   :age,\n    ...>   8,\n    ...>   :>\n    ...> )\n    [\n      %Flop.Filter{field: :age, op: :>, value: 8},\n      %Flop.Filter{field: :name, op: :==, value: \"Joe\"}\n    ]\n\n    iex> put_new_value(\n    ...>   [\n    ...>     %Flop.Filter{field: :age, op: :>, value: 8},\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n    ...>   ],\n    ...>   :name,\n    ...>   \"Jane\"\n    ...> )\n    [\n      %Flop.Filter{field: :age, op: :>, value: 8},\n      %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n    ]","ref":"Flop.Filter.html#put_new_value/4-examples"},{"type":"function","title":"Flop.Filter.put_value/4","doc":"Adds the given filter value to the filter list and removes all existing\nfilters for the same field from the list.","ref":"Flop.Filter.html#put_value/4"},{"type":"function","title":"Examples - Flop.Filter.put_value/4","doc":"iex> put_value(\n    ...>   [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}],\n    ...>   :age,\n    ...>   8\n    ...> )\n    [\n      %Flop.Filter{field: :age, op: :==, value: 8},\n      %Flop.Filter{field: :name, op: :==, value: \"Joe\"}\n    ]\n\n    iex> put_value(\n    ...>   [%Flop.Filter{field: :name, op: :==, value: \"Joe\"}],\n    ...>   :age,\n    ...>   8,\n    ...>   :>=\n    ...> )\n    [\n      %Flop.Filter{field: :age, op: :>=, value: 8},\n      %Flop.Filter{field: :name, op: :==, value: \"Joe\"}\n    ]\n\n    iex> put_value(\n    ...>   [\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n    ...>     %Flop.Filter{field: :age, op: :>, value: 8},\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n    ...>   ],\n    ...>   :name,\n    ...>   \"Jane\"\n    ...> )\n    [\n      %Flop.Filter{field: :name, op: :==, value: \"Jane\"},\n      %Flop.Filter{field: :age, op: :>, value: 8}\n    ]","ref":"Flop.Filter.html#put_value/4-examples"},{"type":"function","title":"Flop.Filter.take/2","doc":"Takes all filters for the given fields from a filter list.","ref":"Flop.Filter.html#take/2"},{"type":"function","title":"Examples - Flop.Filter.take/2","doc":"#","ref":"Flop.Filter.html#take/2-examples"},{"type":"function","title":"Flop.Filter struct - Flop.Filter.take/2","doc":"iex> take(\n    ...>   [\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n    ...>     %Flop.Filter{field: :age, op: :>, value: 8},\n    ...>     %Flop.Filter{field: :color, op: :==, value: \"blue\"},\n    ...>     %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n    ...>   ],\n    ...>   [:name, :color]\n    ...> )\n    [\n      %Flop.Filter{field: :name, op: :==, value: \"Joe\"},\n      %Flop.Filter{field: :color, op: :==, value: \"blue\"},\n      %Flop.Filter{field: :name, op: :==, value: \"Jim\"}\n    ]\n\n#","ref":"Flop.Filter.html#take/2-flop-filter-struct"},{"type":"function","title":"Map with atom keys - Flop.Filter.take/2","doc":"iex> take(\n    ...>   [\n    ...>     %{field: :name, op: :==, value: \"Joe\"},\n    ...>     %{field: :age, op: :>, value: 8},\n    ...>     %{field: :color, op: :==, value: \"blue\"}\n    ...>   ],\n    ...>   [:name, :color]\n    ...> )\n    [\n      %{field: :name, op: :==, value: \"Joe\"},\n      %{field: :color, op: :==, value: \"blue\"}\n    ]\n\n#","ref":"Flop.Filter.html#take/2-map-with-atom-keys"},{"type":"function","title":"Map with string keys - Flop.Filter.take/2","doc":"iex> take(\n    ...>   [\n    ...>     %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n    ...>     %{\"field\" => \"age\", \"op\" => \">\", \"value\" => 8},\n    ...>     %{\"field\" => \"color\", \"op\" => \"==\", \"value\" => \"blue\"}\n    ...>   ],\n    ...>   [:name, :color]\n    ...> )\n    [\n      %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n      %{\"field\" => \"color\", \"op\" => \"==\", \"value\" => \"blue\"}\n    ]\n\n#","ref":"Flop.Filter.html#take/2-map-with-string-keys"},{"type":"function","title":"Indexed map - Flop.Filter.take/2","doc":"Filters passed as an indexed map will be converted to a list, even if no\nmatching filter exists.\n\n    iex> take(\n    ...>   %{\n    ...>     0 => %{field: \"name\", op: \"==\", value: \"Joe\"},\n    ...>     1 => %{field: \"age\", op: \">\", value: 8},\n    ...>     2 => %{field: \"color\", op: \"==\", value: \"blue\"}\n    ...>   },\n    ...>   [:name, :color]\n    ...> )\n    [\n      %{field: \"name\", op: \"==\", value: \"Joe\"},\n      %{field: \"color\", op: \"==\", value: \"blue\"}\n    ]\n\n    iex> take(\n    ...>   %{\n    ...>     \"0\" => %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n    ...>     \"1\" => %{\"field\" => \"age\", \"op\" => \">\", \"value\" => 8},\n    ...>     \"2\" => %{\"field\" => \"color\", \"op\" => \"==\", \"value\" => \"blue\"}\n    ...>   },\n    ...>   [:name, :color]\n    ...> )\n    [\n      %{\"field\" => \"name\", \"op\" => \"==\", \"value\" => \"Joe\"},\n      %{\"field\" => \"color\", \"op\" => \"==\", \"value\" => \"blue\"}\n    ]","ref":"Flop.Filter.html#take/2-indexed-map"},{"type":"type","title":"Flop.Filter.op/0","doc":"Represents valid filter operators.\n\n| Operator        | Value               | WHERE clause                                            |\n| :-------------- | :------------------ | ------------------------------------------------------- |\n| `:==`           | `\"Salicaceae\"`      | `WHERE column = 'Salicaceae'`                           |\n| `:!=`           | `\"Salicaceae\"`      | `WHERE column != 'Salicaceae'`                          |\n| `:=~`           | `\"cyth\"`            | `WHERE column ILIKE '%cyth%'`                           |\n| `:empty`        | `true`              | `WHERE (column IS NULL) = true`                         |\n| `:empty`        | `false`             | `WHERE (column IS NULL) = false`                        |\n| `:not_empty`    | `true`              | `WHERE (column IS NOT NULL) = true`                     |\n| `:not_empty`    | `false`             | `WHERE (column IS NOT NULL) = false`                    |\n| `: =`           | `10`                | `WHERE column >= 10`                                    |\n| `:>`            | `10`                | `WHERE column > 10`                                     |\n| `:in`           | `[\"pear\", \"plum\"]`  | `WHERE column = ANY('pear', 'plum')`                    |\n| `:not_in`       | `[\"pear\", \"plum\"]`  | `WHERE column = NOT IN('pear', 'plum')`                 |\n| `:contains`     | `\"pear\"`            | `WHERE 'pear' = ANY(column)`                            |\n| `:not_contains` | `\"pear\"`            | `WHERE 'pear' = NOT IN(column)`                         |\n| `:like`         | `\"cyth\"`            | `WHERE column LIKE '%cyth%'`                            |\n| `:not_like`     | `\"cyth\"`            | `WHERE column NOT LIKE '%cyth%'`                        |\n| `:like_and`     | `[\"Rubi\", \"Rosa\"]`  | `WHERE column LIKE '%Rubi%' AND column LIKE '%Rosa%'`   |\n| `:like_and`     | `\"Rubi Rosa\"`       | `WHERE column LIKE '%Rubi%' AND column LIKE '%Rosa%'`   |\n| `:like_or`      | `[\"Rubi\", \"Rosa\"]`  | `WHERE column LIKE '%Rubi%' OR column LIKE '%Rosa%'`    |\n| `:like_or`      | `\"Rubi Rosa\"`       | `WHERE column LIKE '%Rubi%' OR column LIKE '%Rosa%'`    |\n| `:ilike`        | `\"cyth\"`            | `WHERE column ILIKE '%cyth%'`                           |\n| `:not_ilike`    | `\"cyth\"`            | `WHERE column NOT ILIKE '%cyth%'`                       |\n| `:ilike_and`    | `[\"Rubi\", \"Rosa\"]`  | `WHERE column ILIKE '%Rubi%' AND column ILIKE '%Rosa%'` |\n| `:ilike_and`    | `\"Rubi Rosa\"`       | `WHERE column ILIKE '%Rubi%' AND column ILIKE '%Rosa%'` |\n| `:ilike_or`     | `[\"Rubi\", \"Rosa\"]`  | `WHERE column ILIKE '%Rubi%' OR column ILIKE '%Rosa%'`  |\n| `:ilike_or`     | `\"Rubi Rosa\"`       | `WHERE column ILIKE '%Rubi%' OR column ILIKE '%Rosa%'`  |\n\nThe filter operators `:empty` and `:not_empty` will regard empty arrays as\nempty values if the field is known to be an array field.\n\nThe filter operators `:ilike_and`, `:ilike_or`, `:like_and` and `:like_or`\naccept both strings and list of strings.\n\n- If the filter value is a string, it will be split at whitespace characters\n  and the segments are combined with `and` or `or`.\n- If a list of strings is passed, the individual strings are not split, and\n  the list items are combined with `and` or `or`.","ref":"Flop.Filter.html#t:op/0"},{"type":"type","title":"Flop.Filter.t/0","doc":"Represents filter query parameters.\n\n#","ref":"Flop.Filter.html#t:t/0"},{"type":"type","title":"Fields - Flop.Filter.t/0","doc":"- `field`: The field the filter is applied to. The allowed fields can be\n  restricted by deriving `Flop.Schema` in your Ecto schema.\n- `op`: The filter operator.\n- `value`: The comparison value of the filter.","ref":"Flop.Filter.html#t:t/0-fields"},{"type":"module","title":"Flop.Meta","doc":"Defines a struct for holding meta information of a query result.","ref":"Flop.Meta.html"},{"type":"function","title":"Flop.Meta.with_errors/3","doc":"Returns a `Flop.Meta` struct with the given params, errors, and opts.\n\nThis function is used internally to build error responses in case of\nvalidation errors. You can use it to add additional parameter validation.\n\nThe given parameters parameters are normalized before being added to the\nstruct. The errors have to be passed as a keyword list (same format as the\nresult of `Ecto.Changeset.traverse_errors(changeset, & &1)`).","ref":"Flop.Meta.html#with_errors/3"},{"type":"function","title":"Example - Flop.Meta.with_errors/3","doc":"In this list function, the given parameters are first validated with\n`Flop.validate/2`, which returns a `Flop` struct on success. You can then pass\nthat struct to a custom validation function, along with the original\nparameters and the opts, which both are needed to call this function.\n\n    def list_pets(%{} = params) do\n      opts = [for: Pet]\n\n      with {:ok, %Flop{} = flop}   Enum.reduce([], &validate_filter/2)\n        |> Enum.reverse()\n\n      if Enum.any?(filter_errors, &(&1 != [])) do\n        errors = [filters: filter_errors]\n        {:error, Flop.Meta.with_errors(params, errors, opts)}\n      else\n        {:ok, flop}\n      end\n    end\n\n    defp validate_filter(%Flop.Filter{field: :date, value: date}, acc)\n         when is_binary(date) do\n      date = Date.from_iso8601!(date)\n\n      if Date.compare(date, Date.utc_today()) != :lt,\n        do: [[value: [{\"date must be in the past\", []}]] | acc],\n        else: [[] | acc]\n    end\n\n    defp validate_filter(%Flop.Filter{}, acc), do: [[] | acc]","ref":"Flop.Meta.html#with_errors/3-example"},{"type":"type","title":"Flop.Meta.t/0","doc":"Meta information for a query result.\n\n- `:flop` - The `Flop` struct used in the query.\n- `:schema` - The schema module passed as `for` option.\n- `:backend` - The backend module if the query was made using a module with\n  `use Flop`.\n- `:current_offset` - The `:offset` value used in the query when using\n  offset-based pagination or a derived value when using page-based pagination.\n  Always `nil` when using cursor-based pagination.\n- `:current_page` - The `:page` value used in the query when using page-based\n  pagination or a derived value when using offset-based pagination. Note that\n  the value will be rounded if the offset lies between pages. Always `nil`\n  when using cursor-based pagination.\n- `:errors` - Any validation errors that occurred. The format is the same as\n  the result of `Ecto.Changeset.traverse_errors(changeset, & &1)`.\n- `:previous_offset`, `:next_offset`, `:previous_page`, `:next_page` - Values\n  based on `:current_page` and `:current_offset`/`page_size`. Always `nil`\n  when using cursor-based pagination.\n- `:start_cursor`, `:end_cursor` - The cursors of the first and last record\n  in the result set. Only set when using cursor-based pagination with\n  `:first`/`:after` or `:last`/`:before`.\n- `:has_previous_page?`, `:has_next_page?` - Set in all pagination types.\n  Note that `:has_previous_page?` is always `true` when using cursor-based\n  pagination with `:first` and `:after` is set; likewise, `:has_next_page?` is\n  always `true` when using cursor-based pagination with `:before` and `:last`\n  is set.\n- `:page_size` - The page size or limit of the query. Set to the `:first`\n  or `:last` parameter when using cursor-based pagination.\n- `:params` - The original, unvalidated params that were passed. Only set\n  if validation errors occurred.\n- `:total_count` - The total count of records for the given query. Always\n  `nil` when using cursor-based pagination.\n- `:total_pages` - The total page count based on the total record count and\n  the page size. Always `nil` when using cursor-based pagination.","ref":"Flop.Meta.html#t:t/0"},{"type":"module","title":"Flop.Relay","doc":"Helpers to turn query results into Relay formats.","ref":"Flop.Relay.html"},{"type":"function","title":"Flop.Relay.connection_from_result/2","doc":"Takes the query results returned by `Flop.run/3`, `Flop.validate_and_run/3`\nor `Flop.validate_and_run!/3` and turns them into the Relay connection\nformat.","ref":"Flop.Relay.html#connection_from_result/2"},{"type":"function","title":"Example - Flop.Relay.connection_from_result/2","doc":"iex> flop = %Flop{order_by: [:name]}\n    iex> meta = %Flop.Meta{flop: flop, start_cursor: \"a\", end_cursor: \"b\"}\n    iex> result = {[%MyApp.Fruit{name: \"Apple\", family: \"Rosaceae\"}], meta}\n    iex> Flop.Relay.connection_from_result(result)\n    %{\n      edges: [\n        %{\n          cursor: \"g3QAAAABdwRuYW1lbQAAAAVBcHBsZQ==\",\n          node: %MyApp.Fruit{family: \"Rosaceae\", id: nil, name: \"Apple\"}\n        }\n      ],\n      page_info: %{\n        end_cursor: \"b\",\n        has_next_page: false,\n        has_previous_page: false,\n        start_cursor: \"a\"\n      }\n    }\n\nSee `Flop.Relay.edges_from_result/2` for an example of adding additional\nfields to the edge.","ref":"Flop.Relay.html#connection_from_result/2-example"},{"type":"function","title":"Options - Flop.Relay.connection_from_result/2","doc":"- `:cursor_value_func`: 2-arity function that takes an item from the query\n  result and the `order_by` fields and returns the unencoded cursor value.","ref":"Flop.Relay.html#connection_from_result/2-options"},{"type":"function","title":"Flop.Relay.edges_from_result/2","doc":"Turns a list of query results into Relay edges.","ref":"Flop.Relay.html#edges_from_result/2"},{"type":"function","title":"Simple queries - Flop.Relay.edges_from_result/2","doc":"If your query returns a list of maps or structs, the function will return\nthe a list of edges with `:cursor` and `:node` as only fields.\n\n    iex> flop = %Flop{order_by: [:name]}\n    iex> meta = %Flop.Meta{flop: flop}\n    iex> result = {[%MyApp.Fruit{name: \"Apple\", family: \"Rosaceae\"}], meta}\n    iex> Flop.Relay.edges_from_result(result)\n    [\n      %{\n        cursor: \"g3QAAAABdwRuYW1lbQAAAAVBcHBsZQ==\",\n        node: %MyApp.Fruit{name: \"Apple\", family: \"Rosaceae\"}\n      }\n    ]","ref":"Flop.Relay.html#edges_from_result/2-simple-queries"},{"type":"function","title":"Supplying additional edge information - Flop.Relay.edges_from_result/2","doc":"If the query result is a list of 2-tuples, this is interpreted as a tuple\nof the node information and the edge information. For example, if you have a\nquery like this:\n\n    Group\n    |> where([g], g.id == ^group_id)\n    |> join(:left, [g], m in assoc(g, :members))\n    |> select([g, m], {m, map(m, [:role])})\n\nThen your query result looks something like:\n\n    [{%Member{id: 242, name: \"Carl\"}, %{role: :owner}}]\n\nIn this case, the members are the nodes, and the maps with the roles is seen\nas edge information.\n\n    [\n      %{\n        cursor: \"AE98RNSTNGN\",\n        node: %Member{id: 242, name: \"Carl\"},\n        role: :owner\n      }\n    ]\n\nNote that in this case, the whole tuple will be passed to the cursor value\nfunction, so that the cursor can be based on both node and edge fields.\n\nHere's an example with fruit that overrides the cursor value function:\n\n    iex> flop = %Flop{order_by: [:name]}\n    iex> meta = %Flop.Meta{flop: flop}\n    iex> items = [{%MyApp.Fruit{name: \"Apple\"}, %{preparation:  :grated}}]\n    iex> func = fn {fruit, _edge}, order_by -> Map.take(fruit, order_by) end\n    iex> Flop.Relay.edges_from_result(\n    ...>   {items, meta},\n    ...>   cursor_value_func: func\n    ...> )\n    [\n      %{\n        cursor: \"g3QAAAABdwRuYW1lbQAAAAVBcHBsZQ==\",\n        node: %MyApp.Fruit{name: \"Apple\"},\n        preparation: :grated\n      }\n    ]","ref":"Flop.Relay.html#edges_from_result/2-supplying-additional-edge-information"},{"type":"function","title":"Options - Flop.Relay.edges_from_result/2","doc":"- `:cursor_value_func`: 2-arity function that takes an item from the query\n  result and the `order_by` fields and returns the unencoded cursor value.","ref":"Flop.Relay.html#edges_from_result/2-options"},{"type":"function","title":"Flop.Relay.page_info_from_meta/1","doc":"Takes a `Flop.Meta` struct and returns a map with the Relay page info.","ref":"Flop.Relay.html#page_info_from_meta/1"},{"type":"function","title":"Example - Flop.Relay.page_info_from_meta/1","doc":"iex> Flop.Relay.page_info_from_meta(%Flop.Meta{\n    ...>    has_previous_page?: true,\n    ...>    has_next_page?: true,\n    ...>    start_cursor: \"a\",\n    ...>    end_cursor: \"b\"\n    ...> })\n    %{\n      has_previous_page: true,\n      has_next_page: true,\n      start_cursor: \"a\",\n      end_cursor: \"b\"\n    }","ref":"Flop.Relay.html#page_info_from_meta/1-example"},{"type":"type","title":"Flop.Relay.connection/0","doc":"","ref":"Flop.Relay.html#t:connection/0"},{"type":"type","title":"Flop.Relay.edge/0","doc":"","ref":"Flop.Relay.html#t:edge/0"},{"type":"type","title":"Flop.Relay.page_info/0","doc":"","ref":"Flop.Relay.html#t:page_info/0"},{"type":"protocol","title":"Flop.Schema","doc":"Flop.Schema is a protocol that allows you to customize and set query options\nin your Ecto schemas.\n\nThis module allows you to define which fields are filterable and sortable, set\ndefault and maximum limits, specify default sort orders, restrict pagination\ntypes, and more.","ref":"Flop.Schema.html"},{"type":"protocol","title":"Usage - Flop.Schema","doc":"To utilize this protocol, derive `Flop.Schema` in your Ecto schema and define\nthe filterable and sortable fields.\n\n    defmodule MyApp.Pet do\n      use Ecto.Schema\n\n      @derive {\n        Flop.Schema,\n        filterable: [:name, :species],\n        sortable: [:name, :age]\n      }\n\n      schema \"pets\" do\n        field :name, :string\n        field :age, :integer\n        field :species, :string\n      end\n    end\n\nSee `t:option/0` for an overview of all available options.\n\n> #### `@derive Flop.Schema` {: .info}\n>\n> When you derive `Flop.Schema`, all the functions required for the\n> `Flop.Schema` protocol will be defined based on the options you set.\n\nAfter that, you can pass the module as the `:for` option to `Flop.validate/2`.\n\n    iex> Flop.validate(%Flop{order_by: [:name]}, for: MyApp.Pet)\n    {:ok,\n     %Flop{\n       filters: [],\n       limit: 50,\n       offset: nil,\n       order_by: [:name],\n       order_directions: nil,\n       page: nil,\n       page_size: nil\n     }}\n\n    iex> {:error, %Flop.Meta{} = meta} = Flop.validate(\n    ...>   %Flop{order_by: [:species]}, for: MyApp.Pet\n    ...> )\n    iex> meta.params\n    %{\"order_by\" => [:species], \"filters\" => []}\n    iex> meta.errors\n    [\n      order_by: [\n        {\"has an invalid entry\",\n         [validation: :subset, enum: [:name, :age, :owner_name, :owner_age]]}\n      ]\n    ]","ref":"Flop.Schema.html#module-usage"},{"type":"protocol","title":"Default and maximum limits - Flop.Schema","doc":"Define a default or maximum limit by setting the `default_limit` and\n`max_limit` options while deriving Flop.Schema. `Flop.validate/1` will apply\nthe default limit and validate the maximum limit.\n\n    @derive {\n      Flop.Schema,\n      filterable: [:name, :species],\n      sortable: [:name, :age],\n      max_limit: 100,\n      default_limit: 50\n    }","ref":"Flop.Schema.html#module-default-and-maximum-limits"},{"type":"protocol","title":"Default sort order - Flop.Schema","doc":"Specify a default sort order by setting the `default_order_by` and\n`default_order_directions` options when deriving Flop.Schema. The default\nvalues will be applied by `Flop.validate/1`. If no order directions are set,\n`:asc` is the default for all fields.\n\n    @derive {\n      Flop.Schema,\n      filterable: [:name, :species],\n      sortable: [:name, :age],\n      default_order: %{\n        order_by: [:name, :age],\n        order_directions: [:asc, :desc]\n      }\n    }","ref":"Flop.Schema.html#module-default-sort-order"},{"type":"protocol","title":"Restricting pagination types - Flop.Schema","doc":"By default, `page`/`page_size`, `offset`/`limit` and cursor-based pagination\n(`first`/`after` and `last`/`before`) are enabled. If you wish to restrict the\npagination type for a schema, you can set the `pagination_types` option.\n\n    @derive {\n      Flop.Schema,\n      filterable: [:name, :species],\n      sortable: [:name, :age],\n      pagination_types: [:first, :last]\n    }\n\nSee also `t:Flop.option/0` and `t:Flop.pagination_type/0`. Setting the value\nto `nil` allows all pagination types.","ref":"Flop.Schema.html#module-restricting-pagination-types"},{"type":"protocol","title":"Alias fields - Flop.Schema","doc":"To sort by calculated values, you can use `Ecto.Query.API.selected_as/2` in\nyour query, define an alias field in your schema, and add the alias field to\nthe list of sortable fields.\n\nSchema:\n\n    @derive {\n      Flop.Schema,\n      filterable: [],\n      sortable: [:pet_count],\n      adapter_opts: [\n        alias_fields: [:pet_count]\n      ]\n    }\n\nQuery:\n\n    Owner\n    |> join(:left, [o], p in assoc(o, :pets), as: :pets)\n    |> group_by([o], o.id)\n    |> select(\n      [o, pets: p],\n      {o.id, p.id |> count() |> selected_as(:pet_count)}\n    )\n    |> Flop.validate_and_run(params, for: Owner)\n\nNote that it is not possible to use field aliases in `WHERE` clauses, which\nmeans you cannot add alias fields to the list of filterable fields, and you\ncannot sort by an alias field if you are using cursor-based pagination.","ref":"Flop.Schema.html#module-alias-fields"},{"type":"protocol","title":"Compound fields - Flop.Schema","doc":"Sometimes you might need to apply a search term to multiple fields at once,\ne.g. you might want to search in both the family name and given name field.\nYou can do that with Flop by defining a compound field.\n\n    @derive {\n      Flop.Schema,\n      filterable: [:full_name],\n      sortable: [:full_name],\n      adapter_opts: [\n        compound_fields: [full_name: [:family_name, :given_name]]\n      ]\n    }\n\nThis allows you to use the field name `:full_name` as any other field in the\nfilter and order parameters.\n\n#","ref":"Flop.Schema.html#module-compound-fields"},{"type":"protocol","title":"Filtering - Flop.Schema","doc":"params = %{\n      filters: [%{\n        field: :full_name,\n        op: :like,\n        value: \"margo\"\n      }]\n    }\n\nThis would translate to:\n\n    WHERE family_name like '%margo%' OR given_name like '%margo%'\n\nPartial matches of the search term can be achieved with one of\nthe like operators.\n\n    params = %{\n      filters: [%{\n        field: :full_name,\n        op: :ilike_and,\n        value: [\"margo\", \"martindale\"]\n      }]\n    }\nor\n    params = %{\n      filters: [%{\n        field: :full_name,\n        op: :ilike_and,\n        value: \"margo martindale\"\n      }]\n    }\nThis would translate to:\n\n    WHERE (family_name ilike '%margo%' OR given_name ilike '%margo%')\n    AND (family_name ilike '%martindale%' OR given_name ilike '%martindale%')\n\n#","ref":"Flop.Schema.html#module-filtering"},{"type":"protocol","title":"Filter operator rules - Flop.Schema","doc":"- `:=~` `:like` `:not_like` `:like_and` `:like_or` `:ilike` `:not_ilike` `:ilike_and` `:ilike_or`  \n  If a string value is passed it will be split at whitespace\n  characters and each segment will be checked separately. If a list of strings is\n  passed the individual strings are not split. The filter matches for a value\n  if it matches for any of the fields.\n- `:empty`  \n  Matches if all fields of the compound field are `nil`.\n- `:not_empty`  \n  Matches if any field of the compound field is not `nil`.\n- `:==` `:!=` `: =` `:>` `:in` `:not_in` `:contains` `:not_contains`  \n  ** These filter operators are ignored for compound fields at the moment.\n  This will be added in a future version.**  \n  The filter value is normalized by splitting the string at whitespaces and\n  joining it with a space. The values of all fields of the compound field are\n  split by whitespace character and joined with a space, and the resulting\n  values are joined with a space again.\n\n#","ref":"Flop.Schema.html#module-filter-operator-rules"},{"type":"protocol","title":"Sorting - Flop.Schema","doc":"params = %{\n      order_by: [:full_name],\n      order_directions: [:desc]\n    }\n\nThis would translate to:\n\n    ORDER BY family_name DESC, given_name DESC\n\nNote that compound fields cannot be used as pagination cursors.","ref":"Flop.Schema.html#module-sorting"},{"type":"protocol","title":"Join fields - Flop.Schema","doc":"If you need to filter or order across tables, you can define join fields.\n\nAs an example, let's define these schemas:\n\n    schema \"owners\" do\n      field :name, :string\n      field :email, :string\n\n      has_many :pets, Pet\n    end\n\n    schema \"pets\" do\n      field :name, :string\n      field :species, :string\n\n      belongs_to :owner, Owner\n    end\n\nAnd now we want to find all owners that have pets of the species\n`\"E. africanus\"`. To do this, first we need to define a join field on the\n`Owner` schema.\n\n    @derive {\n      Flop.Schema,\n      filterable: [:pet_species],\n      sortable: [:pet_species],\n      adapter_opts: [\n        join_fields: [\n          pet_species: [\n            binding: :pets,\n            field: :species,\n            ecto_type: :string\n          ]\n        ]\n      ]\n    }\n\nIn this case, `:pet_species` would be the alias of the field that you can\nrefer to in the filter and order parameters. The options are:\n\n- `:binding` - The named binding you set with the `:as` option in the join\n  statement of your query.\n- `:field` - The field on that binding on which the filter should be applied.\n- `:ecto_type` - The Ecto type of the field. This allows Flop to validate\n  filter values, and also to treat empty arrays and empty maps as empty values\n  depending on the type. See also `Ecto type option` section below.\n\nIn order to retrieve the pagination cursor value for a join field, Flop needs\nto know how to get the field value from the struct that is returned from the\ndatabase. `Flop.Schema.get_field/2` is used for that. By default, Flop assumes\nthat the binding name matches the name of the field for the association in\nyour Ecto schema (the one you set with `has_one`, `has_many` or `belongs_to`).\n\nIn the example above, Flop would try to access the field in the struct under\nthe path `[:pets, :species]`.\n\nIf you have joins across multiple tables, or if you can't give the binding\nthe same name as the association field, you can specify the path explicitly.\n\n    @derive {\n      Flop.Schema,\n      filterable: [:pet_species],\n      sortable: [:pet_species],\n      adapter_opts: [\n        join_fields: [\n          pet_species: [\n            binding: :pets,\n            field: :species,\n            path: [:pets, :species]\n          ]\n        ]\n      ]\n    }\n\nAfter setting up the join fields, you can write a query like this:\n\n    params = %{\n      filters: [%{field: :pet_species, op: :==, value: \"E. africanus\"}]\n    }\n\n    Owner\n    |> join(:left, [o], p in assoc(o, :pets), as: :pets)\n    |> preload([pets: p], [pets: p])\n    |> Flop.validate_and_run!(params, for: Owner)\n\nIf your query returns data in a different format, you don't need to set the\n`:path` option. Instead, you can pass a custom cursor value function in the\noptions. See `Flop.Cursor.get_cursors/2` and `t:Flop.option/0`.\n\nNote that Flop doesn't create the join clauses for you. The named bindings\nalready have to be present in the query you pass to the Flop functions. You\ncan use `Flop.with_named_bindings/4` or `Flop.named_bindings/3` to get the\nbuild the join clauses needed for a query dynamically and avoid adding\nunnecessary joins.","ref":"Flop.Schema.html#module-join-fields"},{"type":"protocol","title":"Filtering by calculated values with subqueries - Flop.Schema","doc":"You can join on a subquery with a named binding and add a join field as\ndescribed above.\n\nSchema:\n\n    @derive {\n      Flop.Schema,\n      filterable: [:pet_count],\n      sortable: [:pet_count],\n      adapter_opts: [\n        join_fields: [\n          pet_count: [\n            binding: :pet_count,\n            field: :count,\n            ecto_type: :integer\n          ]\n        ]\n      ]\n    }\n\nQuery:\n\n    params = %{filters: [%{field: :pet_count, op: :>, value: 2}]}\n\n    pet_count_query =\n      Pet\n      |> where([p], parent_as(:owner).id == p.owner_id)\n      |> select([p], %{count: count(p)})\n\n    q =\n      (o in Owner)\n      |> from(as: :owner)\n      |> join(:inner_lateral, [owner: o], p in subquery(pet_count_query),\n        as: :pet_count\n      )\n      |> Flop.validate_and_run(params, for: Owner)","ref":"Flop.Schema.html#module-filtering-by-calculated-values-with-subqueries"},{"type":"protocol","title":"Custom fields - Flop.Schema","doc":"Custom fields allow for precise control over filter queries, making it\npossible to implement filter logic that the built-in filtering options cannot\nsatisfy.\n\nFor example, you might need to handle dates and times in a particular way that\ntakes into account different time zones, or perform database-specific queries\nusing fragments.\n\nCustom field filters are referenced by a tuple\n`{mod :: module, function :: atom, opts :: keyword}`. The referenced function\nreceives three arguments: the Ecto query, the Flop filter, and an options\nkeyword list.\n\nIf runtime options are necessary (like the timezone of the request or the user\nID of the current user), use the `extra_opts` option when calling Flop\nfunctions.\n\nNote that as of now, custom fields only support filtering, not sorting.\n\nSchema:\n\n    @derive {\n      Flop.Schema,\n      filterable: [:inserted_at_date],\n      adapter_opts: [\n        custom_fields: [\n          inserted_at_date: [\n            filter: {CustomFilters, :date_filter, [source: :inserted_at]},\n            ecto_type: :date,\n            operators: [: =]\n          ]\n        ]\n      ]\n    }\n\nIf you pass the `:ecto_type` option like above, the filter value will be\nautomatically cast.\n\nFilter module:\n\n    defmodule CustomFilters do\n      import Ecto.Query\n\n      def date_filter(query, %Flop.Filter{value: value, op: op}, opts) do\n        source = Keyword.fetch!(opts, :source)\n        timezone = Keyword.fetch!(opts, :timezone)\n\n        expr = dynamic(\n          [r],\n          fragment(\"((? AT TIME ZONE 'utc') AT TIME ZONE ?)::date\",\n          field(r, ^source), ^timezone)\n        )\n\n        conditions =\n          case op do\n            :>= -> dynamic([r], ^expr >= ^value)\n            :  dynamic([r], ^expr <= ^value)\n          end\n\n        where(query, ^conditions)\n      end\n    end\n\nQuery:\n\n    Flop.validate_and_run(\n      MyApp.Pet,\n      params,\n      for: MyApp.Pet,\n      extra_opts: [timezone: timezone]\n    )\n\nIf your custom filter requires certain named bindings, you can use the\n`:bindings` option to specify them. Then, using `Flop.with_named_bindings/4`,\nthese bindings can be conditionally added to your query based on filter\nconditions.","ref":"Flop.Schema.html#module-custom-fields"},{"type":"protocol","title":"Ecto type option - Flop.Schema","doc":"Flop automatically retrieves the field type from the schema module for regular\nschema fields, enabling it to correctly cast filter values. Compound fields\nare always treated as string fields.\n\nFor join and custom fields, Flop cannot automatically determine the Ecto type.\nTherefore, you need to specify the `ecto_type` option. This helps Flop cast\nfilter values for join and custom fields properly. If this option is not set,\nFlop will accept any filter value, potentially leading to an\n`Ecto.Query.CastError` if an invalid filter value is used. Additionally,\nwithout this option, Flop cannot identify empty lists and maps as empty values\nfor array and map fields.\n\n    @derive {\n      Flop.Schema,\n      filterable: [:full_text, :pet_species],\n      sortable: [:id],\n      adapter_opts: [\n        join_fields: [\n          pet_species: [\n            binding: :pets,\n            field: :species,\n            ecto_type: :string\n          ]\n        ],\n        custom_fields: [\n          full_text: [\n            filter: {__MODULE__, :full_text_filter, []},\n            ecto_type: :string\n          ]\n        ]\n      ]\n    }\n\nYou can specify any Ecto type with the `ecto_type` option. Here are some\nexamples:\n\n- A simple string: `ecto_type: :string`\n- An integer: `ecto_type: :integer`\n- An array of strings: `ecto_type: {:array, :string}`\n- A custom Ecto type: `ecto_type: MyCustomType`\n\nFor parameterized types, use the following syntax:\n\n- `ecto_type: {:parameterized, Ecto.Enum, Ecto.Enum.init(values: [:one, :two])}`\n\nIf you're working with `Ecto.Enum` types, you can use a more convenient\nsyntax:\n\n- `ecto_type: {:ecto_enum, [:one, :two]}`\n\nFurthermore, you can reference a type from another schema:\n\n- `ecto_type: {:from_schema, MyApp.Pet, :mood}`\n\nNote that `Flop.Phoenix` encodes all filters in query string using\n`Plug.Conn.Query`. It is expected that filter values can be converted to\nstrings with `to_string/1`. If you are using an Ecto custom type that casts\nas a struct, you will need to implement the `String.Chars` protocol for that\nstruct.","ref":"Flop.Schema.html#module-ecto-type-option"},{"type":"function","title":"Flop.Schema.default_limit/1","doc":"Returns the default limit of a schema.\n\n    iex> Flop.Schema.default_limit(%MyApp.Fruit{})\n    60","ref":"Flop.Schema.html#default_limit/1"},{"type":"function","title":"Flop.Schema.default_order/1","doc":"Returns the default order of a schema.\n\n    iex> Flop.Schema.default_order(%MyApp.Fruit{})\n    %{order_by: [:name], order_directions: [:asc]}","ref":"Flop.Schema.html#default_order/1"},{"type":"function","title":"Flop.Schema.default_pagination_type/1","doc":"Returns the allowed pagination types of a schema.\n\n    iex> Flop.Schema.pagination_types(%MyApp.Fruit{})\n    [:first, :last, :offset]","ref":"Flop.Schema.html#default_pagination_type/1"},{"type":"function","title":"Flop.Schema.field_info/2","doc":"Returns the field information for the given field name.","ref":"Flop.Schema.html#field_info/2"},{"type":"function","title":"Examples - Flop.Schema.field_info/2","doc":"iex> field_info(%MyApp.Pet{}, :age)\n    %Flop.FieldInfo{ecto_type: :integer, extra: %{type: :normal, field: :age}}\n    iex> field_info(%MyApp.Pet{}, :full_name)\n    %Flop.FieldInfo{\n      ecto_type: :string,\n      operators: [\n        :=~,\n        :like,\n        :not_like,\n        :like_and,\n        :like_or,\n        :ilike,\n        :not_ilike,\n        :ilike_and,\n        :ilike_or,\n        :empty,\n        :not_empty\n      ],\n      extra: %{type: :compound, fields: [:family_name, :given_name]}\n    }\n    iex> field_info(%MyApp.Pet{}, :owner_name)\n    %Flop.FieldInfo{\n      ecto_type: :string,\n      extra: %{\n        type: :join,\n        path: [:owner, :name],\n        binding: :owner,\n        field: :name\n      }\n    }\n    iex> field_info(%MyApp.Pet{}, :reverse_name)\n    %Flop.FieldInfo{\n      ecto_type: :string,\n      extra: %{\n        type: :custom,\n        filter: {MyApp.Pet, :reverse_name_filter, []},\n        bindings: []\n      }\n    }","ref":"Flop.Schema.html#field_info/2-examples"},{"type":"function","title":"Flop.Schema.field_type/2","doc":"Returns the field type in a schema.\n\n- `{:normal, atom}` - An ordinary field on the schema. The second tuple\n  element is the field name.\n- `{:compound, [atom]}` - A combination of fields defined with the\n  `compound_fields` option. The list of atoms refers to the list of fields\n  that are included.\n- `{:join, map}` - A field from a named binding as defined with the\n  `join_fields` option. The map has keys for the `:binding`, `:field` and\n  `:path`.\n- `{:custom, keyword}` - A filter field that uses a custom filter function.","ref":"Flop.Schema.html#field_type/2"},{"type":"function","title":"Examples - Flop.Schema.field_type/2","doc":"iex> field_type(%MyApp.Pet{}, :age)\n    {:normal, :age}\n    iex> field_type(%MyApp.Pet{}, :full_name)\n    {:compound, [:family_name, :given_name]}\n    iex> field_type(%MyApp.Pet{}, :owner_name)\n    {\n      :join,\n      %{\n        binding: :owner,\n        field: :name,\n        path: [:owner, :name],\n        ecto_type: :string\n      }\n    }\n    iex> field_type(%MyApp.Pet{}, :reverse_name)\n    {\n      :custom,\n      %{\n        filter: {MyApp.Pet, :reverse_name_filter, []},\n        ecto_type: :string,\n        operators: nil,\n        bindings: []\n      }\n    }","ref":"Flop.Schema.html#field_type/2-examples"},{"type":"function","title":"Flop.Schema.filterable/1","doc":"Returns the filterable fields of a schema.\n\n    iex> Flop.Schema.filterable(%MyApp.Pet{})\n    [\n      :age,\n      :full_name,\n      :mood,\n      :name,\n      :owner_age,\n      :owner_name,\n      :owner_tags,\n      :pet_and_owner_name,\n      :species,\n      :tags,\n      :custom,\n      :reverse_name\n    ]","ref":"Flop.Schema.html#filterable/1"},{"type":"function","title":"Flop.Schema.get_field/2","doc":"Gets the field value from a struct.\n\nResolves join fields and compound fields according to the config.\n\n    # join_fields: [owner_name: [binding: :owner, field: :name]]\n    iex> pet = %MyApp.Pet{name: \"George\", owner: %MyApp.Owner{name: \"Carl\"}}\n    iex> Flop.Schema.get_field(pet, :name)\n    \"George\"\n    iex> Flop.Schema.get_field(pet, :owner_name)\n    \"Carl\"\n\n    # compound_fields: [full_name: [:family_name, :given_name]]\n    iex> pet = %MyApp.Pet{given_name: \"George\", family_name: \"Gooney\"}\n    iex> Flop.Schema.get_field(pet, :full_name)\n    \"Gooney George\"\n\nFor join fields, this function relies on the binding name in the schema config\nmatching the field name for the association in the struct.","ref":"Flop.Schema.html#get_field/2"},{"type":"function","title":"Flop.Schema.max_limit/1","doc":"Returns the maximum limit of a schema.\n\n    iex> Flop.Schema.max_limit(%MyApp.Pet{})\n    1000","ref":"Flop.Schema.html#max_limit/1"},{"type":"function","title":"Flop.Schema.pagination_types/1","doc":"Returns the allowed pagination types of a schema.\n\n    iex> Flop.Schema.pagination_types(%MyApp.Fruit{})\n    [:first, :last, :offset]","ref":"Flop.Schema.html#pagination_types/1"},{"type":"function","title":"Flop.Schema.sortable/1","doc":"Returns the sortable fields of a schema.\n\n    iex> Flop.Schema.sortable(%MyApp.Pet{})\n    [:name, :age, :owner_name, :owner_age]","ref":"Flop.Schema.html#sortable/1"},{"type":"type","title":"Flop.Schema.adapter_option/0","doc":"Options specific to the adapter.\n\n- `:join_fields` - A list of fields on named bindings.\n- `:compound_fields` - Groups of fields that can be combined and filtered, for\n  example a family name plus a given name field.\n- `:custom_fields` - Custom fields with user-defined filter functions.\n- `:alias_field` - Fields that reference aliases defined with\n  `Ecto.Query.API.selected_as/2`.","ref":"Flop.Schema.html#t:adapter_option/0"},{"type":"type","title":"Flop.Schema.custom_field_option/0","doc":"Defines the options for a custom field.\n\n- `:filter` (required) - A module/function/options tuple referencing a\n  custom filter function. The function must take the Ecto query, the\n  `Flop.Filter` struct, and the options from the tuple as arguments.\n- `:ecto_type` - The Ecto type of the field. The filter operator and value\n  validation is based on this option.\n- `:bindings` - If the custom filter function requires certain named bindings\n  to be present in the Ecto query, you can specify them here. These bindings\n  will be conditionally added by `Flop.with_named_bindings/4` if the filter\n  is used.\n- `:operators` - Defines which filter operators are allowed for this field.\n  If omitted, all operators will be accepted.\n\nIf both the `:ecto_type` and the `:operators` option are set, the `:operators`\noption takes precedence and only the filter value validation is based on the\n`:ecto_type`.","ref":"Flop.Schema.html#t:custom_field_option/0"},{"type":"type","title":"Flop.Schema.ecto_type/0","doc":"Either an Ecto type, or reference to the type of an existing schema field, or\nan adhoc Ecto.Enum.","ref":"Flop.Schema.html#t:ecto_type/0"},{"type":"type","title":"Examples - Flop.Schema.ecto_type/0","doc":"You can pass any Ecto type:\n\n- `:string`\n- `:integer`\n- `Ecto.UUID`\n- `{:parameterized, Ecto.Enum, Ecto.Enum.init(values: [:one, :two])}`\n\nOr reference a schema field:\n\n`{:from_schema, MyApp.Pet, :mood}`\n\nOr build an adhoc Ecto.Enum:\n\n- `{:ecto_enum, [:one, :two]}` (This has the same effect as the `:parameterized`\n  example above.)\n- `{:ecto_enum, [one: 1, two: 2]}`\n\nNote that if you make an `Ecto.Enum` type this way, the filter value will be\ncast as an atom. This means the field you filter on also needs to be an\n`Ecto.Enum`, or a custom type that is able to cast atoms. You cannot use this\non a string field.","ref":"Flop.Schema.html#t:ecto_type/0-examples"},{"type":"type","title":"Flop.Schema.join_field_option/0","doc":"Defines the options for a join field.\n\n- `:binding` (required) - Any named binding\n- `:field` (required)\n- `:ecto_type` - The Ecto type of the field. The filter operator and value\n  validation is based on this option.\n- `:path` - This option is used by `Flop.Schema.get_field/2` to retrieve the\n  field value from a row. That function is also used by the default cursor\n  functions in `Flop.Cursor` to determine the cursors. If the option is\n  omitted, it defaults to `[binding, field]`.","ref":"Flop.Schema.html#t:join_field_option/0"},{"type":"type","title":"Flop.Schema.option/0","doc":"Options that can be passed when deriving the Flop.Schema protocol.\n\nThese are either general schema options or adapter-specific options nested\nunder the `:adapter_opts` key. For backward compatibility, the options of the\nEcto adapter can be set directly at the root level as well.\n\n- `:filterable` (required) - A list of fields that can be used in filters.\n  Supports fields from the Ecto schema, join fields, compound fields and\n  custom fields. Alias fields are not supported.\n- `:sortable` (required) - A list of fields that can be used for sorting.\n  Supports fields from the Ecto schema, join fields, and alias fields. Custom\n  fields and compound fields are not supported.\n- `:default_limit` - The default limit applied if no `limit`, `page_size`,\n  `first` or `last` parameter is set.\n- `:max_limit` - The maximum limit that can be set via parameters.\n- `:default_order` - The default order applied when no order parameters are\n  set.\n- `:pagination_types` - A list of allowed pagination types for this schema.\n- `:default_pagination_type` - The default pagination type used if no\n  pagination parameters are set.\n- `:adapter_opts` - Additional adapter-specific options.","ref":"Flop.Schema.html#t:option/0"},{"type":"type","title":"Flop.Schema.t/0","doc":"All the types that implement this protocol.","ref":"Flop.Schema.html#t:t/0"},{"type":"exception","title":"Flop.InvalidCursorError","doc":"Raised when an invalid cursor is passed to the decode function.\n\nA cursor might be invalid if it is malformed, not in the expected format, or\ncontains unexpected data types.","ref":"Flop.InvalidCursorError.html"},{"type":"function","title":"Flop.InvalidCursorError.message/1","doc":"","ref":"Flop.InvalidCursorError.html#message/1"},{"type":"exception","title":"Flop.InvalidDefaultOrderError","doc":"","ref":"Flop.InvalidDefaultOrderError.html"},{"type":"function","title":"Flop.InvalidDefaultOrderError.message/1","doc":"","ref":"Flop.InvalidDefaultOrderError.html#message/1"},{"type":"exception","title":"Flop.InvalidDefaultPaginationTypeError","doc":"","ref":"Flop.InvalidDefaultPaginationTypeError.html"},{"type":"function","title":"Flop.InvalidDefaultPaginationTypeError.message/1","doc":"","ref":"Flop.InvalidDefaultPaginationTypeError.html#message/1"},{"type":"exception","title":"Flop.InvalidDirectionsError","doc":"An error that is raised when invalid directions are passed.","ref":"Flop.InvalidDirectionsError.html"},{"type":"function","title":"Flop.InvalidDirectionsError.message/1","doc":"","ref":"Flop.InvalidDirectionsError.html#message/1"},{"type":"exception","title":"Flop.InvalidParamsError","doc":"Raised when parameter validation fails.\n\nThis can occur under a number of circumstances, such as:\n\n- Pagination parameters are improperly formatted or invalid.\n- Filter values are incompatible with the respective field's type or specified\n  operator.\n- Filters are applied on fields that have not been configured as filterable.\n- Ordering parameters are applied on fields that have not been configured as\n  sortable.","ref":"Flop.InvalidParamsError.html"},{"type":"function","title":"Flop.InvalidParamsError.message/1","doc":"","ref":"Flop.InvalidParamsError.html#message/1"},{"type":"type","title":"Flop.InvalidParamsError.t/0","doc":"","ref":"Flop.InvalidParamsError.html#t:t/0"},{"type":"exception","title":"Flop.NoRepoError","doc":"","ref":"Flop.NoRepoError.html"},{"type":"function","title":"Flop.NoRepoError.message/1","doc":"","ref":"Flop.NoRepoError.html#message/1"},{"type":"exception","title":"Flop.UnknownFieldError","doc":"","ref":"Flop.UnknownFieldError.html"},{"type":"function","title":"Flop.UnknownFieldError.message/1","doc":"","ref":"Flop.UnknownFieldError.html#message/1"},{"type":"extras","title":"Flop","doc":"# Flop\n\n![CI](https://github.com/woylie/flop/workflows/CI/badge.svg) [![Hex](https://img.shields.io/hexpm/v/flop)](https://hex.pm/packages/flop) [![codecov](https://codecov.io/gh/woylie/flop/branch/main/graph/badge.svg?token=32BSY8O2LI)](https://codecov.io/gh/woylie/flop)\n\nFlop is an Elixir library designed to easily apply filtering, ordering, and\npagination to your Ecto queries.","ref":"readme.html"},{"type":"extras","title":"Features - Flop","doc":"- **Offset-based pagination:** Allows pagination through `offset`/`limit` or\n  `page`/`page_size` parameters.\n- **Cursor-based pagination:** Also known as key set pagination, provides a more\n  efficient alternative to offset-based pagination. Compatible with Relay\n  pagination arguments.\n- **Sorting:** Applies sort parameters on multiple fields in any direction.\n- **Filtering:** Allows complex data filtering using multiple conditions,\n  operators, and fields.\n- **Parameter validation:** Ensures the validity of provided parameters.\n- **Configurable filterable and sortable fields:** Only applies parameters to\n  the fields that were explicitly configured as filterable or sortable.\n- **Join fields:** Allows the application of pagination, sort, and filter\n  parameters on any named binding. Provides functions to help you to avoid\n  unnecessary join clauses.\n- **Compound fields:** Provides the ability to apply filter parameters on\n  multiple string fields, for example for a full name filter.\n- **Custom fields:** Provides an escape hatch for filters that Flop is not able\n  to build on its own.\n- **Relay connection formatter:** Formats the connection in Relay style,\n  providing edges, nodes, and page info.\n- **UI helpers and URL builders through\n  [Flop Phoenix](https://hex.pm/packages/flop_phoenix):** Pagination, sortable\n  tables and filter forms.","ref":"readme.html#features"},{"type":"extras","title":"Installation - Flop","doc":"To get started, add `flop` to your dependencies list in your project's `mix.exs`\nfile:\n\n```elixir\ndef deps do\n  [\n    {:flop, \"~> 0.24.0\"}\n  ]\nend\n```\n\nYou can also configure a default repo for Flop by adding the following line to\nyour config file:\n\n```elixir\nconfig :flop, repo: MyApp.Repo\n```\n\nInstead of configuring Flop globally, you can also use a configuration module.\nPlease refer to the Flop module documentation for more information.","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Flop","doc":"#","ref":"readme.html#usage"},{"type":"extras","title":"Define sortable and filterable fields - Flop","doc":"To define sortable and filterable fields in your Ecto schema, you can derive\n`Flop.Schema`. This step is optional but highly recommended, particularly when\nthe parameters passed to Flop's functions are user-provided. Deriving\n`Flop.Schema` ensures that Flop applies filtering and sorting parameters only to\nthe fields you've explicitly configured.\n\n```elixir\ndefmodule MyApp.Pet do\n  use Ecto.Schema\n\n  @derive {\n    Flop.Schema,\n    filterable: [:name, :species],\n    sortable: [:name, :age, :species]\n  }\n\n  schema \"pets\" do\n    field :name, :string\n    field :age, :integer\n    field :species, :string\n    field :social_security_number, :string\n  end\nend\n```\n\nBesides sortable and filterable fields, `Flop.Schema` also allows the definition\nof join fields, compound fields, or custom fields. You can also set maximum or\ndefault limits, among other options. For a comprehensive list of available\noptions, check the `Flop.Schema` documentation.\n\n#","ref":"readme.html#define-sortable-and-filterable-fields"},{"type":"extras","title":"Query data - Flop","doc":"Use the `Flop.validate_and_run/3` or `Flop.validate_and_run!/3` function to both\nvalidate the parameters and fetch data from the database, and acquire pagination\nmetadata in one operation.\n\nHere is an example of how you might use this in your code:\n\n```elixir\ndefmodule MyApp.Pets do\n  import Ecto.Query, warn: false\n\n  alias Ecto.Changeset\n  alias MyApp.{Pet, Repo}\n\n  @spec list_pets(map) ::\n          {:ok, {[Pet.t()], Flop.Meta.t()}} | {:error, Flop.Meta.t()}\n  def list_pets(params \\\\ %{}) do\n    Flop.validate_and_run(Pet, params, for: Pet)\n  end\nend\n```\n\nThe `for` option sets the Ecto schema for which you derived `Flop.Schema`. If\nyou haven't derived `Flop.Schema` as described above, this option can be\nomitted. However, this is not recommended unless all parameters are generated\ninternally and are guaranteed to be safe.\n\nOn success, `Flop.validate_and_run/3` returns an `:ok` tuple. The second element\nof this tuple is another tuple containing the fetched data and metadata.\n\n```elixir\n{:ok, {[%Pet{}], %Flop.Meta{}}}\n```\n\nYou can learn more about the `Flop.Meta` struct in the\n[module documentation](https://hexdocs.pm/flop/Flop.Meta.html).\n\nAlternatively, you may separate parameter validation and data fetching into\ndifferent steps using the Flop.validate/2, Flop.validate!/2, and Flop.run/3\nfunctions. This allows you to manipulate the validated parameters, to modify the\nquery depending on the parameters, or to move the parameter validation to a\ndifferent layer of your application.\n\n```elixir\nwith {:ok, flop} <- Flop.validate(params, for: Pet) do\n  Flop.run(Pet, flop, for: Pet)\nend\n```\n\nThe aforementioned functions internally call the lower-level functions\n`Flop.all/3`, `Flop.meta/3`, and `Flop.count/3`. If you have advanced\nrequirements, you might prefer to use these functions directly. However, it's\nimportant to note that these lower-level functions do not validate the\nparameters. If parameters are generated based on user input, they should always\nbe validated first using `Flop.validate/2` or `Flop.validate!/2` to ensure safe\nexecution.\n\nThe examples above assume that you configured a default repo. However, you can\nalso pass the repo directly to the functions:\n\n```elixir\nFlop.validate_and_run(Pet, flop, repo: MyApp.Repo)\nFlop.all(Pet, flop, repo: MyApp.Repo)\nFlop.meta(Pet, flop, repo: MyApp.Repo)\n```\n\nFor more detailed information, refer the\n[documentation](https://hexdocs.pm/flop/readme.html).","ref":"readme.html#query-data"},{"type":"extras","title":"Parameter format - Flop","doc":"The Flop library requires parameters to be provided in a specific format as a\nmap. This map can be translated into a URL query parameter string, typically\nfor use in a web framework like Phoenix.\n\n#","ref":"readme.html#parameter-format"},{"type":"extras","title":"Pagination - Flop","doc":"##","ref":"readme.html#pagination"},{"type":"extras","title":"Offset / limit - Flop","doc":"You can specify an offset to start from and a limit to the number of results.\n\n```elixir\n%{offset: 20, limit: 10}\n```\n\nThis translates to the following query parameter string:\n\n```html\n?offset=20&limit=10\n```\n\n##","ref":"readme.html#offset-limit"},{"type":"extras","title":"Page / page size - Flop","doc":"You can specify the page number and the size of each page.\n\n```elixir\n%{page: 2, page_size: 10}\n```\n\nThis translates to the following query parameter string:\n\n```html\n?page=2&page_size=10\n```\n\n##","ref":"readme.html#page-page-size"},{"type":"extras","title":"Cursor - Flop","doc":"You can fetch a specific number of results before or after a given cursor.\n\n```elixir\n%{first: 10, after: \"g3QAAAABZAACaWRiAAACDg==\"}\n%{last: 10, before: \"g3QAAAABZAACaWRiAAACDg==\"}\n```\n\nThese translate to the following query parameter strings:\n\n```html\n?first=10&after=g3QAAAABZAACaWRiAAACDg==\n?last=10&before=g3QAAAABZAACaWRiAAACDg==\n```\n\n#","ref":"readme.html#cursor"},{"type":"extras","title":"Ordering - Flop","doc":"To sort the results, specify fields to order by and the direction of sorting for\neach field.\n\n```elixir\n%{order_by: [:name, :age], order_directions: [:asc, :desc]}\n```\n\nThis translates to the following query parameter string:\n\n```html\n?order_by[]=name&order_by[]=age&order_directions[]=asc&order_directions[]=desc\n```\n\n#","ref":"readme.html#ordering"},{"type":"extras","title":"Filters - Flop","doc":"You can filter the results by providing a field, an operator, and a value. The\noperator is optional and defaults to `==`. Multiple filters are combined with a\nlogical `AND`. At the moment, combining filters with `OR` is not supported.\n\n```elixir\n%{filters: [%{field: :name, op: :ilike_and, value: \"Jane\"}]}\n```\n\nThis translates to the following query parameter string:\n\n```html\n?filters[0][field]=name&filters[0][op]=ilike_and&filters[0][value]=Jane\n```\n\nRefer to the `Flop.Filter` documentation and `t:Flop.t/0` type documentation for\nmore details on using filters.","ref":"readme.html#filters"},{"type":"extras","title":"Internal parameters - Flop","doc":"Flop is designed to manage parameters that come from the user side. While it is\npossible to alter those parameters and append extra filters upon receiving them,\nit is advisable to clearly differentiate parameters coming from outside and the\nparameters that your application adds internally.\n\nConsider the scenario where you need to scope a query based on the current user.\nIn this case, it is better to create a separate function that introduces the\nnecessary `WHERE` clauses:\n\n```elixir\ndef list_pets(%{} = params, %User{} = current_user) do\n  Pet\n  |> scope(current_user)\n  |> Flop.validate_and_run(params, for: Pet)\nend\n\ndefp scope(q, %User{role: :admin}), do: q\ndefp scope(q, %User{id: user_id}), do: where(q, user_id: ^user_id)\n```\n\nIf you need to add extra filters that are only used internally and aren't\nexposed to the user, you can pass them as a separate argument. This same\nargument can be used to override certain options depending on the context in\nwhich the function is called.\n\n```elixir\ndef list_pets(%{} = args, opts \\\\ [], %User{} = current_user) do\n  flop_opts =\n    opts\n    |> Keyword.take([\n      :default_limit,\n      :default_pagination_type,\n      :pagination_types\n    ])\n    |> Keyword.put(:for, Pet)\n\n  Pet\n  |> scope(current_user)\n  |> apply_filters(opts)\n  |> Flop.validate_and_run(flop, flop_opts)\nend\n\ndefp scope(q, %User{role: :admin}), do: q\ndefp scope(q, %User{id: user_id}), do: where(q, user_id: ^user_id)\n\ndefp apply_filters(q, opts) do\n  Enum.reduce(opts, q, fn\n    {:last_health_check, dt}, q -> where(q, [p], p.last_health_check   where(q, [p], p.reminder_service == ^bool)\n    _, q -> q\n  end)\nend\n```\n\nWith this approach, you maintain a clean separation between user-driven\nparameters and system-driven parameters, leading to more maintainable and less\nerror-prone code.","ref":"readme.html#internal-parameters"},{"type":"extras","title":"Relay and Absinthe - Flop","doc":"The `Flop.Relay` module is useful if you are using\n[absinthe](https://hex.pm/packages/absinthe) with\n[absinthe_relay](https://hex.pm/packages/absinthe_relay), or if you simply need\nto adhere to the Relay cursor specification. This module provides functions that\nhelp transform query responses into a format compatible with Relay.\n\nConsider the scenario where you have defined node objects for owners and pets,\nalong with a connection field for pets on the owner node object.\n\n```elixir\nnode object(:owner) do\n  field :name, non_null(:string)\n  field :email, non_null(:string)\n\n  connection field :pets, node_type: :pet do\n    resolve &MyAppWeb.Resolvers.Pet.list_pets/2\n  end\nend\n\nnode object(:pet) do\n  field :name, non_null(:string)\n  field :age, non_null(:integer)\n  field :species, non_null(:string)\nend\n\nconnection(node_type: :pet)\n```\n\nAbsinthe Relay will establish the arguments `after`, `before`, `first` and\n`last` on the `pets` field. These argument names align with those used by Flop, facilitating their application.\n\nNext, we'll define a `list_pets_by_owner/2` function in the Pets context.\n\n```elixir\ndefmodule MyApp.Pets do\n  import Ecto.Query\n\n  alias MyApp.{Owner, Pet, Repo}\n\n  @spec list_pets_by_owner(Owner.t(), map) ::\n          {:ok, {[Pet.t()], Flop.Meta.t()}} | {:error, Flop.Meta.t()}\n  def list_pets_by_owner(%Owner{id: owner_id}, params \\\\ %{}) do\n    Pet\n    |> where(owner_id: ^owner_id)\n    |> Flop.validate_and_run(params, for: Pet)\n  end\nend\n```\n\nNow, within your resolver, you merely need to invoke the function and call\n`Flop.Relay.connection_from_result/1`, which transforms the result into a tuple\ncomposed of the edges and the page_info, as required by `absinthe_relay`.\n\n```elixir\ndefmodule MyAppWeb.Resolvers.Pet do\n  alias MyApp.{Owner, Pet}\n\n  def list_pets(args, %{source: %Owner{} = owner} = resolution) do\n    with {:ok, result} <- Pets.list_pets_by_owner(owner, args) do\n      {:ok, Flop.Relay.connection_from_result(result)}\n    end\n  end\nend\n```\n\nIn case you want to introduce additional filter arguments, you can employ\n`Flop.nest_filters/3` to convert simple filter arguments into Flop filters,\nwithout necessitating API users to understand the Flop filter format.\n\nLet's add `name` and `species` filter arguments to the `pets` connection field.\n\n```elixir\nnode object(:owner) do\n  field :name, non_null(:string)\n  field :email, non_null(:string)\n\n  connection field :pets, node_type: :pet do\n    arg :name, :string\n    arg :species, :string\n\n    resolve &MyAppWeb.Resolvers.Pet.list_pets/2\n  end\nend\n```\n\nAssuming that these fields have been already configured as filterable with\n`Flop.Schema`, we can use `Flop.nest_filters/3` to take the filter arguments and\ntransform them into a list of Flop filters.\n\n```elixir\ndefmodule MyAppWeb.Resolvers.Pet do\n  alias MyApp.{Owner, Pet}\n\n  def list_pets(args, %{source: %Owner{} = owner} = resolution) do\n    args = nest_filters(args, [:name, :species])\n\n    with {:ok, result} <- Pets.list_pets_by_owner(owner, args) do\n      {:ok, Flop.Relay.connection_from_result(result)}\n    end\n  end\nend\n```\n\n`Flop.nest_filters/3` uses the equality operator `:==` by default.\nYou can override the default operator per field.\n\n```elixir\nargs = nest_filters(args, [:name, :species], operators: %{name: :ilike_and})\n```","ref":"readme.html#relay-and-absinthe"},{"type":"extras","title":"Flop Phoenix - Flop","doc":"[Flop Phoenix](https://hex.pm/packages/flop_phoenix) is a companion library that\nprovides Phoenix components for pagination, sortable tables, and filter forms,\nusable with both Phoenix LiveView and in dead views. It also defines helper\nfunctions to build URLs with Flop query parameters.","ref":"readme.html#flop-phoenix"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"Unreleased - Changelog","doc":"## [0.24.0] - 2023-11-14\n\n#","ref":"changelog.html#unreleased"},{"type":"extras","title":"Changed - Changelog","doc":"- If an invalid operator is passed in a filter, the error will now include the\n  list of allowed operators for that field.\n\n## [0.23.0] - 2023-09-26\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Added `directions` option to `Flop.push_order/3`.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Escape backlash character in queries using one of the `like` operators.\n\n## [0.22.1] - 2023-07-18\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Updated version requirement for Ecto to `~> 3.10.3`. Flop 0.22.0 relies\n  on a feature added in that version and doesn't compile with lower versions.\n\n## [0.22.0] - 2023-07-17\n\nThis release includes a substantial refactoring to lay the groundwork for the\nupcoming adapter feature. While this release contains deprecations and changes,\nthey are either backward compatible or affect functions that are unlikely to be\nused by end users. The primary aim has been to ensure a seamless transition and\nmaintain compatibility with previous versions.\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Added a `Flop.FieldInfo` struct that contains metadata for a field for use\n  by adapters.\n- Added the `Flop.Schema.field_info/2` function, which derives field information\n  and replaces the previous `Flop.Schema.field_type/2` function with a more\n  standardized and structured output.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- The Ecto-specific options `alias_fields`, `compound_fields`, `custom_fields`,\n  and `join_fields` within `Flop.Schema`, as well as `repo` and `query_opts`\n  within `use Flop`, are now nested under the `adapter_opts` keyword. The old\n  configuration format is still supported.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Deprecated - Changelog","doc":"- `Flop.Schema.field_type/2` was deprecated in favor of\n  `Flop.Schema.field_info/2`.\n\n#","ref":"changelog.html#deprecated"},{"type":"extras","title":"Removed - Changelog","doc":"- Removed `Flop.Schema.apply_order_by/3`.\n- Removed `Flop.Schema.cursor_dynamic/3`.\n\n#","ref":"changelog.html#removed"},{"type":"extras","title":"Upgrade guide - Changelog","doc":"While the old configuration format is still supported, you are invited to\nupdate your application to the new structure to prepare for future versions.\n\nTo do this, place the field configuration for `Flop.Schema` under\n`adapter_opts`:\n\n```diff\n@derive {\n  Flop.Schema,\n  filterable: [],\n  sortable: [],\n-  alias_fields: [],\n-  compound_fields: [],\n-  custom_fields: [],\n-  join_fields: []\n+  adapter_opts: [\n+    alias_fields: [],\n+    compound_fields: [],\n+    custom_fields: [],\n+    join_fields: []\n+  ]\n}\n```\n\nSimilarly for `use Flop`, you can nest `repo` and `query_opts` under\n`adapter_opts`:\n\n```diff\nuse Flop,\n  default_limit: 50,\n-  repo: MyApp.Repo,\n-  query_opts: [prefix: \"some-prefix\"]\n+  adapter_opts: [\n+    repo: MyApp.Repo,\n+    query_opts: [prefix: \"some-prefix\"]\n+  ]\n```\n\n## [0.21.0] - 2023-07-02\n\n#","ref":"changelog.html#upgrade-guide"},{"type":"extras","title":"Added - Changelog","doc":"- Introduced `operators` as a new option for restricting acceptable operators\n  for a custom field.\n- Added bindings option for custom fields, allowing required named bindings to\n  be added via `Flop.with_named_bindings/4`.\n- The `ecto_type` option on join and custom fields now supports\n  references: `{:from_schema, MySchema, :some_field}`.\n- The `ecto_type` option now supports a convenient syntax for adhoc enums:\n  `{:ecto_enum, [:one, :two]}`.\n- Improved documentation with added type definitions: `t:Flop.Schema.option/0`,\n  `t:Flop.Schema.join_field_option/0`, `t:Flop.Schema.custom_field_option/0`,\n  and `t:Flop.Schema.ecto_type/0`, describing options available when deriving\n  the `Flop.Schema` protocol.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- **Breaking change:** Filter values are now dynamically cast based on the\n  field type and operator, instead of allowing any arbitrary filter value. This\n  change ensures that invalid filter values cause validation errors instead of\n  cast errors.\n- The options for deriving the `Flop.Schema` protocol and for `use Flop`\n  now undergo stricter validation with `NimbleOptions`.\n- `Flop.Cursor.encode/1` now explicitly sets the minor version option for\n  `:erlang.term_to_binary/2` to `2`, aligning with the new default in OTP 26.\n  Before, this option was not set at all.\n- Added a `decoded_cursor` field to the `Flop` struct. This field temporarily\n  stores the decoded cursor between validation and querying and is\n  discarded when generating the meta data.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Deprecated - Changelog","doc":"- The tuple syntax for defining join fields has been deprecated in favor of a\n  keyword list.\n\n#","ref":"changelog.html#deprecated"},{"type":"extras","title":"Fixed - Changelog","doc":"- Resolved an issue where setting `replace_invalid_params` to `true` still\n  caused validation errors for pagination and sorting parameters due to cast\n  errors, instead of defaulting to valid parameters.\n- Fixed the type specification for `Flop.Filter.allowed_operators/1`.\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Upgrade notes - Changelog","doc":"The newly implemented dynamic casting of filter values could impact your code:\n\n- Filter values failing to cast into the determined type will now yield a\n  validation error or result in the removal of the invalid filter if the\n  `replace_invalid_params` option is enabled.\n- The `value` field of the `Flop.Filter` struct now holds the cast value\n  instead of the original parameter value. For instance, while handling\n  parameters generated via an HTML form with Flop, previously all filter values\n  would be represented as strings in the struct. However, they may now be\n  integers, `DateTime` structs, and so forth. Look out for this if you are\n  directly reading or manipulating `Flop.Filter` structs.\n- For join and custom fields, the type is determined with the `ecto_type`\n  option. Previously, this option was only used for operator validation.\n  Ensure the correct Ecto type is set. If the option is omitted, the filter\n  values will continue to use their incoming format.\n- Manual casting of filter values in a custom filter function is no longer\n  required if the `ecto_type` option is set.\n- If join fields point to `Ecto.Enum` fields, previously you could simply set\n  `ecto_type` to string. This will continue to work if the filter value is\n  passed as a string, but passing it as an atom will cause an error. Make sure\n  to correctly reference the schema field\n  (`{:from_schema, MySchema, :some_field}`) or directly pass the Enum values\n  (`{:ecto_enum, [:one, :two}`).\n- To enable `Flop.Phoenix` to build a query string for filter parameters, the\n  filter value must be convertible into a string via `to_string/1`. If\n  `ecto_type` is set to a custom Ecto type that casts values into a struct, the\n  `String.Chars` protocol must be implemented for that struct.\n- If you use the result of `Flop.Phoenix.to_query/2` in a `~p` sigil for\n  verified routes or in a route helper function, Phoenix converts filter values\n  into a string using the `Phoenix.Param` protocol. If you use `Date`,\n  `DateTime`, `NaiveDateTime`, `Time` filters, or filters using custom structs,\n  you need to implement that protocol for these structs in your application.\n\nPlease review the newly added \"Ecto type option\" section in the `Flop.Schema`\nmodule documentation.\n\n##","ref":"changelog.html#upgrade-notes"},{"type":"extras","title":"Join field syntax - Changelog","doc":"If you are using tuples to define join fields when deriving `Flop.Schema`,\nupdate the configuration to use keyword lists instead:\n\n```diff\n@derive {\n  Flop.Schema,\n  join_fields: [\n-    owner_name: {:owner, :name}\n+    owner_name: [binding: :owner, field: :name]\n  ]\n}\n```\n\n## [0.20.3] - 2023-06-23\n\n#","ref":"changelog.html#join-field-syntax"},{"type":"extras","title":"Changed - Changelog","doc":"- `Flop.count/3` will now wrap queries that have `GROUP BY` clauses in a\n  subquery.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed cursor-based pagination on composite types.\n\n## [0.20.2] - 2023-06-09\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Added nutrition facts about `use Flop` and `@derive Flop.Schema`.\n- The minimum Elixir version is now 1.11.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed a deprecation warning about `Logger.warn/1`.\n- Fixed a deprecation warning about passing an MFA to `:with` in\n  cast_assoc/cast_embed introduced in Ecto 3.10.2.\n\n## [0.20.1] - 2023-05-19\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Added the `:count` override option to `Flop.count/3`.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- The `default_pagination_type` can now be set in the schema.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Don't raise function clause error in `Flop.to_previous_cursor/1` and\n  `Flop.to_next_cursor/1` when the start cursor or end cursor are `nil`.\n\n## [0.20.0] - 2023-03-21\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Added `Flop.unnest_filters/3` as a reverse operation of `Flop.nest_filters/3`\n  after retrieving data from the database.\n- Added `Flop.Filter.fetch_value/2`, `Flop.Filter.get_value/2`,\n  `Flop.Filter.put_value/4`, `Flop.Filter.put_new_value/4`,\n  `Flop.Filter.pop_value/3` and `Flop.Filter.pop_first_value/3`.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Several of the functions for manipulating lists of filters in the\n  `Flop.Filter` module now accept lists of maps with atom keys, lists of maps\n  with string keys, and indexed maps as produced by Phoenix HTML forms as\n  argument.\n- The `empty` and `not_empty` operators now treat empty maps as empty values on\n  map fields and empty arrays as empty values on array fields.\n- `%` and `_` characters in filter values for the `like`, `ilike` and `=~`\n  operators are now escaped.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed an issue that caused filter conditions for `like_and`, `like_or`,\n  `ilike_and` and `ilike_or` to be incorrectly combined when applied to compound\n  fields.\n\n## [0.19.0] - 2023-01-15\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Support for custom fields. These fields allow you to run custom filter\n  functions for anything that cannot be expressed with Flop filters.\n- Added `Flop.with_named_bindings/4` for dynamically adding bindings needed for\n  a Flop query.\n- Added `fetch`, `get`, `get_all`, `delete`, `delete_first`, `drop`, `new`,\n  `take`, `pop`, `pop_first`, `put` and `put_new` functions to `Flop.Filter`.\n- Added `Flop.Meta.with_errors/3`.\n- Added `ecto_type` option to join fields.\n- Added `not_like` and `not_ilike` filter operators.\n- Added a cheatsheet for schema configuration.\n- Added `opts` field to `Flop.Meta` struct.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Renamed `Flop.bindings/3` to `Flop.named_bindings/3`.\n- `Flop.Filter.allowed_operators/2` now tries to determine the Ecto type by\n  reading the Flop field type from the schema module. This function is used\n  during parameter validation, which means the validation step will be a bit\n  stricter now. For join and custom fields, the Ecto type is determined via the\n  new `ecto_type` option. If the option is not set, the function returns all\n  operators as before. For compound fields, only the supported operators are\n  returned.\n\n## [0.18.4] - 2022-11-17\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- The `:ilike_and`, `:ilike_or`, `:like_and` and `:like_or` filter operators can\n  now also be used with a list of strings as filter value.\n\n## [0.18.3] - 2022-10-27\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- `default_pagination_type` can be overridden by passing `false` now.\n\n## [0.18.2] - 2022-10-19\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- `Flop.bindings/3` did not consider join fields that are used as part of a\n  compound field.\n\n## [0.18.1] - 2022-10-14\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- If the given map already has a `:filters` / `\"filters\"` key,\n  `Flop.nest_filters/3` will now merge the derived filters into the existing\n  filters. If the existing filters are formatted as a map (as produced by an\n  HTML form), they are converted to a list first.\n- `use Flop` will now also compile `validate/2` and `validate!/2` functions that\n  apply the options of your config module.\n- Allow setting `default_limit` and `max_limit` to `false`, which removes the\n  default/max limit without falling back to global options.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- `Flop.bindings/3` was returning bindings for filters with `nil` values.\n\n## [0.18.0] - 2022-10-10\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Added `alias_fields` option to `Flop.Schema`, which allows you to sort by\n  field aliases defined with `Ecto.Query.API.selected_as/2`.\n- Added `aliases/2` for getting the alias fields needed for a query.\n- Added documentation example for filtering by calculated values.\n- New option `rename` for `Flop.map_to_filter_params/2` and\n  `Flop.nest_filters/3`.\n- New option `:replace_invalid_params`. This option can be passed to the\n  `validate` and `validate_and_run` functions or set in the global configuration\n  or in a config module. Setting the value to `true` will cause Flop to replace\n  invalid parameters with default values where possible or remove the parameter\n  otherwise during the validation step, instead of returning validation errors.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Require `ecto ~> 3.9.0`.\n- `Flop.Schema` does not raise an error anymore if a compound or join field is\n  defined with the same name as a regular Ecto schema field. This was done so that\n  you can add virtual fields with the same name. It is not possible to\n  differentiate between non-virtual and virtual fields at compile time (at least\n  I don't know how), so we cannot differentiate in the validation step.\n- Flop applies a default limit of `50` and a max limit of `1000` now, unless\n  other values are set.\n- In offset/limit based pagination, the `limit` parameter is now required, in\n  line with the other pagination types. If not set, it will fall back to a\n  default limit.\n\n## [0.17.2] - 2022-10-03\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed an issue where the `repo` option was not read from a backend module.\n\n## [0.17.1] - 2022-10-02\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Added a `backend` field to the `Flop.Meta` struct.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed an issue where the schema options were overridden by the backend module\n  options.\n\n## [0.17.0] - 2022-08-26\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Added the filter operators `not_in` and `not_contains`.\n- Added examples for integration with Relay to the documentation.\n- Added examples for the parameter format to the documentation.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Refactored the query builder. This does not affect users of the library, but\n  makes the code base more readable and lays the groundwork for upcoming\n  features.\n- Added the `:query_opts` option to Flop callbacks to pass on options to the\n  Ecto repo on query execution. If you are already using the `:prefix` option\n  you now have to pass this through `:query_opts`.\n\nIf you configured the Repo `:prefix` in the application config:\n\n```diff\nconfig :flop,\n-  prefix: \"some-prefix\"\n+  query_opts: [prefix: \"some-prefix\"]\n```\n\nIf you set the `:prefix` when calling the Flop functions:\n\n```diff\n- Flop.validate_and_run(Pet, params, prefix: \"some-prefix\")\n+ Flop.validate_and_run(Pet, params, query_opts: [prefix: \"some-prefix\"])\n```\n\n## [0.16.1] - 2022-04-05\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Wrong type spec for `Flop.Schema.default_order/1` callback.\n\n## [0.16.0] - 2022-03-22\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- You can now define a configuration module with `use Flop` to set defaults\n  instead of or in addition to the application configuration. This makes it\n  easier to work with multiple Ecto repos.\n- The new function `Flop.bindings/3` returns the necessary bindings for a\n  given Flop query. You can use it in case you want to optimize your queries by\n  only joining tables that are actually needed.\n- Added a `count_query` option to override the count query used by\n  `Flop.run/3`, `Flop.validate_and_run/3` and `Flop.validate_and_run!/3`.\n- You can get a list of allowed operators for a given Ecto type or a given\n  schema field with `Flop.Filter.allowed_operators/1` and\n  `Flop.Filter.allowed_operators/2` now.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Breaking: The `:empty` and `:not_empty` filters now require a boolean value.\n  If no value is passed, the filter is ignored, just as it is handled for all\n  other filter operators. This change was necessary to make the integration\n  with filter forms (checkboxes) easier.\n- Breaking: The default order needs to be passed as a map now when deriving\n  `Flop.Schema`. The previous implementation already converted the two separate\n  configuration keys to a map. This meant that the configuration passed when\n  deriving `Flop.Schema` had a different format from the one you had to pass\n  when overriding the default order with the `opts`.\n  With this change, the configuration format is the same everywhere. A compile\n  time exception is raised if you are still using the old format, guiding you in\n  the update.\n- It is now validated that the filter operator matches the field type.\n- The compile time validation of the options passed when deriving `Flop.Schema`\n  has been improved.\n- Allow passing page as string to `Flop.set_page/2`.\n- Allow passing offset as string to `Flop.set_offset/2`.\n\n## [0.15.0] - 2021-11-14\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `Flop.reset_filters/1` and `Flop.reset_order/1`.\n- Add `Flop.current_order/2` to retrieve the order of a given field.\n- Add `Flop.to_next_page/2` and `Flop.to_previous_page/1`.\n- Add `Flop.set_cursor/2`, `Flop.to_next_cursor/1` and\n  `Flop.to_previous_cursor/1`.\n- Add `Flop.set_offset/2`, `Flop.to_previous_offset/1`, `Flop.to_next_offset_2`\n  and `Flop.reset_cursors/2`.\n- Add `Flop.nest_filters/3` for converting filters between a key/value map and\n  a list of `Flop.Filter` parameters.\n- You can now set the `default_pagination_type` option, which forces a certain\n  set of parameters when defaults are applied and the pagination type cannot\n  be determined from the given parameters.\n- Add optional `default` argument to `get_option`.\n- Add `pagination` option. If set to `true`, pagination parameters are not cast.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- `Flop.map_to_filter_params/2` returns maps with string keys if the original\n  map has string keys now.\n- The `has_previous_page?` value of the `Flop.Meta` struct is now always `true`\n  if `first` is used with `after`. `has_next_page?` is always `true` when\n  `last` is used with `before`.\n- `push_order/2` resets the `:after` and `:before` parameters now, since the\n  cursors depend on the order.\n- `validate_and_run/3` and `validate_and_run!/3` pass all given options to\n  the validate functions now, allowing you to override defaults set in the\n  schema.\n- If the `pagination_types` option is used, parameters for other pagination\n  types will not be cast now instead of casting them and returning validation\n  errors.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Removed - Changelog","doc":"- Remove `Flop.Cursor.get_cursor_from_map/2`. Use\n  `Flop.Cursor.get_cursor_from_node/2` instead.\n\n## [0.14.0] - 2021-11-08\n\n#","ref":"changelog.html#removed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `:contains` operator.\n- Add `Flop.map_to_filter_params/2`.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- `Flop.validate/2` and `Flop.validate_and_run/3` return `{:error, Flop.Meta.t}`\n  instead of `{:error, Ecto.Changeset.t}` now. The Meta struct has the new\n  fields `:errors` and `:params`, which are set when validation errors occur.\n  This accompanies the changes in `Flop.Phoenix`, which include the\n  implementation of the `Phoenix.HTML.FormData` protocol for the `Flop.Meta`\n  struct.\n- `Flop.validate!/2` and `Flop.validate_and_run!/3` raise a\n  `Flop.InvalidParamsError` instead of an `Ecto.InvalidChangesetError` now.\n- Add `:schema` key to `Flop.Meta`. This field points to the schema module set\n  by passing the `:for` option.\n- Minimum Ecto version changed to 3.5.\n- Replace `Operator` and `OrderDirection` custom Ecto types with Ecto.Enum.\n- Update `Flop.Meta` struct default values for the fields `:flop`,\n  `:has_next_page?` and `:has_previous_page?`.\n\n## [0.13.2] - 2021-10-16\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix error when sorting by a compound field that consists of at least one\n  join field.\n- Fix import conflict when importing `Ecto.Changeset` in a module that derives\n  `Flop.Schema` and configures a compound field.\n\n## [0.13.1] - 2021-08-23\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Wrong type spec for cursor_dynamic/3 callback.\n\n## [0.13.0] - 2021-08-22\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Support ordering by join fields.\n- Support ordering by compound fields.\n- Support join fields as cursor fields.\n- New function `Flop.Schema.get_field/2`.\n- `Flop.Cursor.get_cursor_from_edge/2` and `Flop.Cursor.get_cursor_from_node/2`\n  can get cursor values from join and compound fields now.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"To get the pagination cursor value from a join field, Flop needs to know how\nto access the field value from the returned struct or map. The configuration\nformat for join fields has been changed to allow specifying the path to the\nnested field.\n\nBefore:\n\n```elixir\n@derive {\n  Flop.Schema,\n  join_fields: [\n    owner_name: {:owner, :name}\n  ]\n}\n```\n\nAfter:\n\n```elixir\n@derive {\n  Flop.Schema,\n  join_fields: [\n    owner_name: [binding: :owner, field: :name, path: [:owner, :name]]\n  ]\n}\n```\n\nThe `:path` is optional and inferred from the `:binding` and `:field` options,\nif omitted.\n\nThe old configuration format is still accepted. All of these settings are\nequivalent:\n\n```\n[owner_name: {:owner, :name}]\n\n[owner_name: [binding: :owner, field: :name]]\n\n[owner_name: [binding: :owner, field: :name, path: [:owner, :name]]]\n```\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Cursor pagination failed when one of the cursor field values was `nil`.\n\n## [0.12.0] - 2021-08-11\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Allow to define join fields in `Flop.Schema`.\n- Allow to define compound fields in `Flop.Schema`.\n- Support filtering by join fields.\n- Support filtering by compound fields.\n- New filter operator `empty`.\n- New filter operator `not_empty`.\n- New function `Flop.set_page/2`.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Rename option `get_cursor_value_func` to `cursor_value_func`.\n- Silently ignore filters with `nil` value for the field or the value instead of\n  raising an `ArgumentError`.\n- Allow passing a string as the second argument to `Flop.push_order/2`.\n\n## [0.11.0] - 2021-06-13\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- New functions `Flop.Cursor.get_cursor_from_node/2` and\n  `Flop.Cursor.get_cursor_from_edge/2`.\n- New function `Flop.get_option/2`.\n- Support Ecto prefixes.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Use `Flop.Cursor.get_cursor_from_node/2` as default for the\n  `:get_cursor_value_func` option.\n- `Flop.Relay.edges_from_result/2` can now handle `nil` instead of a map as\n  edge information in a query result.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Deprecated - Changelog","doc":"- Deprecate `Flop.Cursor.get_cursor_from_map/2`. Use\n  `Flop.Cursor.get_cursor_from_node/2` instead.\n\n## [0.10.0] - 2021-05-03\n\n#","ref":"changelog.html#deprecated"},{"type":"extras","title":"Added - Changelog","doc":"- Add function `Flop.push_order/2` for updating the `order_by` and\n  `order_directions` values of a Flop struct.\n\n## [0.9.1] - 2020-10-21\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed type spec of `Flop.Schema.default_order/1`.\n\n## [0.9.0] - 2020-10-16\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `like`, `like_and`, `like_or`, `ilike`, `ilike_and` and `ilike_or` filter\n  operators.\n- Add option to disable pagination types globally, for a schema or locally.\n- Add options to disable ordering or filtering.\n- Allow global configuration of `get_cursor_value_func`, `max_limit` and\n  `default_limit`.\n- Add `Flop.option` type, improve documentation of available options.\n- Add `Flop.Cursor.decode!/1`.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Refactored the parameter validation. Default limits are now applied to all\n  pagination types. Added validation for the `after` / `before` cursor values.\n- `Flop.Cursor.decode/1` returns `:ok` tuple or `:error` now instead of raising\n  an error if the cursor is invalid.\n- `Flop.Cursor.decode/1` returns an error if the decoded cursor value is not a\n  map with atom keys.\n- Improved documentation.\n\n## [0.8.4] - 2020-10-14\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Default limit was overriding `first` / `last` parameters when building query.\n\n## [0.8.3] - 2020-10-14\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Cursor-based pagination: `has_next_page?` was set when querying with `last`\n  based on `before` being set. Likewise, `has_previous_page?` was set when\n  querying with `first` based on `after` being set. Both assumptions are wrong.\n  In both cases, the values are always set to `false` now.\n\n## [0.8.2] - 2020-10-08\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Order directions are not restricted anymore for cursor-based pagination.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Query for cursor-based pagination returned wrong results when using more than\n  one cursor field.\n- Query for cursor-based pagination returned wrong results when using\n  `last`/`before`.\n\n## [0.8.1] - 2020-10-07\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Allow structs in cursor values.\n\n## [0.8.0] - 2020-10-07\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Support for cursor-based pagination. Thanks to @bunker-inspector.\n- Add functions to turn query results into Relay connection format when using\n  cursor-based pagination.\n\n## [0.7.1] - 2020-09-04\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Calculation of `has_next_page?` was wrong.\n\n## [0.7.0] - 2020-08-04\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- `Flop.Schema` now allows to set a default sort order.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Passing a limit without an offset will now set the offset to 0.\n- Passing a page size without a page will now set the page to 1.\n\n## [0.6.1] - 2020-06-17\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Add Flop to Meta struct.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Type `Flop.Filter.op` didn't include all operators.\n\n## [0.6.0] - 2020-06-14\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- New struct `Flop.Meta`.\n- New function `Flop.all/3`.\n- New function `Flop.count/3`.\n- New function `Flop.meta/3`.\n- New function `Flop.run/3`.\n- New function `Flop.validate_and_run/3`.\n- New function `Flop.validate_and_run!/3`.\n\n## [0.5.0] - 2020-05-28\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- New function `Flop.validate!/2`.\n- New filter operator `:in`.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Filter validation was using sortable fields instead of filterable fields.\n\n## [0.4.0] - 2020-05-27\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Added `=~` filter operator.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Query function wasn't generating valid where clauses for filters.\n\n## [0.3.0] - 2020-05-22\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Added a `default_limit` option to `Flop.Schema`.\n\n## [0.2.0] - 2020-05-20\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Added a `max_limit` option to `Flop.Schema`. When set, Flop validates that the\n  `limit` and `page_size` parameters don't exceed the configured max limit.\n\n## [0.1.0] - 2019-10-19\n\ninitial release","ref":"changelog.html#added"},{"type":"extras","title":"Schema configuration","doc":"# Schema configuration","ref":"schema.html"},{"type":"extras","title":"Basics - Schema configuration","doc":"{: .col-2}\n\n#","ref":"schema.html#basics"},{"type":"extras","title":"Minimal configuration - Schema configuration","doc":"```elixir\ndefmodule Pet do\n  use Ecto.Schema\n\n  @derive {\n    Flop.Schema,\n    filterable: [:name, :species],\n    sortable: [:name, :age]\n  }\n\n  schema \"pets\" do\n    field :name, :string\n    field :age, :integer\n    field :species, :string\n  end\nend\n```\n\n#","ref":"schema.html#minimal-configuration"},{"type":"extras","title":"Options - Schema configuration","doc":"##","ref":"schema.html#options"},{"type":"extras","title":"Limit - Schema configuration","doc":"```elixir\n@derive {\n  Flop.Schema,\n  filterable: [:name, :species],\n  sortable: [:name, :age],\n  max_limit: 100,\n  default_limit: 50\n}\n```\n\n##","ref":"schema.html#limit"},{"type":"extras","title":"Order - Schema configuration","doc":"```elixir\n@derive {\n  Flop.Schema,\n  filterable: [:name, :species],\n  sortable: [:name, :age],\n  default_order: %{\n    order_by: [:name, :age],\n    order_directions: [:asc, :desc]\n  }\n}\n```\n\n##","ref":"schema.html#order"},{"type":"extras","title":"Pagination types - Schema configuration","doc":"```elixir\n@derive {\n  Flop.Schema,\n  filterable: [:name, :species],\n  sortable: [:name, :age],\n  pagination_types: [:first, :last],\n  default_pagination_type: :first\n}\n```","ref":"schema.html#pagination-types"},{"type":"extras","title":"Alias fields - Schema configuration","doc":"{: .col-2}\n\n#","ref":"schema.html#alias-fields"},{"type":"extras","title":"Schema - Schema configuration","doc":"```elixir\ndefmodule Owner do\n  use Ecto.Schema\n\n  @derive {\n    Flop.Schema,\n    filterable: [:name],\n    sortable: [:name, :pet_count],\n    adapter_opts: [\n      alias_fields: [:pet_count]\n    ]\n  }\n\n  schema \"owners\" do\n    field :name, :string\n    has_many :pets, Pet\n  end\nend\n```\n\n#","ref":"schema.html#schema"},{"type":"extras","title":"Query - Schema configuration","doc":"```elixir\nparams = %{order_by: [:pet_count]}\n\nOwner\n|> join(:left, [o], p in assoc(o, :pets), as: :pets)\n|> group_by([o], o.id)\n|> select(\n  [o, pets: p],\n  {o.id, p.id |> count() |> selected_as(:pet_count)}\n)\n|> Flop.validate_and_run(params, for: Owner)\n```","ref":"schema.html#query"},{"type":"extras","title":"Compound fields - Schema configuration","doc":"{: .col-2}\n\n#","ref":"schema.html#compound-fields"},{"type":"extras","title":"Schema - Schema configuration","doc":"```elixir\ndefmodule User do\n  use Ecto.Schema\n\n  @derive {\n    Flop.Schema,\n    filterable: [:full_name],\n    sortable: [:full_name],\n    adapter_opts: [\n      compound_fields: [\n        full_name: [:family_name, :given_name]\n      ]\n    ]\n  }\n\n  schema \"users\" do\n    field :family_name, :string\n    field :given_name, :string\n  end\nend\n```\n\n#","ref":"schema.html#schema"},{"type":"extras","title":"Query - Schema configuration","doc":"```elixir\nparams = %{\n  filters: [\n    %{field: :full_name, op: :ilike_and, value: \"pea\"}\n  ]\n}\n\nFlop.validate_and_run(User, params, for: Owner)\n```","ref":"schema.html#query"},{"type":"extras","title":"Join fields - Schema configuration","doc":"{: .col-2}\n\n#","ref":"schema.html#join-fields"},{"type":"extras","title":"Schema - Schema configuration","doc":"##","ref":"schema.html#schema"},{"type":"extras","title":"Owner - Schema configuration","doc":"```elixir\ndefmodule Owner do\n  use Ecto.Schema\n\n  @derive {\n    Flop.Schema,\n    filterable: [:name, :pet_age],\n    sortable: [:name],\n    adapter_opts: [\n      join_fields: [\n        pet_age: [\n          binding: :pets,\n          field: :age,\n          ecto_type: :integer\n        ]\n      ]\n    ]\n  }\n\n  schema \"owners\" do\n    field :name, :string\n    has_many :pets, Pet\n  end\nend\n```\n\n##","ref":"schema.html#owner"},{"type":"extras","title":"Pet - Schema configuration","doc":"```elixir\ndefmodule Pet do\n  use Ecto.Schema\n\n  schema \"pets\" do\n    field :age, :integer\n  end\nend\n```\n\n#","ref":"schema.html#pet"},{"type":"extras","title":"Query - Schema configuration","doc":"##","ref":"schema.html#query"},{"type":"extras","title":"Only filtering or sorting - Schema configuration","doc":"```elixir\nparams = %{\n  filters: [\n    %{field: :pet_age, op: :==, value: 8}\n  ]\n}\n\nOwner\n|> join([o], p in assoc(o, :pets), as: :pets)\n|> Flop.validate_and_run(params, for: Pet)\n```\n\n##","ref":"schema.html#only-filtering-or-sorting"},{"type":"extras","title":"With preload - Schema configuration","doc":"```elixir\nOwner\n|> join([o], p in assoc(o, :pets), as: :pets)\n|> preload([pets: p], pets: p)\n|> Flop.validate_and_run(params, for: Pet)\n```","ref":"schema.html#with-preload"},{"type":"extras","title":"Join field for nested association - Schema configuration","doc":"{: .col-2}\n\n#","ref":"schema.html#join-field-for-nested-association"},{"type":"extras","title":"Schema - Schema configuration","doc":"##","ref":"schema.html#schema"},{"type":"extras","title":"Owner - Schema configuration","doc":"```elixir\ndefmodule Owner do\n  use Ecto.Schema\n\n  @derive {\n    Flop.Schema,\n    filterable: [:name, :toy_description],\n    sortable: [:name],\n    adapter_opts: [\n      join_fields: [\n        pet_age: [\n          binding: :toys,\n          field: :description,\n          ecto_type: :string,\n          # only needed with cursor pagination when sorting\n          # by the join field, so that Flop can find the\n          # cursor value\n          path: [:pets, :toys]\n        ]\n      ]\n    ]\n  }\n\n  schema \"owners\" do\n    field :name, :string\n    has_many :pets, Pet\n  end\nend\n```\n\n##","ref":"schema.html#owner"},{"type":"extras","title":"Pet - Schema configuration","doc":"```elixir\ndefmodule Pet do\n  use Ecto.Schema\n\n  schema \"pets\" do\n    field :age, :integer\n    has_many :toys, Toy\n  end\nend\n```\n\n##","ref":"schema.html#pet"},{"type":"extras","title":"Toy - Schema configuration","doc":"```elixir\ndefmodule Toy do\n  use Ecto.Schema\n\n  schema \"toys\" do\n    field :description, :string\n  end\nend\n```\n\n#","ref":"schema.html#toy"},{"type":"extras","title":"Query with preload - Schema configuration","doc":"```elixir\nparams = %{order_by: [:toy_description]}\n\nOwner\n|> join([o], p in assoc(o, :pets), as: :pets)\n|> join([pets: p], t in assoc(p, :toys), as: :toys)\n|> preload([pets: p, toys: t], pets: {p, toys: t})\n|> Flop.validate_and_run(params, for: Owner)\n```","ref":"schema.html#query-with-preload"},{"type":"extras","title":"Join field for subquery - Schema configuration","doc":"{: .col-2}\n\n#","ref":"schema.html#join-field-for-subquery"},{"type":"extras","title":"Schema - Schema configuration","doc":"##","ref":"schema.html#schema"},{"type":"extras","title":"Owner - Schema configuration","doc":"```elixir\ndefmodule Owner do\n  use Ecto.Schema\n\n  @derive {\n    Flop.Schema,\n    filterable: [:name],\n    sortable: [:name, :pet_count],\n    adapter_opts: [\n      join_fields: [\n        pet_count: [\n          binding: :pet_count,\n          field: :count\n        ]\n      ]\n    ]\n  }\n\n  schema \"owners\" do\n    field :name, :string\n    has_many :pets, Pet\n  end\nend\n```\n\n##","ref":"schema.html#owner"},{"type":"extras","title":"Pet - Schema configuration","doc":"```elixir\ndefmodule Pet do\n  use Ecto.Schema\n\n  schema \"pets\" do\n    field :age, :integer\n  end\nend\n```\n\n#","ref":"schema.html#pet"},{"type":"extras","title":"Query - Schema configuration","doc":"```elixir\nparams = %{filters: [%{field: :pet_count, op: :>, value: 2}]}\n\npet_count_query =\n  Pet\n  |> where([p], parent_as(:owner).id == p.owner_id)\n  |> select([p], %{count: count(p)})\n\nq =\n  Owner\n  |> from(as: :owner)\n  |> join(:inner_lateral, [o], p in subquery(pet_count_query),\n    as: :pet_count\n  )\n  |> Flop.validate_and_run(params, for: Owner)\n```","ref":"schema.html#query"},{"type":"extras","title":"Custom fields - Schema configuration","doc":"{: .col-2}\n\n#","ref":"schema.html#custom-fields"},{"type":"extras","title":"Schema - Schema configuration","doc":"```elixir\ndefmodule Pet do\n  use Ecto.Schema\n\n  @derive {\n    Flop.Schema,\n    filterable: [:name, :human_age],\n    sortable: [:name],\n    adapter_opts: [\n      custom_fields: [\n        human_age: [\n          filter: {CustomFilters, :human_age, []},\n          ecto_type: :integer\n        ]\n      ]\n    ]\n  }\n\n  schema \"pets\" do\n    field :name, :string\n    field :age, :integer\n  end\nend\n```\n\n#","ref":"schema.html#schema"},{"type":"extras","title":"Custom filter function - Schema configuration","doc":"```elixir\ndefmodule CustomFilters do\n  import Ecto.Query\n\n  def human_age(q, %Flop.Filter{value: value, op: op}, _) do\n    case Ecto.Type.cast(:integer, value) do\n      {:ok, human_years} ->\n        value_in_dog_years = round(human_years / 7)\n\n        case op do\n          :== -> where(q, [p], p == ^value_in_dog_years)\n          :!= -> where(q, [p], p != ^value_in_dog_years)\n          :> -> where(q, [p], p > ^value_in_dog_years)\n          :  where(q, [p], p  = -> where(q, [p], p >= ^value_in_dog_years)\n          :  where(q, [p], p  \n        # cannot cast filter value, ignore\n        q\n    end\n  end\nend\n```\n\n#","ref":"schema.html#custom-filter-function"},{"type":"extras","title":"Query - Schema configuration","doc":"```elixir\nparams = %{\n  filters: [\n    %{field: :human_age, op: :==, value: 30}\n  ]\n}\n\nFlop.validate_and_run(Pet, params, for: Pet)\n```","ref":"schema.html#query"}],"content_type":"text/markdown"}